@@ -111,7 +111,10 @@ bool ctdb_set_helper(const char *type, char *helper, size_t size,
 			      ("Unable to set %s - dir is NULL\n", type));
 		return false;
 	} else {
-		if (snprintf(helper, size, "%s/%s", dir, file) >= size) {
+		int ret;
+
+		ret = snprintf(helper, size, "%s/%s", dir, file);
+		if (ret < 0 || (size_t)ret >= size) {
 			DEBUG(DEBUG_ERR,
 			      ("Unable to set %s - path too long\n", type));
 			return false;
@@ -385,6 +388,13 @@ void ctdb_canonicalize_ip(const ctdb_sock_addr *ip, ctdb_sock_addr *cip)
 	}
 }
 
+void ctdb_canonicalize_ip_inplace(ctdb_sock_addr *ip)
+{
+	ctdb_sock_addr tmp;
+	ctdb_canonicalize_ip(ip, &tmp);
+	memcpy(ip, &tmp, sizeof(tmp));
+}
+
 bool ctdb_same_ip(const ctdb_sock_addr *tip1, const ctdb_sock_addr *tip2)
 {
 	ctdb_sock_addr ip1, ip2;
