@@ -114,8 +114,6 @@ static void ctdb_opcode_print(uint32_t opcode, FILE *fp)
 		{ CTDB_CONTROL_GET_DBMAP, "GET_DBMAP" },
 		{ CTDB_CONTROL_GET_NODEMAPv4, "GET_NODEMAPv4" },
 		{ CTDB_CONTROL_SET_DMASTER, "SET_DMASTER" },
-		{ CTDB_CONTROL_PULL_DB, "PULL_DB" },
-		{ CTDB_CONTROL_PUSH_DB, "PUSH_DB" },
 		{ CTDB_CONTROL_GET_RECMODE, "GET_RECMODE" },
 		{ CTDB_CONTROL_SET_RECMODE, "SET_RECMODE" },
 		{ CTDB_CONTROL_STATISTICS_RESET, "STATISTICS_RESET" },
@@ -245,6 +243,10 @@ static void ctdb_opcode_print(uint32_t opcode, FILE *fp)
 		{ CTDB_CONTROL_VACUUM_FETCH, "VACUUM_FETCH" },
 		{ CTDB_CONTROL_DB_VACUUM, "DB_VACUUM" },
 		{ CTDB_CONTROL_ECHO_DATA, "ECHO_DATA" },
+		{ CTDB_CONTROL_DISABLE_NODE, "DISABLE_NODE" },
+		{ CTDB_CONTROL_ENABLE_NODE, "ENABLE_NODE" },
+		{ CTDB_CONTROL_TCP_CLIENT_DISCONNECTED, "TCP_CLIENT_DISCONNECTED" },
+		{ CTDB_CONTROL_TCP_CLIENT_PASSED, "TCP_CLIENT_PASSED" },
 		{ MAP_END, "" },
 	};
 
