@@ -157,22 +157,11 @@ static DIR *skel_fdopendir(vfs_handle_struct *handle, files_struct *fsp,
 
 static struct dirent *skel_readdir(vfs_handle_struct *handle,
 				   struct files_struct *dirfsp,
-				   DIR *dirp,
-				   SMB_STRUCT_STAT *sbuf)
+				   DIR *dirp)
 {
 	return NULL;
 }
 
-static void skel_seekdir(vfs_handle_struct *handle, DIR *dirp, long offset)
-{
-	;
-}
-
-static long skel_telldir(vfs_handle_struct *handle, DIR *dirp)
-{
-	return (long)-1;
-}
-
 static void skel_rewind_dir(vfs_handle_struct *handle, DIR *dirp)
 {
 	;
@@ -665,8 +654,10 @@ static NTSTATUS skel_get_real_filename_at(struct vfs_handle_struct *handle,
 	return NT_STATUS_NOT_IMPLEMENTED;
 }
 
-static const char *skel_connectpath(struct vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname)
+static const char *skel_connectpath(
+	struct vfs_handle_struct *handle,
+	const struct files_struct *dirfsp,
+	const struct smb_filename *smb_fname)
 {
 	errno = ENOSYS;
 	return NULL;
@@ -991,8 +982,6 @@ static struct vfs_fn_pointers skel_opaque_fns = {
 
 	.fdopendir_fn = skel_fdopendir,
 	.readdir_fn = skel_readdir,
-	.seekdir_fn = skel_seekdir,
-	.telldir_fn = skel_telldir,
 	.rewind_dir_fn = skel_rewind_dir,
 	.mkdirat_fn = skel_mkdirat,
 	.closedir_fn = skel_closedir,
