@@ -134,6 +134,21 @@ typedef struct {
 #define KRB5_ERROR_CODE(k)	((k)->error)
 #endif /* HAVE_E_DATA_POINTER_IN_KRB5_ERROR */
 
+#ifndef HAVE_KRB5_CONST_PAC
+#ifdef KRB5_CONST_PAC_GET_BUFFER
+typedef const struct krb5_pac_data *krb5_const_pac;
+#else
+/*
+ * Certain Heimdal versions include a version of krb5_pac_get_buffer() that is
+ * unusable in certain cases, taking a krb5_pac when a krb5_const_pac may be all
+ * that we can supply. Furthermore, MIT Kerberos doesn't declare krb5_const_pac
+ * at all. In such cases, we must declare krb5_const_pac as a non-const typedef
+ * so that the build can succeed.
+ */
+typedef struct krb5_pac_data *krb5_const_pac;
+#endif
+#endif
+
 krb5_error_code smb_krb5_parse_name(krb5_context context,
 				const char *name, /* in unix charset */
                                 krb5_principal *principal);
@@ -155,6 +170,14 @@ krb5_error_code krb5_auth_con_setuseruserkey(krb5_context context, krb5_auth_con
 void krb5_free_unparsed_name(krb5_context ctx, char *val);
 #endif
 
+#if !defined(HAVE_KRB5_FREE_ENCTYPES)
+void krb5_free_enctypes(krb5_context context, krb5_enctype *val);
+#endif
+
+#if !defined(HAVE_KRB5_FREE_STRING)
+void krb5_free_string(krb5_context context, char *val);
+#endif
+
 /* Stub out initialize_krb5_error_table since it is not present in all
  * Kerberos implementations. If it's not present, it's not necessary to
  * call it.
@@ -209,12 +232,13 @@ krb5_error_code smb_krb5_kt_get_name(TALLOC_CTX *mem_ctx,
 				     const char **keytab_name);
 krb5_error_code smb_krb5_kt_seek_and_delete_old_entries(krb5_context context,
 							krb5_keytab keytab,
+							bool keep_old_kvno,
 							krb5_kvno kvno,
+							bool enctype_only,
 							krb5_enctype enctype,
 							const char *princ_s,
 							krb5_principal princ,
-							bool flush,
-							bool keep_old_entries);
+							bool flush);
 krb5_error_code smb_krb5_kt_add_entry(krb5_context context,
 				      krb5_keytab keytab,
 				      krb5_kvno kvno,
@@ -222,8 +246,7 @@ krb5_error_code smb_krb5_kt_add_entry(krb5_context context,
 				      const char *salt_principal,
 				      krb5_enctype enctype,
 				      krb5_data *password,
-				      bool no_salt,
-				      bool keep_old_entries);
+				      bool no_salt);
 
 krb5_error_code smb_krb5_get_credentials(krb5_context context,
 					 krb5_ccache ccache,
@@ -288,7 +311,7 @@ krb5_error_code smb_krb5_cc_get_lifetime(krb5_context context,
 #elif defined (HAVE_FREE_CHECKSUM)
 void smb_krb5_free_checksum_contents(krb5_context ctx, krb5_checksum *cksum);
 #else
-#error krb5_free_checksum_contents/free_Checksum is not vailable
+#error krb5_free_checksum_contents/free_Checksum is not available
 #endif
 
 krb5_error_code smb_krb5_make_pac_checksum(TALLOC_CTX *mem_ctx,
@@ -383,6 +406,11 @@ krb5_error_code smb_krb5_copy_data_contents(krb5_data *p,
 					    const void *data,
 					    size_t len);
 
+krb5_data smb_krb5_make_data(void *data,
+			     size_t len);
+
+krb5_data smb_krb5_data_from_blob(DATA_BLOB blob);
+
 int smb_krb5_principal_get_type(krb5_context context,
 				krb5_const_principal principal);
 
