@@ -22,15 +22,21 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "includes.h"
+#include "replace.h"
 #include "system/locale.h"
+#include "charset.h"
+#include "lib/util/fault.h"
 
 #ifdef strcasecmp
 #undef strcasecmp
 #endif
+#ifdef strncasecmp
+#undef strncasecmp
+#endif
+
 
 /**
- Case insensitive string compararison, handle specified for testing
+ Case insensitive string comparison, handle specified for testing
 **/
 _PUBLIC_ int strcasecmp_m_handle(struct smb_iconv_handle *iconv_handle,
 				 const char *s1, const char *s2)
@@ -80,7 +86,7 @@ _PUBLIC_ int strcasecmp_m_handle(struct smb_iconv_handle *iconv_handle,
 }
 
 /**
- Case insensitive string compararison
+ Case insensitive string comparison
 **/
 _PUBLIC_ int strcasecmp_m(const char *s1, const char *s2)
 {
@@ -89,7 +95,8 @@ _PUBLIC_ int strcasecmp_m(const char *s1, const char *s2)
 }
 
 /**
- Case insensitive string compararison, length limited, handle specified for testing
+ Case insensitive string comparison, length limited, handle specified for
+ testing
 **/
 _PUBLIC_ int strncasecmp_m_handle(struct smb_iconv_handle *iconv_handle,
 				  const char *s1, const char *s2, size_t n)
@@ -160,7 +167,7 @@ _PUBLIC_ int strncasecmp_m_handle(struct smb_iconv_handle *iconv_handle,
 }
 
 /**
- Case insensitive string compararison, length limited
+ Case insensitive string comparison, length limited
 **/
 _PUBLIC_ int strncasecmp_m(const char *s1, const char *s2, size_t n)
 {
@@ -524,15 +531,14 @@ _PUBLIC_ bool strhasupper(const char *string)
 
 char *strstr_m(const char *src, const char *findstr)
 {
+	TALLOC_CTX *mem_ctx = NULL;
 	smb_ucs2_t *p;
 	smb_ucs2_t *src_w, *find_w;
 	const char *s;
 	char *s2;
-	char *retp;
+	char *retp = NULL;
 	size_t converted_size, findstr_len = 0;
 
-	TALLOC_CTX *frame; /* Only set up in the iconv case */
-
 	/* for correctness */
 	if (!findstr[0]) {
 		return discard_const_p(char, src);
@@ -567,31 +573,36 @@ char *strstr_m(const char *src, const char *findstr)
 	s = src;
 #endif
 
-	frame = talloc_stackframe();
-
-	if (!push_ucs2_talloc(frame, &src_w, src, &converted_size)) {
-		TALLOC_FREE(frame);
+	/*
+	 * Use get_iconv_handle() just as a non-NULL talloc ctx. In
+	 * case we leak memory, this should then be more obvious in
+	 * the talloc report.
+	 */
+	mem_ctx = talloc_new(get_iconv_handle());
+	if (mem_ctx == NULL) {
 		return NULL;
 	}
 
-	if (!push_ucs2_talloc(frame, &find_w, findstr, &converted_size)) {
-		TALLOC_FREE(frame);
-		return NULL;
+	if (!push_ucs2_talloc(mem_ctx, &src_w, src, &converted_size)) {
+		goto done;
+	}
+
+	if (!push_ucs2_talloc(mem_ctx, &find_w, findstr, &converted_size)) {
+		goto done;
 	}
 
 	p = strstr_w(src_w, find_w);
 
 	if (!p) {
-		TALLOC_FREE(frame);
-		return NULL;
+		goto done;
 	}
 
 	*p = 0;
-	if (!pull_ucs2_talloc(frame, &s2, src_w, &converted_size)) {
-		TALLOC_FREE(frame);
-		return NULL;
+	if (!pull_ucs2_talloc(mem_ctx, &s2, src_w, &converted_size)) {
+		goto done;
 	}
 	retp = discard_const_p(char, (s+strlen(s2)));
-	TALLOC_FREE(frame);
+done:
+	TALLOC_FREE(mem_ctx);
 	return retp;
 }
