@@ -112,7 +112,7 @@ static int tstream_smbXcli_np_destructor(struct tstream_smbXcli_np *cli_nps)
 	 * tstream_disconnect_send()/_recv(), this will
 	 * never be called.
 	 *
-	 * We use a maximun timeout of 1 second == 1000 msec.
+	 * We use a maximum timeout of 1 second == 1000 msec.
 	 */
 	cli_nps->timeout = MIN(cli_nps->timeout, 1000);
 
@@ -280,10 +280,14 @@ static void tstream_smbXcli_np_open_done(struct tevent_req *subreq)
 	if (state->is_smb1) {
 		status = smb1cli_ntcreatex_recv(subreq, &state->fnum);
 	} else {
-		status = smb2cli_create_recv(subreq,
-					     &state->fid_persistent,
-					     &state->fid_volatile,
-					     NULL, NULL, NULL);
+		status = smb2cli_create_recv(
+			subreq,
+			&state->fid_persistent,
+			&state->fid_volatile,
+			NULL,
+			NULL,
+			NULL,
+			NULL);
 	}
 	TALLOC_FREE(subreq);
 	if (!NT_STATUS_IS_OK(status)) {
