@@ -1,20 +1,20 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    routines for marshalling/unmarshalling string types
 
    Copyright (C) Andrew Tridgell 2003
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -120,7 +120,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_string(struct ndr_pull *ndr, int ndr_flags,
 	case LIBNDR_FLAG_STR_NULLTERM:
 		/*
 		 * We ensure that conv_str_len cannot return 0 by
-		 * requring that there be enough bytes for at least
+		 * requiring that there be enough bytes for at least
 		 * the NULL terminator
 		 */
 		if (byte_mul == 1) {
@@ -209,7 +209,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_string(struct ndr_push *ndr, int ndr_flags,
 	if (NDR_BE(ndr)) {
 		chset = CH_UTF16BE;
 	}
-	
+
 	s_len = s?strlen(s):0;
 
 	if (flags & LIBNDR_FLAG_STR_ASCII) {
@@ -236,13 +236,16 @@ _PUBLIC_ enum ndr_err_code ndr_push_string(struct ndr_push *ndr, int ndr_flags,
 		s_len++;
 	}
 
-	if (!do_convert) {
+	if (s_len == 0) {
+		d_len = 0;
+		dest = (uint8_t *)talloc_strdup(ndr, "");
+	} else if (!do_convert) {
 		d_len = s_len;
 		dest = (uint8_t *)talloc_strndup(ndr, s, s_len);
 	} else if (!convert_string_talloc(ndr, CH_UNIX, chset, s, s_len,
 				   (void **)(void *)&dest, &d_len))
 	{
-		return ndr_push_error(ndr, NDR_ERR_CHARCNV, 
+		return ndr_push_error(ndr, NDR_ERR_CHARCNV,
 				      "Bad character push conversion with flags 0x%x", flags);
 	}
 
@@ -287,7 +290,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_string(struct ndr_push *ndr, int ndr_flags,
 	default:
 		if (ndr->flags & LIBNDR_FLAG_REMAINING) {
 			NDR_CHECK(ndr_push_bytes(ndr, dest, d_len));
-			break;		
+			break;
 		}
 
 		return ndr_push_error(ndr, NDR_ERR_STRING, "Bad string flags 0x%x\n",
@@ -334,6 +337,10 @@ _PUBLIC_ size_t ndr_string_array_size(struct ndr_push *ndr, const char *s)
 
 _PUBLIC_ void ndr_print_string(struct ndr_print *ndr, const char *name, const char *s)
 {
+	if (NDR_HIDE_SECRET(ndr)) {
+		ndr->print(ndr, "%-25s: <REDACTED SECRET VALUE>", name);
+		return;
+	}
 	if (s) {
 		ndr->print(ndr, "%-25s: '%s'", name, s);
 	} else {
@@ -341,13 +348,51 @@ _PUBLIC_ void ndr_print_string(struct ndr_print *ndr, const char *name, const ch
 	}
 }
 
-_PUBLIC_ uint32_t ndr_size_string(int ret, const char * const* string, int flags) 
+_PUBLIC_ uint32_t ndr_size_string(int ret, const char * const* string, int flags)
 {
 	/* FIXME: Is this correct for all strings ? */
 	if(!(*string)) return ret;
 	return ret+strlen(*string)+1;
 }
 
+static uint32_t guess_string_array_size(struct ndr_pull *ndr, int ndr_flags)
+{
+	/*
+	 * Here we could do something clever like count the number of zeros in
+	 * the ndr data, but it is probably sufficient to pick a lowish number
+	 * (compared to the overhead of the talloc header) and let the
+	 * expontential resizing deal with longer arrays.
+	 */
+	return 5;
+}
+
+static enum ndr_err_code extend_string_array(struct ndr_pull *ndr,
+					     const char ***_a,
+					     uint32_t *count)
+{
+	const char **a = *_a;
+	uint32_t inc = *count / 4 + 3;
+	uint32_t alloc_size = *count + inc;
+
+	if (alloc_size < *count) {
+		/* overflow ! */
+		return NDR_ERR_ALLOC;
+	}
+	/*
+	 * We allocate and zero two more bytes than we report back, so that
+	 * the string array will always be NULL terminated.
+	 */
+	a = talloc_realloc(ndr->current_mem_ctx, a,
+			   const char *,
+			   alloc_size);
+	NDR_ERR_HAVE_NO_MEMORY(a);
+
+	memset(a + *count, 0, inc * sizeof(a[0]));
+	*_a = a;
+	*count = alloc_size - 2;
+	return NDR_ERR_SUCCESS;
+}
+
 /**
   pull a general string array from the wire
 */
@@ -357,14 +402,19 @@ _PUBLIC_ enum ndr_err_code ndr_pull_string_array(struct ndr_pull *ndr, int ndr_f
 	uint32_t count;
 	unsigned flags = ndr->flags;
 	unsigned saved_flags = ndr->flags;
+	uint32_t alloc_size;
 
 	if (!(ndr_flags & NDR_SCALARS)) {
 		return NDR_ERR_SUCCESS;
 	}
 
+	alloc_size = guess_string_array_size(ndr, ndr_flags);
+	a = talloc_zero_array(ndr->current_mem_ctx, const char *, alloc_size + 2);
+	NDR_ERR_HAVE_NO_MEMORY(a);
+
 	switch (flags & (LIBNDR_FLAG_STR_NULLTERM|LIBNDR_FLAG_STR_NOTERM)) {
 	case LIBNDR_FLAG_STR_NULLTERM:
-		/* 
+		/*
 		 * here the strings are null terminated
 		 * but also the array is null terminated if LIBNDR_FLAG_REMAINING
 		 * is specified
@@ -372,20 +422,21 @@ _PUBLIC_ enum ndr_err_code ndr_pull_string_array(struct ndr_pull *ndr, int ndr_f
 		for (count = 0;; count++) {
 			TALLOC_CTX *tmp_ctx;
 			const char *s = NULL;
-			a = talloc_realloc(ndr->current_mem_ctx, a, const char *, count + 2);
-			NDR_ERR_HAVE_NO_MEMORY(a);
-			a[count]   = NULL;
-			a[count+1]   = NULL;
+			if (count == alloc_size) {
+				NDR_CHECK(extend_string_array(ndr,
+							      &a,
+							      &alloc_size));
+			}
 
 			tmp_ctx = ndr->current_mem_ctx;
 			ndr->current_mem_ctx = a;
 			NDR_CHECK(ndr_pull_string(ndr, ndr_flags, &s));
+			ndr->current_mem_ctx = tmp_ctx;
 			if ((ndr->data_size - ndr->offset) == 0 && ndr->flags & LIBNDR_FLAG_REMAINING)
 			{
 				a[count] = s;
 				break;
 			}
-			ndr->current_mem_ctx = tmp_ctx;
 			if (strcmp("", s)==0) {
 				a[count] = NULL;
 				break;
@@ -404,10 +455,10 @@ _PUBLIC_ enum ndr_err_code ndr_pull_string_array(struct ndr_pull *ndr, int ndr_f
 		}
 		/*
 		 * here the strings are not null terminated
-		 * but serarated by a null terminator
+		 * but separated by a null terminator
 		 *
 		 * which means the same as:
-		 * Every string is null terminated exept the last
+		 * Every string is null terminated except the last
 		 * string is terminated by the end of the buffer
 		 *
 		 * as LIBNDR_FLAG_STR_NULLTERM also end at the end
@@ -422,10 +473,11 @@ _PUBLIC_ enum ndr_err_code ndr_pull_string_array(struct ndr_pull *ndr, int ndr_f
 		for (count = 0; ((ndr->data_size - ndr->offset) > 0); count++) {
 			TALLOC_CTX *tmp_ctx;
 			const char *s = NULL;
-			a = talloc_realloc(ndr->current_mem_ctx, a, const char *, count + 2);
-			NDR_ERR_HAVE_NO_MEMORY(a);
-			a[count]   = NULL;
-			a[count+1]   = NULL;
+			if (count == alloc_size) {
+				NDR_CHECK(extend_string_array(ndr,
+							      &a,
+							      &alloc_size));
+			}
 
 			tmp_ctx = ndr->current_mem_ctx;
 			ndr->current_mem_ctx = a;
@@ -434,7 +486,9 @@ _PUBLIC_ enum ndr_err_code ndr_pull_string_array(struct ndr_pull *ndr, int ndr_f
 			a[count] = s;
 		}
 
-		*_a =a;
+		a = talloc_realloc(ndr->current_mem_ctx, a, const char *, count + 1);
+		NDR_ERR_HAVE_NO_MEMORY(a);
+		*_a = a;
 		break;
 
 	default:
@@ -493,7 +547,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_string_array(struct ndr_push *ndr, int ndr_f
 		return ndr_push_error(ndr, NDR_ERR_STRING, "Bad string flags 0x%x\n",
 				      ndr->flags & LIBNDR_STRING_FLAGS);
 	}
-	
+
 	ndr->flags = saved_flags;
 	return NDR_ERR_SUCCESS;
 }
@@ -547,7 +601,7 @@ _PUBLIC_ size_t ndr_size_string_array(const char **a, uint32_t count, int flags)
 }
 
 /**
- * Return number of elements in a string including the last (zeroed) element 
+ * Return number of elements in a string including the last (zeroed) element
  */
 _PUBLIC_ uint32_t ndr_string_length(const void *_var, uint32_t element_size)
 {
@@ -646,7 +700,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_charset(struct ndr_pull *ndr, int ndr_flags,
 				   discard_const_p(void *, var),
 				   &converted_size))
 	{
-		return ndr_pull_error(ndr, NDR_ERR_CHARCNV, 
+		return ndr_pull_error(ndr, NDR_ERR_CHARCNV,
 				      "Bad character conversion");
 	}
 	NDR_CHECK(ndr_pull_advance(ndr, length*byte_mul));
@@ -701,7 +755,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_charset(struct ndr_push *ndr, int ndr_flags,
 		return ndr_push_error(ndr, NDR_ERR_LENGTH, "length overflow");
 	}
 	required = byte_mul * length;
-	
+
 	NDR_PUSH_NEED_BYTES(ndr, required);
 
 	if (required) {
@@ -714,7 +768,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_charset(struct ndr_push *ndr, int ndr_flags,
 		if (!convert_string(CH_UNIX, chset,
 			     var, strlen(var),
 			     ndr->data+ndr->offset, required, &size)) {
-			return ndr_push_error(ndr, NDR_ERR_CHARCNV, 
+			return ndr_push_error(ndr, NDR_ERR_CHARCNV,
 				      "Bad character conversion");
 		}
 
