@@ -25,6 +25,8 @@ from samba import credentials
 import samba.tests
 import os
 import binascii
+from samba.dcerpc import misc
+
 
 class CredentialsTests(samba.tests.TestCaseInTempDir):
 
@@ -107,17 +109,43 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         self.creds.set_workstation("myworksta")
         self.assertEqual("myworksta", self.creds.get_workstation())
 
+    def test_secure_channel_type(self):
+        self.assertEqual(misc.SEC_CHAN_NULL,
+                         self.creds.get_secure_channel_type())
+        self.creds.set_secure_channel_type(misc.SEC_CHAN_BDC)
+        self.assertEqual(misc.SEC_CHAN_BDC,
+                         self.creds.get_secure_channel_type())
+
     def test_get_nt_hash(self):
-        password="geheim"
-        hex_nthash="c2ae1fe6e648846352453e816f2aeb93"
+        password = "geheim"
+        hex_nthash = "c2ae1fe6e648846352453e816f2aeb93"
         self.creds.set_password(password)
         self.assertEqual(password, self.creds.get_password())
         self.assertEqual(binascii.a2b_hex(hex_nthash),
                          self.creds.get_nt_hash())
 
+    def test_get_ntlm_response(self):
+        password = "SecREt01"
+        hex_challenge = "0123456789abcdef"
+        hex_nthash = "cd06ca7c7e10c99b1d33b7485a2ed808"
+        hex_session_key = "3f373ea8e4af954f14faa506f8eebdc4"
+        hex_ntlm_response = "25a98c1c31e81847466b29b2df4680f39958fb8c213a9cc6"
+        self.creds.set_username("fred")
+        self.creds.set_domain("nurk")
+        self.creds.set_password(password)
+        self.assertEqual(password, self.creds.get_password())
+        self.assertEqual(binascii.a2b_hex(hex_nthash),
+                         self.creds.get_nt_hash())
+        response = self.creds.get_ntlm_response(flags=credentials.CLI_CRED_NTLM_AUTH,
+                                                challenge=binascii.a2b_hex(hex_challenge))
+
+        self.assertEqual(response["nt_response"], binascii.a2b_hex(hex_ntlm_response))
+        self.assertEqual(response["nt_session_key"], binascii.a2b_hex(hex_session_key))
+        self.assertEqual(response["flags"], credentials.CLI_CRED_NTLM_AUTH)
+
     def test_get_nt_hash_string(self):
         self.creds.set_password_will_be_nt_hash(True)
-        hex_nthash="c2ae1fe6e648846352453e816f2aeb93"
+        hex_nthash = "c2ae1fe6e648846352453e816f2aeb93"
         self.creds.set_password(hex_nthash)
         self.assertEqual(None, self.creds.get_password())
         self.assertEqual(binascii.a2b_hex(hex_nthash),
@@ -161,13 +189,13 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         self.assertEqual(creds.authentication_requested(), False)
 
     def test_parse_file_1(self):
-        realm="realm.example.com"
-        domain="dom"
-        password="pass"
-        username="user"
+        realm = "realm.example.com"
+        domain = "dom"
+        password = "pass"
+        username = "user"
 
         passwd_file_name = os.path.join(self.tempdir, "parse_file")
-        passwd_file_fd = open(passwd_file_name, 'wx')
+        passwd_file_fd = open(passwd_file_name, 'x')
         passwd_file_fd.write("realm=%s\n" % realm)
         passwd_file_fd.write("domain=%s\n" % domain)
         passwd_file_fd.write("username=%s\n" % username)
@@ -184,13 +212,13 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         os.unlink(passwd_file_name)
 
     def test_parse_file_2(self):
-        realm="realm.example.com"
-        domain="dom"
-        password="pass"
-        username="user"
+        realm = "realm.example.com"
+        domain = "dom"
+        password = "pass"
+        username = "user"
 
         passwd_file_name = os.path.join(self.tempdir, "parse_file")
-        passwd_file_fd = open(passwd_file_name, 'wx')
+        passwd_file_fd = open(passwd_file_name, 'x')
         passwd_file_fd.write("realm=%s\n" % realm)
         passwd_file_fd.write("domain=%s\n" % domain)
         passwd_file_fd.write("username=%s\\%s\n" % (domain, username))
@@ -207,15 +235,15 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         os.unlink(passwd_file_name)
 
     def test_parse_file_3(self):
-        realm="realm.example.com"
-        domain="domain"
-        password="password"
-        username="username"
+        realm = "realm.example.com"
+        domain = "domain"
+        password = "password"
+        username = "username"
 
-        userdom="userdom"
+        userdom = "userdom"
 
         passwd_file_name = os.path.join(self.tempdir, "parse_file")
-        passwd_file_fd = open(passwd_file_name, 'wx')
+        passwd_file_fd = open(passwd_file_name, 'x')
         passwd_file_fd.write("realm=%s\n" % realm)
         passwd_file_fd.write("domain=%s\n" % domain)
         passwd_file_fd.write("username=%s/%s\n" % (userdom, username))
@@ -232,15 +260,13 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         os.unlink(passwd_file_name)
 
     def test_parse_file_4(self):
-        realm="realm.example.com"
-        domain="domain"
-        password="password"
-        username="username"
+        password = "password"
+        username = "username"
 
-        userdom="userdom"
+        userdom = "userdom"
 
         passwd_file_name = os.path.join(self.tempdir, "parse_file")
-        passwd_file_fd = open(passwd_file_name, 'wx')
+        passwd_file_fd = open(passwd_file_name, 'x')
         passwd_file_fd.write("username=%s\\%s%%%s\n" % (userdom, username, password))
         passwd_file_fd.write("realm=ignorerealm\n")
         passwd_file_fd.write("domain=ignoredomain\n")
@@ -257,15 +283,13 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         os.unlink(passwd_file_name)
 
     def test_parse_file_5(self):
-        realm="realm.example.com"
-        domain="domain"
-        password="password"
-        username="username"
+        password = "password"
+        username = "username"
 
-        userdom="userdom"
+        userdom = "userdom"
 
         passwd_file_name = os.path.join(self.tempdir, "parse_file")
-        passwd_file_fd = open(passwd_file_name, 'wx')
+        passwd_file_fd = open(passwd_file_name, 'x')
         passwd_file_fd.write("realm=ignorerealm\n")
         passwd_file_fd.write("username=%s\\%s%%%s\n" % (userdom, username, password))
         passwd_file_fd.write("domain=ignoredomain\n")
@@ -300,7 +324,7 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         os.environ["USER"] = "env_user"
         creds.guess(lp)
         realm = "realm.example.com"
-        creds.set_realm(realm, credentials.UNINITIALISED)
+        creds.set_realm(realm, credentials.SMB_CONF)
         creds.parse_string("user")
         self.assertEqual(creds.get_username(), "user")
         self.assertEqual(creds.get_domain(), lp.get("workgroup").upper())
@@ -314,7 +338,7 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         lp = samba.tests.env_loadparm()
         os.environ["USER"] = "env_user"
         creds.guess(lp)
-        creds.parse_string("domain\user")
+        creds.parse_string("domain\\user")
         self.assertEqual(creds.get_username(), "user")
         self.assertEqual(creds.get_domain(), "DOMAIN")
         self.assertEqual(creds.get_realm(), None)
@@ -328,7 +352,7 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         os.environ["USER"] = "env_user"
         creds.guess(lp)
         realm = "realm.example.com"
-        creds.set_realm(realm, credentials.UNINITIALISED)
+        creds.set_realm(realm, credentials.SMB_CONF)
         self.assertEqual(creds.get_username(), "env_user")
         self.assertEqual(creds.get_domain(), lp.get("workgroup").upper())
         self.assertEqual(creds.get_realm(), realm.upper())
@@ -338,7 +362,7 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         self.assertEqual(creds.get_domain(), lp.get("workgroup").upper())
         self.assertEqual(creds.get_realm(), realm.upper())
         self.assertEqual(creds.get_principal(), "unknown@realm.example.com")
-        creds.parse_string("domain\user")
+        creds.parse_string("domain\\user")
         self.assertEqual(creds.get_username(), "user")
         self.assertEqual(creds.get_domain(), "DOMAIN")
         self.assertEqual(creds.get_realm(), realm.upper())
@@ -362,7 +386,7 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         self.assertEqual(creds.get_domain(), lp.get("workgroup").upper())
         self.assertEqual(creds.get_realm(), realm.upper())
         self.assertEqual(creds.get_principal(), "unknown@realm.example.com")
-        creds.parse_string("domain\user")
+        creds.parse_string("domain\\user")
         self.assertEqual(creds.get_username(), "user")
         self.assertEqual(creds.get_domain(), "DOMAIN")
         self.assertEqual(creds.get_realm(), "DOMAIN")
@@ -402,7 +426,7 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         lp = samba.tests.env_loadparm()
         os.environ["USER"] = "env_user"
         creds.guess(lp)
-        creds.parse_string("domain\user%pass")
+        creds.parse_string("domain\\user%pass")
         self.assertEqual(creds.get_username(), "user")
         self.assertEqual(creds.get_domain(), "DOMAIN")
         self.assertEqual(creds.get_password(), "pass")
@@ -424,3 +448,54 @@ class CredentialsTests(samba.tests.TestCaseInTempDir):
         self.assertEqual(creds.get_principal(), "user@samba.org")
         self.assertEqual(creds.is_anonymous(), False)
         self.assertEqual(creds.authentication_requested(), True)
+
+    def test_smb_signing(self):
+        creds = credentials.Credentials()
+        self.assertEqual(creds.get_smb_signing(), credentials.SMB_SIGNING_DEFAULT)
+        creds.set_smb_signing(credentials.SMB_SIGNING_REQUIRED)
+        self.assertEqual(creds.get_smb_signing(), credentials.SMB_SIGNING_REQUIRED)
+
+    def test_smb_signing_set_conf(self):
+        lp = samba.tests.env_loadparm()
+
+        creds = credentials.Credentials()
+        creds.set_conf(lp)
+        self.assertEqual(creds.get_smb_signing(), credentials.SMB_SIGNING_DEFAULT)
+        creds.set_smb_signing(credentials.SMB_SIGNING_OFF)
+        self.assertEqual(creds.get_smb_signing(), credentials.SMB_SIGNING_OFF)
+        creds.set_conf(lp)
+        self.assertEqual(creds.get_smb_signing(), credentials.SMB_SIGNING_OFF)
+
+    def test_smb_ipc_signing(self):
+        creds = credentials.Credentials()
+        self.assertEqual(creds.get_smb_ipc_signing(), credentials.SMB_SIGNING_REQUIRED)
+        creds.set_smb_ipc_signing(credentials.SMB_SIGNING_OFF)
+        self.assertEqual(creds.get_smb_ipc_signing(), credentials.SMB_SIGNING_OFF)
+
+    def test_smb_ipc_signing_set_conf(self):
+        lp = samba.tests.env_loadparm()
+
+        creds = credentials.Credentials()
+        creds.set_conf(lp)
+        self.assertEqual(creds.get_smb_ipc_signing(), credentials.SMB_SIGNING_REQUIRED)
+        creds.set_smb_ipc_signing(credentials.SMB_SIGNING_OFF)
+        self.assertEqual(creds.get_smb_ipc_signing(), credentials.SMB_SIGNING_OFF)
+        creds.set_conf(lp)
+        self.assertEqual(creds.get_smb_ipc_signing(), credentials.SMB_SIGNING_OFF)
+
+    def test_smb_encryption(self):
+        creds = credentials.Credentials()
+        self.assertEqual(creds.get_smb_encryption(), credentials.SMB_ENCRYPTION_DEFAULT)
+        creds.set_smb_encryption(credentials.SMB_ENCRYPTION_REQUIRED)
+        self.assertEqual(creds.get_smb_encryption(), credentials.SMB_ENCRYPTION_REQUIRED)
+
+    def test_smb_encryption_set_conf(self):
+        lp = samba.tests.env_loadparm()
+
+        creds = credentials.Credentials()
+        creds.set_conf(lp)
+        self.assertEqual(creds.get_smb_encryption(), credentials.SMB_ENCRYPTION_DEFAULT)
+        creds.set_smb_encryption(credentials.SMB_ENCRYPTION_OFF)
+        self.assertEqual(creds.get_smb_encryption(), credentials.SMB_ENCRYPTION_OFF)
+        creds.set_conf(lp)
+        self.assertEqual(creds.get_smb_encryption(), credentials.SMB_ENCRYPTION_OFF)
