@@ -15,8 +15,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-
-from __future__ import print_function
 """Tests process restarting in the pre-fork process model.
    NOTE: As this test kills samba processes it won't play nicely with other
          tests, so needs to be run in it's own environment.
@@ -33,7 +31,7 @@ from samba.dcerpc import echo, netlogon
 from samba.messaging import Messaging
 from samba.samdb import SamDB
 from samba.credentials import Credentials, DONT_USE_KERBEROS
-from samba.compat import get_string
+from samba.common import get_string
 from samba.dsdb import (
     UF_WORKSTATION_TRUST_ACCOUNT,
     UF_PASSWD_NOTREQD)
@@ -192,12 +190,12 @@ class PreforkProcessRestartTests(TestCase):
         self.assertIsNotNone(new_pid)
 
         # check that the pid has changed
-        self.assertNotEquals(pid, new_pid)
+        self.assertNotEqual(pid, new_pid)
 
         # check that the worker processes have restarted
         new_workers = self.get_worker_pids("ldap", NUM_WORKERS)
         for x in range(NUM_WORKERS):
-            self.assertNotEquals(workers[x], new_workers[x])
+            self.assertNotEqual(workers[x], new_workers[x])
 
         # check that the previous server entries have been removed.
         self.check_for_duplicate_processes()
@@ -232,7 +230,7 @@ class PreforkProcessRestartTests(TestCase):
         # check that the worker processes have restarted
         new_workers = self.get_worker_pids("ldap", NUM_WORKERS)
         # process 0 should have a new pid the others should be unchanged
-        self.assertNotEquals(workers[0], new_workers[0])
+        self.assertNotEqual(workers[0], new_workers[0])
         self.assertEqual(workers[1], new_workers[1])
         self.assertEqual(workers[2], new_workers[2])
         self.assertEqual(workers[3], new_workers[3])
@@ -275,7 +273,7 @@ class PreforkProcessRestartTests(TestCase):
         # check that the worker processes have restarted
         new_workers = self.get_worker_pids("ldap", NUM_WORKERS)
         for x in range(NUM_WORKERS):
-            self.assertNotEquals(workers[x], new_workers[x])
+            self.assertNotEqual(workers[x], new_workers[x])
 
         # check that the previous server entries have been removed.
         self.check_for_duplicate_processes()
@@ -308,12 +306,12 @@ class PreforkProcessRestartTests(TestCase):
         self.assertIsNotNone(new_pid)
 
         # check that the pid has changed
-        self.assertNotEquals(pid, new_pid)
+        self.assertNotEqual(pid, new_pid)
 
         # check that the worker processes have restarted
         new_workers = self.get_worker_pids("rpc", NUM_WORKERS)
         for x in range(NUM_WORKERS):
-            self.assertNotEquals(workers[x], new_workers[x])
+            self.assertNotEqual(workers[x], new_workers[x])
 
         # check that the previous server entries have been removed.
         self.check_for_duplicate_processes()
@@ -349,7 +347,7 @@ class PreforkProcessRestartTests(TestCase):
         # check that the worker processes have restarted
         new_workers = self.get_worker_pids("rpc", NUM_WORKERS)
         # process 0 should have a new pid the others should be unchanged
-        self.assertNotEquals(workers[0], new_workers[0])
+        self.assertNotEqual(workers[0], new_workers[0])
         self.assertEqual(workers[1], new_workers[1])
         self.assertEqual(workers[2], new_workers[2])
         self.assertEqual(workers[3], new_workers[3])
@@ -392,7 +390,7 @@ class PreforkProcessRestartTests(TestCase):
         # check that the worker processes have restarted
         new_workers = self.get_worker_pids("rpc", NUM_WORKERS)
         for x in range(NUM_WORKERS):
-            self.assertNotEquals(workers[x], new_workers[x])
+            self.assertNotEqual(workers[x], new_workers[x])
 
         # check that the previous server entries have been removed.
         self.check_for_duplicate_processes()
@@ -436,7 +434,7 @@ class PreforkProcessRestartTests(TestCase):
         # check that the worker processes have restarted
         new_workers = self.get_worker_pids("echo", NUM_WORKERS)
         for x in range(NUM_WORKERS):
-            self.assertNotEquals(workers[x], new_workers[x])
+            self.assertNotEqual(workers[x], new_workers[x])
 
         # check that the previous server entries have been removed.
         self.check_for_duplicate_processes()
