@@ -128,21 +128,21 @@ class JoinLmdbSizeTestCase(SambaToolCmdTest):
         (result, out, err) = self.run_command(
             'samba-tool domain join --backend-store-size "2"')
         self.assertGreater(result, 0)
-        self.assertRegexpMatches(err,
-                                 r"--backend-store-size invalid suffix ''")
+        self.assertRegex(err,
+                         r"--backend-store-size invalid suffix ''")
 
     def test_invalid_unit_suffix(self):
         (result, out, err) = self.run_command(
             'samba-tool domain join --backend-store-size "2 cd"')
         self.assertGreater(result, 0)
-        self.assertRegexpMatches(err,
-                                 r"--backend-store-size invalid suffix 'cd'")
+        self.assertRegex(err,
+                         r"--backend-store-size invalid suffix 'cd'")
 
     def test_non_numeric(self):
         (result, out, err) = self.run_command(
             'samba-tool domain join --backend-store-size "two Gb"')
         self.assertGreater(result, 0)
-        self.assertRegexpMatches(
+        self.assertRegex(
             err,
             r"backend-store-size option requires a numeric value, with an"
             " optional unit suffix")
