@@ -329,8 +329,8 @@ class UserCmdCryptShaTestCase(SambaToolCmdTest):
         sha512 = _get_attribute(out, "virtualCryptSHA512")
 
         out = self._get_password("virtualCryptSHA256,virtualCryptSHA512")
-        self.assertEquals(sha256, _get_attribute(out, "virtualCryptSHA256"))
-        self.assertEquals(sha512, _get_attribute(out, "virtualCryptSHA512"))
+        self.assertEqual(sha256, _get_attribute(out, "virtualCryptSHA256"))
+        self.assertEqual(sha512, _get_attribute(out, "virtualCryptSHA512"))
 
     # gpg decryption not enabled.
     # both virtual attributes specified, rounds specified
@@ -352,8 +352,8 @@ class UserCmdCryptShaTestCase(SambaToolCmdTest):
         sha512 = _get_attribute(out, "virtualCryptSHA512")
 
         out = self._get_password("virtualCryptSHA256,virtualCryptSHA512")
-        self.assertEquals(sha256, _get_attribute(out, "virtualCryptSHA256"))
-        self.assertEquals(sha512, _get_attribute(out, "virtualCryptSHA512"))
+        self.assertEqual(sha256, _get_attribute(out, "virtualCryptSHA256"))
+        self.assertEqual(sha512, _get_attribute(out, "virtualCryptSHA512"))
 
     # gpg decryption not enabled.
     # both virtual attributes specified, rounds specified
@@ -379,8 +379,8 @@ class UserCmdCryptShaTestCase(SambaToolCmdTest):
 
         out = self._get_password("virtualCryptSHA256;rounds=2561," +
                                  "virtualCryptSHA512;rounds=5129")
-        self.assertEquals(sha256, _get_attribute(out, "virtualCryptSHA256"))
-        self.assertEquals(sha512, _get_attribute(out, "virtualCryptSHA512"))
+        self.assertEqual(sha256, _get_attribute(out, "virtualCryptSHA256"))
+        self.assertEqual(sha512, _get_attribute(out, "virtualCryptSHA512"))
 
         # Number of rounds should match that specified
         self.assertTrue(sha256.startswith("{CRYPT}$5$rounds=2561"))
@@ -411,15 +411,15 @@ class UserCmdCryptShaTestCase(SambaToolCmdTest):
 
         out = self._get_password("virtualCryptSHA256;rounds=4000," +
                                  "virtualCryptSHA512;rounds=5000")
-        self.assertEquals(sha256, _get_attribute(out, "virtualCryptSHA256"))
-        self.assertEquals(sha512, _get_attribute(out, "virtualCryptSHA512"))
+        self.assertEqual(sha256, _get_attribute(out, "virtualCryptSHA256"))
+        self.assertEqual(sha512, _get_attribute(out, "virtualCryptSHA512"))
 
         # As the number of rounds did not match, should have returned the
-        # first hash of the coresponding scheme
+        # first hash of the corresponding scheme
         out = self._get_password("virtualCryptSHA256," +
                                  "virtualCryptSHA512")
-        self.assertEquals(sha256, _get_attribute(out, "virtualCryptSHA256"))
-        self.assertEquals(sha512, _get_attribute(out, "virtualCryptSHA512"))
+        self.assertEqual(sha256, _get_attribute(out, "virtualCryptSHA256"))
+        self.assertEqual(sha512, _get_attribute(out, "virtualCryptSHA512"))
 
     # gpg decryption enabled.
     # both virtual attributes specified, no rounds option
@@ -440,8 +440,8 @@ class UserCmdCryptShaTestCase(SambaToolCmdTest):
         sha512 = _get_attribute(out, "virtualCryptSHA512")
 
         out = self._get_password("virtualCryptSHA256,virtualCryptSHA512", True)
-        self.assertEquals(sha256, _get_attribute(out, "virtualCryptSHA256"))
-        self.assertEquals(sha512, _get_attribute(out, "virtualCryptSHA512"))
+        self.assertEqual(sha256, _get_attribute(out, "virtualCryptSHA256"))
+        self.assertEqual(sha512, _get_attribute(out, "virtualCryptSHA512"))
 
     # gpg decryption enabled.
     # both virtual attributes specified, rounds specified
@@ -499,8 +499,8 @@ class UserCmdCryptShaTestCase(SambaToolCmdTest):
         out = self._get_password("virtualCryptSHA256;rounds=2561," +
                                  "virtualCryptSHA512;rounds=5129",
                                  True)
-        self.assertEquals(sha256, _get_attribute(out, "virtualCryptSHA256"))
-        self.assertEquals(sha512, _get_attribute(out, "virtualCryptSHA512"))
+        self.assertEqual(sha256, _get_attribute(out, "virtualCryptSHA256"))
+        self.assertEqual(sha512, _get_attribute(out, "virtualCryptSHA512"))
 
         # The returned hashes should specify the correct number of rounds
         self.assertTrue(sha256.startswith("{CRYPT}$5$rounds=2561"))
