@@ -15,14 +15,15 @@ use IO::Poll qw(POLLIN);
 
 sub new($$$$$) {
 	my ($classname, $bindir, $srcdir, $server_maxtime,
-	    $opt_socket_wrapper_pcap, $opt_socket_wrapper_keep_pcap) = @_;
+	    $opt_socket_wrapper_pcap, $opt_socket_wrapper_keep_pcap,
+	    $default_ldb_backend) = @_;
 
 	my $self = {
 	    opt_socket_wrapper_pcap => $opt_socket_wrapper_pcap,
 	    opt_socket_wrapper_keep_pcap => $opt_socket_wrapper_keep_pcap,
 	};
 	$self->{samba3} = new Samba3($self, $bindir, $srcdir, $server_maxtime);
-	$self->{samba4} = new Samba4($self, $bindir, $srcdir, $server_maxtime);
+	$self->{samba4} = new Samba4($self, $bindir, $srcdir, $server_maxtime, $default_ldb_backend);
 	bless $self;
 	return $self;
 }
@@ -126,6 +127,10 @@ sub setup_env($$$)
 		warn("failed to start up environment '$envname'");
 		return undef;
 	}
+	if ($env eq "UNKNOWN") {
+		warn("unknown environment '$envname'");
+		return $env;
+	}
 
 	$target->{vars}->{$envname} = $env;
 	$target->{vars}->{$envname}->{target} = $target;
@@ -191,6 +196,7 @@ sub prepare_keyblobs($)
 
 	my $cadir = "$ENV{SRCDIR_ABS}/selftest/manage-ca/CA-samba.example.com";
 	my $cacert = "$cadir/Public/CA-samba.example.com-cert.pem";
+	# A file containing a CRL with no revocations.
 	my $cacrl_pem = "$cadir/Public/CA-samba.example.com-crl.pem";
 	my $dcdnsname = "$ctx->{hostname}.$ctx->{dnsname}";
 	my $dcdir = "$cadir/DCs/$dcdnsname";
@@ -377,6 +383,7 @@ sub mk_krb5_conf($$)
 	enable-pkinit = true
 	pkinit_identity = FILE:$ctx->{tlsdir}/kdc.pem,$ctx->{tlsdir}/key.pem
 	pkinit_anchors = FILE:$ctx->{tlsdir}/ca.pem
+	pkinit_revoke = FILE:$ctx->{tlsdir}/crl.pem
 
 ";
 		}
@@ -940,6 +947,10 @@ my @exported_envvars = (
 	"PASSWORD",
 	"DC_USERNAME",
 	"DC_PASSWORD",
+	"DOMAIN_ADMIN",
+	"DOMAIN_ADMIN_PASSWORD",
+	"DOMAIN_USER",
+	"DOMAIN_USER_PASSWORD",
 
 	# UID/GID for rfc2307 mapping tests
 	"UID_RFC2307TEST",
@@ -958,6 +969,7 @@ my @exported_envvars = (
 	"UNACCEPTABLE_PASSWORD",
 	"LOCK_DIR",
 	"SMBD_TEST_LOG",
+	"KRB5_CRL_FILE",
 
 	# nss_wrapper
 	"NSS_WRAPPER_PASSWD",
@@ -970,6 +982,25 @@ my @exported_envvars = (
 	# resolv_wrapper
 	"RESOLV_WRAPPER_CONF",
 	"RESOLV_WRAPPER_HOSTS",
+
+	# ctdb stuff
+	"NUM_NODES",
+	"CTDB_BASE",
+	"CTDB_SOCKET",
+	"CTDB_SERVER_NAME",
+	"CTDB_IFACE_IP",
+	"CTDB_BASE_NODE0",
+	"CTDB_SOCKET_NODE0",
+	"CTDB_SERVER_NAME_NODE0",
+	"CTDB_IFACE_IP_NODE0",
+	"CTDB_BASE_NODE1",
+	"CTDB_SOCKET_NODE1",
+	"CTDB_SERVER_NAME_NODE1",
+	"CTDB_IFACE_IP_NODE1",
+	"CTDB_BASE_NODE2",
+	"CTDB_SOCKET_NODE2",
+	"CTDB_SERVER_NAME_NODE2",
+	"CTDB_IFACE_IP_NODE2",
 );
 
 sub exported_envvars_str
