@@ -29,6 +29,7 @@
 #include "auth/gensec/gensec.h"
 #include "auth/credentials/credentials.h"
 #include "lib/global_contexts.h"
+#include "lib/util/idtree.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_AUTH
@@ -151,7 +152,9 @@ static NTSTATUS check_samba4_security(
 
 	nt_status = auth_convert_user_info_dc_saminfo3(mem_ctx,
 						       user_info_dc,
-						       &info3);
+						       AUTH_INCLUDE_RESOURCE_GROUPS,
+						       &info3,
+						       NULL);
 	if (NT_STATUS_IS_OK(nt_status)) {
 		/* We need the strings from the server_info to be valid as long as the info3 is around */
 		talloc_steal(info3, user_info_dc);
