@@ -81,7 +81,7 @@ struct _SMBCSRV {
         struct policy_handle pol;
 	time_t last_echo_time;
 
-	SMBCSRV *next, *prev;
+	struct _SMBCSRV *next, *prev;
 };
 
 /*
@@ -116,7 +116,7 @@ struct _SMBCFILE {
 	struct smbc_dirplus_list *dirplus_list, *dirplus_end, *dirplus_next;
 	int dir_type, dir_error;
 
-	SMBCFILE *next, *prev;
+	struct _SMBCFILE *next, *prev;
 };
 
 
@@ -192,10 +192,9 @@ struct SMBC_internal_data {
         bool                                    case_sensitive;
 
 	/*
-	 * Auth info needed for DFS traversal.
+	 * Credentials needed for DFS traversal.
 	 */
-
-	struct user_auth_info			*auth_info;
+	struct cli_credentials *creds;
 
         struct smbc_server_cache * server_cache;
 
@@ -401,7 +400,7 @@ int
 SMBC_close_ctx(SMBCCTX *context,
                SMBCFILE *file);
 
-bool
+NTSTATUS
 SMBC_getatr(SMBCCTX * context,
             SMBCSRV *srv,
             const char *path,
