@@ -52,12 +52,9 @@ static NTSTATUS libnetapi_samr_lookup_and_open_alias(TALLOC_CTX *mem_ctx,
 					 &user_rids,
 					 &name_types,
 					 &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		return status;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		return result;
-	}
 	if (user_rids.count != 1) {
 		return NT_STATUS_INVALID_NETWORK_RESPONSE;
 	}
@@ -110,11 +107,7 @@ static NTSTATUS libnetapi_samr_open_alias_queryinfo(TALLOC_CTX *mem_ctx,
 				       rid,
 				       &alias_handle,
 				       &result);
-	if (!NT_STATUS_IS_OK(status)) {
-		goto done;
-	}
-	if (!NT_STATUS_IS_OK(result)) {
-		status = result;
+	if (any_nt_status_not_ok(status, result, &status)) {
 		goto done;
 	}
 
@@ -123,11 +116,7 @@ static NTSTATUS libnetapi_samr_open_alias_queryinfo(TALLOC_CTX *mem_ctx,
 					    level,
 					    &_alias_info,
 					    &result);
-	if (!NT_STATUS_IS_OK(status)) {
-		goto done;
-	}
-	if (!NT_STATUS_IS_OK(result)) {
-		status = result;
+	if (any_nt_status_not_ok(status, result, &status)) {
 		goto done;
 	}
 
@@ -239,15 +228,10 @@ WERROR NetLocalGroupAdd_r(struct libnetapi_ctx *ctx,
 					    &alias_handle,
 					    &rid,
 					    &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
-
 
 	if (r->in.level == 1 && info1->lgrpi1_comment) {
 
@@ -260,14 +244,10 @@ WERROR NetLocalGroupAdd_r(struct libnetapi_ctx *ctx,
 						  ALIASINFODESCRIPTION,
 						  &alias_info,
 						  &result);
-		if (!NT_STATUS_IS_OK(status)) {
+		if (any_nt_status_not_ok(status, result, &status)) {
 			werr = ntstatus_to_werror(status);
 			goto done;
 		}
-		if (!NT_STATUS_IS_OK(result)) {
-			werr = ntstatus_to_werror(result);
-			goto done;
-		}
 	}
 
 	werr = WERR_OK;
@@ -383,14 +363,10 @@ WERROR NetLocalGroupDel_r(struct libnetapi_ctx *ctx,
 	status = dcerpc_samr_DeleteDomAlias(b, talloc_tos(),
 					    &alias_handle,
 					    &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	ZERO_STRUCT(alias_handle);
 
@@ -564,14 +540,10 @@ WERROR NetLocalGroupGetInfo_r(struct libnetapi_ctx *ctx,
 					    ALIASINFOALL,
 					    &alias_info,
 					    &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	werr = map_alias_info_to_buffer(ctx,
 					r->in.group_name,
@@ -750,14 +722,10 @@ WERROR NetLocalGroupSetInfo_r(struct libnetapi_ctx *ctx,
 					  alias_level,
 					  alias_info,
 					  &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	werr = WERR_OK;
 
@@ -866,14 +834,10 @@ WERROR NetLocalGroupEnum_r(struct libnetapi_ctx *ctx,
 					     2,
 					     &builtin_info,
 					     &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	if (r->out.total_entries) {
 		*r->out.total_entries += builtin_info->general.num_aliases;
@@ -884,14 +848,10 @@ WERROR NetLocalGroupEnum_r(struct libnetapi_ctx *ctx,
 					     2,
 					     &domain_info,
 					     &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	if (r->out.total_entries) {
 		*r->out.total_entries += domain_info->general.num_aliases;
@@ -904,14 +864,10 @@ WERROR NetLocalGroupEnum_r(struct libnetapi_ctx *ctx,
 					       r->in.prefmaxlen,
 					       &entries_read,
 					       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	for (i=0; i<builtin_sam_array->count; i++) {
 		union samr_AliasInfo *alias_info = NULL;
@@ -945,14 +901,10 @@ WERROR NetLocalGroupEnum_r(struct libnetapi_ctx *ctx,
 					       r->in.prefmaxlen,
 					       &entries_read,
 					       &result);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		werr = ntstatus_to_werror(status);
 		goto done;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		werr = ntstatus_to_werror(result);
-		goto done;
-	}
 
 	for (i=0; i<domain_sam_array->count; i++) {
 
@@ -1259,14 +1211,10 @@ static WERROR NetLocalGroupModifyMembers_r(struct libnetapi_ctx *ctx,
 						       &alias_handle,
 						       &current_sids,
 						       &result);
-		if (!NT_STATUS_IS_OK(status)) {
+		if (any_nt_status_not_ok(status, result, &status)) {
 			werr = ntstatus_to_werror(status);
 			goto done;
 		}
-		if (!NT_STATUS_IS_OK(result)) {
-			werr = ntstatus_to_werror(result);
-			goto done;
-		}
 
 		/* add list */
 
@@ -1320,14 +1268,10 @@ static WERROR NetLocalGroupModifyMembers_r(struct libnetapi_ctx *ctx,
 						    &alias_handle,
 						    &add_sids[i],
 						    &result);
-		if (!NT_STATUS_IS_OK(status)) {
+		if (any_nt_status_not_ok(status, result, &status)) {
 			werr = ntstatus_to_werror(status);
 			goto done;
 		}
-		if (!NT_STATUS_IS_OK(result)) {
-			werr = ntstatus_to_werror(result);
-			goto done;
-		}
 	}
 
 	/* del list */
@@ -1337,14 +1281,10 @@ static WERROR NetLocalGroupModifyMembers_r(struct libnetapi_ctx *ctx,
 						       &alias_handle,
 						       &del_sids[i],
 						       &result);
-		if (!NT_STATUS_IS_OK(status)) {
+		if (any_nt_status_not_ok(status, result, &status)) {
 			werr = ntstatus_to_werror(status);
 			goto done;
 		}
-		if (!NT_STATUS_IS_OK(result)) {
-			werr = ntstatus_to_werror(result);
-			goto done;
-		}
 	}
 
 	werr = WERR_OK;
