@@ -19,9 +19,6 @@
 #include "includes.h"
 #include "messages.h"
 #include "lib/util/talloc_report_printf.h"
-#ifdef HAVE_MALLINFO
-#include <malloc.h>
-#endif /* HAVE_MALLINFO */
 
 static bool pool_usage_filter(struct messaging_rec *rec, void *private_data)
 {
@@ -84,5 +81,5 @@ void register_msg_pool_usage(
 		DBG_WARNING("messaging_filtered_read_send failed\n");
 		return;
 	}
-	DEBUG(2, ("Registered MSG_REQ_POOL_USAGE\n"));
+	DBG_INFO("Registered MSG_REQ_POOL_USAGE\n");
 }
