@@ -45,7 +45,7 @@
 #define smb_panic(x) exit(1)
 #endif
 
-/* are we running as non-root? This is used by the regresison test code,
+/* are we running as non-root? This is used by the regression test code,
    and potentially also for sites that want non-root smbd */
 static uid_t initial_uid;
 static gid_t initial_gid;
@@ -99,7 +99,7 @@ gid_t sec_initial_gid(void)
 /**
  * @brief Check if we are running in root mode.
  *
- * @return If we samba root privileges it returns true, false otehrwise.
+ * @return Return whether Samba has root privileges
  */
 bool root_mode(void)
 {
@@ -444,17 +444,17 @@ void become_user_permanently(uid_t uid, gid_t gid)
 	assert_gid(gid, gid);
 }
 
-#ifdef HAVE___THREAD
-	struct cache_t {
+#if defined(HAVE_LINUX_THREAD_CREDENTIALS) && defined(HAVE___THREAD)
+	struct set_thread_credentials_cache {
 		bool active;
 		uid_t uid;
 		gid_t gid;
 		size_t setlen;
 		uintptr_t gidset;
 	};
-
-static __thread struct cache_t cache;
+static __thread struct set_thread_credentials_cache cache;
 #endif
+
 /**********************************************************
  Function to set thread specific credentials. Leave
  saved-set uid/gid alone.Must be thread-safe code.
