@@ -105,8 +105,6 @@ static ADS_STATUS ads_do_search_retry_internal(ADS_STRUCT *ads, const char *bind
 		status = ads_connect(ads);
 
 		if (!ADS_ERR_OK(status)) {
-			bool orig_is_mine = ads->is_mine;
-
 			DEBUG(1,("ads_search_retry: failed to reconnect (%s)\n",
 				 ads_errstr(status)));
 			/*
@@ -114,9 +112,7 @@ static ADS_STATUS ads_do_search_retry_internal(ADS_STRUCT *ads, const char *bind
 			 * from being freed here as we don't own it and
 			 * callers depend on it being around.
 			 */
-			ads->is_mine = false;
-			ads_destroy(&ads);
-			ads->is_mine = orig_is_mine;
+			ads_disconnect(ads);
 			SAFE_FREE(bp);
 			return status;
 		}
