@@ -103,6 +103,8 @@ SMBC_open_ctx(SMBCCTX *context,
 	if (strlen(path) > 0 && path[strlen(path) - 1] == '\\') {
 		status = NT_STATUS_OBJECT_PATH_INVALID;
 	} else {
+		struct cli_credentials *creds = NULL;
+
 		file = SMB_MALLOC_P(SMBCFILE);
 		if (!file) {
 			errno = ENOMEM;
@@ -112,9 +114,11 @@ SMBC_open_ctx(SMBCCTX *context,
 
 		ZERO_STRUCTP(file);
 
+		creds = context->internal->creds;
 		/*d_printf(">>>open: resolving %s\n", path);*/
 		status = cli_resolve_path(
-			frame, "", context->internal->auth_info,
+			frame, "",
+			creds,
 			srv->cli, path, &targetcli, &targetpath);
 		if (!NT_STATUS_IS_OK(status)) {
 			d_printf("Could not resolve %s\n", path);
@@ -132,8 +136,8 @@ SMBC_open_ctx(SMBCCTX *context,
 			/* Handle the error ... */
 
 			SAFE_FREE(file);
-			errno = SMBC_errno(context, targetcli);
 			TALLOC_FREE(frame);
+			errno = cli_status_to_errno(status);
 			return NULL;
 		}
 
@@ -193,12 +197,11 @@ SMBC_open_ctx(SMBCCTX *context,
 	/* Check if opendir needed ... */
 
 	if (!NT_STATUS_IS_OK(status)) {
-		int eno = 0;
-
-		eno = SMBC_errno(context, srv->cli);
 		file = smbc_getFunctionOpendir(context)(context, fname);
-		if (!file) errno = eno;
 		TALLOC_FREE(frame);
+		if (file == NULL) {
+			errno = cli_status_to_errno(status);
+		}
 		return file;
 	}
 
@@ -277,8 +280,8 @@ SMBC_read_ctx(SMBCCTX *context,
 	status = cli_read(file->targetcli, file->cli_fd, (char *)buf, offset,
 			  count, &ret);
 	if (!NT_STATUS_IS_OK(status)) {
-		errno = SMBC_errno(context, file->targetcli);
 		TALLOC_FREE(frame);
+		errno = cli_status_to_errno(status);
 		return -1;
 	}
 
@@ -325,8 +328,8 @@ SMBC_splice_ctx(SMBCCTX *context,
 			    count, srcfile->offset, dstfile->offset, &written,
 			    splice_cb, priv);
 	if (!NT_STATUS_IS_OK(status)) {
-		errno = SMBC_errno(context, srcfile->targetcli);
 		TALLOC_FREE(frame);
+		errno = cli_status_to_errno(status);
 		return -1;
 	}
 
@@ -398,6 +401,7 @@ SMBC_close_ctx(SMBCCTX *context,
                SMBCFILE *file)
 {
 	TALLOC_CTX *frame = talloc_stackframe();
+	NTSTATUS status;
 
 	if (!context || !context->internal->initialized) {
 		errno = EINVAL;
@@ -417,19 +421,20 @@ SMBC_close_ctx(SMBCCTX *context,
 		return smbc_getFunctionClosedir(context)(context, file);
 	}
 
-	if (!NT_STATUS_IS_OK(cli_close(file->targetcli, file->cli_fd))) {
+	status = cli_close(file->targetcli, file->cli_fd);
+	if (!NT_STATUS_IS_OK(status)) {
 		SMBCSRV *srv;
 		DEBUG(3, ("cli_close failed on %s. purging server.\n",
 			  file->fname));
 		/* Deallocate slot and remove the server
 		 * from the server cache if unused */
-		errno = SMBC_errno(context, file->targetcli);
 		srv = file->srv;
 		DLIST_REMOVE(context->internal->files, file);
 		SAFE_FREE(file->fname);
 		SAFE_FREE(file);
 		smbc_getFunctionRemoveUnusedServer(context)(context, srv);
 		TALLOC_FREE(frame);
+		errno = cli_status_to_errno(status);
 		return -1;
 	}
 
@@ -444,7 +449,7 @@ SMBC_close_ctx(SMBCCTX *context,
  * Get info from an SMB server on a file. Use a qpathinfo call first
  * and if that fails, use getatr, as Win95 sometimes refuses qpathinfo
  */
-bool
+NTSTATUS
 SMBC_getatr(SMBCCTX * context,
             SMBCSRV *srv,
             const char *path,
@@ -459,48 +464,50 @@ SMBC_getatr(SMBCCTX * context,
 	struct timespec access_time_ts = {0};
 	struct timespec write_time_ts = {0};
 	struct timespec change_time_ts = {0};
+	struct timespec w_time_ts = {0};
 	time_t write_time = 0;
 	SMB_INO_T ino = 0;
+	struct cli_credentials *creds = NULL;
 	TALLOC_CTX *frame = talloc_stackframe();
 	NTSTATUS status;
 
 	if (!context || !context->internal->initialized) {
-		errno = EINVAL;
 		TALLOC_FREE(frame);
- 		return False;
+		return NT_STATUS_INVALID_PARAMETER;
  	}
 
 	/* path fixup for . and .. */
 	if (ISDOT(path) || ISDOTDOT(path)) {
 		fixedpath = talloc_strdup(frame, "\\");
 		if (!fixedpath) {
-			errno = ENOMEM;
 			TALLOC_FREE(frame);
-			return False;
+			return NT_STATUS_NO_MEMORY;
 		}
 	} else {
 		fixedpath = talloc_strdup(frame, path);
 		if (!fixedpath) {
-			errno = ENOMEM;
 			TALLOC_FREE(frame);
-			return False;
+			return NT_STATUS_NO_MEMORY;
 		}
 		trim_string(fixedpath, NULL, "\\..");
 		trim_string(fixedpath, NULL, "\\.");
 	}
 	DEBUG(4,("SMBC_getatr: sending qpathinfo\n"));
 
-	status = cli_resolve_path(frame, "", context->internal->auth_info,
+	creds = context->internal->creds;
+
+	status = cli_resolve_path(frame, "",
+				  creds,
 				  srv->cli, fixedpath,
 				  &targetcli, &targetpath);
 	if (!NT_STATUS_IS_OK(status)) {
 		d_printf("Couldn't resolve %s\n", path);
-                errno = ENOENT;
 		TALLOC_FREE(frame);
-		return False;
+		return NT_STATUS_OBJECT_NAME_NOT_FOUND;
 	}
 
 	if (!srv->no_pathinfo2) {
+		bool not_supported_error = false;
 		status = cli_qpathinfo2(targetcli,
 					targetpath,
 					&create_time_ts,
@@ -513,11 +520,21 @@ SMBC_getatr(SMBCCTX * context,
 		if (NT_STATUS_IS_OK(status)) {
 			goto setup_stat;
 		}
+		if (NT_STATUS_EQUAL(status, NT_STATUS_INVALID_LEVEL) ||
+		    NT_STATUS_EQUAL(status, NT_STATUS_NOT_SUPPORTED)) {
+			not_supported_error = true;
+		}
+		if (!not_supported_error) {
+			/* "Normal error". Just return it to caller. */
+			TALLOC_FREE(frame);
+			return status;
+		}
         }
 
 	srv->no_pathinfo2 = True;
 
 	if (!srv->no_pathinfo3) {
+		bool not_supported_error = false;
 		status = cli_qpathinfo3(targetcli,
 					targetpath,
 					&create_time_ts,
@@ -530,6 +547,15 @@ SMBC_getatr(SMBCCTX * context,
 		if (NT_STATUS_IS_OK(status)) {
 			goto setup_stat;
 		}
+		if (NT_STATUS_EQUAL(status, NT_STATUS_INVALID_LEVEL) ||
+		    NT_STATUS_EQUAL(status, NT_STATUS_NOT_SUPPORTED)) {
+			not_supported_error = true;
+		}
+		if (!not_supported_error) {
+			/* "Normal error". Just return it to caller. */
+			TALLOC_FREE(frame);
+			return status;
+		}
         }
 
 	srv->no_pathinfo3 = True;
@@ -540,14 +566,11 @@ SMBC_getatr(SMBCCTX * context,
         }
 
 	status = cli_getatr(targetcli, targetpath, &attr, &size, &write_time);
-	if (NT_STATUS_IS_OK(status)) {
-		struct timespec w_time_ts =
-			convert_time_t_to_timespec(write_time);
-
-		access_time_ts = change_time_ts = write_time_ts = w_time_ts;
-
-		goto setup_stat;
+	if (!NT_STATUS_IS_OK(status)) {
+		goto all_failed;
 	}
+	w_time_ts = convert_time_t_to_timespec(write_time);
+	access_time_ts = change_time_ts = write_time_ts = w_time_ts;
 
 setup_stat:
 	setup_stat(sb,
@@ -561,15 +584,14 @@ setup_stat:
 		   write_time_ts);
 
 	TALLOC_FREE(frame);
-	return true;
+	return NT_STATUS_OK;
 
 all_failed:
 	srv->no_pathinfo2 = False;
 	srv->no_pathinfo3 = False;
 
-        errno = EPERM;
 	TALLOC_FREE(frame);
-	return False;
+	return status;
 }
 
 /*
@@ -591,8 +613,8 @@ SMBC_setatr(SMBCCTX * context, SMBCSRV *srv, char *path,
             uint16_t attr)
 {
         uint16_t fd;
-        int ret;
 	uint32_t lattr = (uint32_t)attr;
+	NTSTATUS status;
 	TALLOC_CTX *frame = talloc_stackframe();
 
 	if (attr == (uint16_t)-1) {
@@ -636,17 +658,20 @@ SMBC_setatr(SMBCCTX * context, SMBCSRV *srv, char *path,
                 srv->no_pathinfo = True;
 
                 /* Open the file */
-                if (!NT_STATUS_IS_OK(cli_open(srv->cli, path, O_RDWR, DENY_NONE, &fd))) {
-                        errno = SMBC_errno(context, srv->cli);
+		status = cli_open(srv->cli, path, O_RDWR, DENY_NONE, &fd);
+		if (!NT_STATUS_IS_OK(status)) {
 			TALLOC_FREE(frame);
+                        errno = cli_status_to_errno(status);
                         return False;
                 }
 
                 /* Set the new attributes */
-                ret = NT_STATUS_IS_OK(cli_setattrE(srv->cli, fd,
-                                   change_time.tv_sec,
-                                   access_time.tv_sec,
-                                   write_time.tv_sec));
+		status = cli_setattrE(
+			srv->cli,
+			fd,
+			change_time.tv_sec,
+			access_time.tv_sec,
+			write_time.tv_sec);
 
                 /* Close the file */
                 cli_close(srv->cli, fd);
@@ -657,13 +682,13 @@ SMBC_setatr(SMBCCTX * context, SMBCSRV *srv, char *path,
                  * cli_setatr() for that, and with only this parameter, it
                  * seems to work on win98.
                  */
-                if (ret && attr != (uint16_t) -1) {
-                        ret = NT_STATUS_IS_OK(cli_setatr(srv->cli, path, (uint32_t)attr, 0));
+                if (NT_STATUS_IS_OK(status) && attr != (uint16_t) -1) {
+			status = cli_setatr(srv->cli, path, (uint32_t)attr, 0);
                 }
 
-                if (! ret) {
-                        errno = SMBC_errno(context, srv->cli);
+                if (!NT_STATUS_IS_OK(status)) {
 			TALLOC_FREE(frame);
+                        errno = cli_status_to_errno(status);
                         return False;
                 }
         }
