@@ -369,12 +369,7 @@ void brl_init(bool read_only)
 		return;
 	}
 
-	tdb_flags =
-		TDB_DEFAULT|
-		TDB_VOLATILE|
-		TDB_CLEAR_IF_FIRST|
-		TDB_INCOMPATIBLE_HASH|
-		TDB_SEQNUM;
+	tdb_flags = SMBD_VOLATILE_TDB_FLAGS | TDB_SEQNUM;
 
 	db_path = lock_path(talloc_tos(), "brlock.tdb");
 	if (db_path == NULL) {
@@ -383,7 +378,7 @@ void brl_init(bool read_only)
 	}
 
 	brlock_db = db_open(NULL, db_path,
-			    SMB_OPEN_DATABASE_TDB_HASH_SIZE, tdb_flags,
+			    SMBD_VOLATILE_TDB_HASH_SIZE, tdb_flags,
 			    read_only?O_RDONLY:(O_RDWR|O_CREAT), 0644,
 			    DBWRAP_LOCK_ORDER_2, DBWRAP_FLAG_NONE);
 	if (!brlock_db) {
@@ -882,7 +877,7 @@ static NTSTATUS brl_lock_posix(struct byte_range_lock *br_lck,
 
 	/*
 	 * Break oplocks while we hold a brl. Since lock() and unlock() calls
-	 * are not symetric with POSIX semantics, we cannot guarantee our
+	 * are not symmetric with POSIX semantics, we cannot guarantee our
 	 * contend_level2_oplocks_begin/end calls will be acquired and
 	 * released one-for-one as with Windows semantics. Therefore we only
 	 * call contend_level2_oplocks_begin if this is the first POSIX brl on
@@ -971,14 +966,6 @@ static NTSTATUS brl_lock_posix(struct byte_range_lock *br_lck,
 	return status;
 }
 
-NTSTATUS smb_vfs_call_brl_lock_windows(struct vfs_handle_struct *handle,
-				       struct byte_range_lock *br_lck,
-				       struct lock_struct *plock)
-{
-	VFS_FIND(brl_lock_windows);
-	return handle->fns->brl_lock_windows_fn(handle, br_lck, plock);
-}
-
 /****************************************************************************
  Lock a range of bytes.
 ****************************************************************************/
@@ -1250,14 +1237,6 @@ static bool brl_unlock_posix(struct byte_range_lock *br_lck,
 	return True;
 }
 
-bool smb_vfs_call_brl_unlock_windows(struct vfs_handle_struct *handle,
-				     struct byte_range_lock *br_lck,
-				     const struct lock_struct *plock)
-{
-	VFS_FIND(brl_unlock_windows);
-	return handle->fns->brl_unlock_windows_fn(handle, br_lck, plock);
-}
-
 /****************************************************************************
  Unlock a range of bytes.
 ****************************************************************************/
