@@ -28,6 +28,7 @@
 #include "dbwrap/dbwrap.h"
 #include "dbwrap/dbwrap_rbt.h"
 #include "util_tdb.h"
+#include "smbd/fd_handle.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_LOCKING
@@ -51,7 +52,7 @@ static struct db_context *posix_pending_close_db;
 
 static int map_posix_lock_type( files_struct *fsp, enum brl_type lock_type)
 {
-	if((lock_type == WRITE_LOCK) && !fsp->can_write) {
+	if ((lock_type == WRITE_LOCK) && !fsp->fsp_flags.can_write) {
 		/*
 		 * Many UNIX's cannot get a write lock on a file opened read-only.
 		 * Win32 locking semantics allow this.
@@ -84,6 +85,8 @@ static const char *posix_lock_type_name(int lock_type)
  False if not.
 ****************************************************************************/
 
+#define SMB_OFF_T_BITS (sizeof(off_t)*8)
+
 static bool posix_lock_in_range(off_t *offset_out, off_t *count_out,
 				uint64_t u_offset, uint64_t u_count)
 {
@@ -176,14 +179,6 @@ static bool posix_lock_in_range(off_t *offset_out, off_t *count_out,
 	return True;
 }
 
-bool smb_vfs_call_lock(struct vfs_handle_struct *handle,
-		       struct files_struct *fsp, int op, off_t offset,
-		       off_t count, int type)
-{
-	VFS_FIND(lock);
-	return handle->fns->lock_fn(handle, fsp, op, offset, count, type);
-}
-
 /****************************************************************************
  Actual function that does POSIX locks. Copes with 64 -> 32 bit cruft and
  broken NFS implementations.
@@ -194,7 +189,7 @@ static bool posix_fcntl_lock(files_struct *fsp, int op, off_t offset, off_t coun
 	bool ret;
 
 	DEBUG(8,("posix_fcntl_lock %d %d %jd %jd %d\n",
-		 fsp->fh->fd,op,(intmax_t)offset,(intmax_t)count,type));
+		 fsp_get_io_fd(fsp),op,(intmax_t)offset,(intmax_t)count,type));
 
 	ret = SMB_VFS_LOCK(fsp, op, offset, count, type);
 
@@ -240,15 +235,6 @@ static bool posix_fcntl_lock(files_struct *fsp, int op, off_t offset, off_t coun
 	return ret;
 }
 
-bool smb_vfs_call_getlock(struct vfs_handle_struct *handle,
-			  struct files_struct *fsp, off_t *poffset,
-			  off_t *pcount, int *ptype, pid_t *ppid)
-{
-	VFS_FIND(getlock);
-	return handle->fns->getlock_fn(handle, fsp, poffset, pcount, ptype, 
-				       ppid);
-}
-
 /****************************************************************************
  Actual function that gets POSIX locks. Copes with 64 -> 32 bit cruft and
  broken NFS implementations.
@@ -260,7 +246,7 @@ static bool posix_fcntl_getlock(files_struct *fsp, off_t *poffset, off_t *pcount
 	bool ret;
 
 	DEBUG(8, ("posix_fcntl_getlock %d %ju %ju %d\n",
-		  fsp->fh->fd, (uintmax_t)*poffset, (uintmax_t)*pcount,
+		  fsp_get_io_fd(fsp), (uintmax_t)*poffset, (uintmax_t)*pcount,
 		  *ptype));
 
 	ret = SMB_VFS_GETLOCK(fsp, poffset, pcount, ptype, &pid);
@@ -518,10 +504,11 @@ static void add_fd_to_close_entry_fn(
 	void *private_data)
 {
 	struct add_fd_to_close_entry_state *state = private_data;
+	int fd = fsp_get_pathref_fd(state->fsp);
 	TDB_DATA values[] = {
 		value,
-		{ .dptr = (uint8_t *)&(state->fsp->fh->fd),
-		  .dsize = sizeof(state->fsp->fh->fd) },
+		{ .dptr = (uint8_t *)&fd,
+		  .dsize = sizeof(fd) },
 	};
 	NTSTATUS status;
 
@@ -548,7 +535,7 @@ static void add_fd_to_close_entry(const files_struct *fsp)
 	SMB_ASSERT(NT_STATUS_IS_OK(status));
 
 	DBG_DEBUG("added fd %d file %s\n",
-		  fsp->fh->fd,
+		  fsp_get_pathref_fd(fsp),
 		  fsp_str_dbg(fsp));
 }
 
@@ -557,6 +544,7 @@ static void fd_close_posix_fn(
 	TDB_DATA data,
 	void *private_data)
 {
+	int *saved_errno = (int *)private_data;
 	size_t num_fds, i;
 
 	SMB_ASSERT((data.dsize % sizeof(int)) == 0);
@@ -564,8 +552,12 @@ static void fd_close_posix_fn(
 
 	for (i=0; i<num_fds; i++) {
 		int fd;
+		int ret;
 		memcpy(&fd, data.dptr, sizeof(int));
-		close(fd);
+		ret = close(fd);
+		if (ret == -1) {
+			*saved_errno = errno;
+		}
 		data.dptr += sizeof(int);
 	}
 	dbwrap_record_delete(rec);
@@ -585,14 +577,14 @@ int fd_close_posix(const struct files_struct *fsp)
 
 	if (!lp_locking(fsp->conn->params) ||
 	    !lp_posix_locking(fsp->conn->params) ||
-	    fsp->use_ofd_locks)
+	    fsp->fsp_flags.use_ofd_locks)
 	{
 		/*
 		 * No locking or POSIX to worry about or we are using POSIX
 		 * open file description lock semantics which only removes
 		 * locks on the file descriptor we're closing. Just close.
 		 */
-		return close(fsp->fh->fd);
+		return close(fsp_get_pathref_fd(fsp));
 	}
 
 	if (get_lock_ref_count(fsp)) {
@@ -600,7 +592,7 @@ int fd_close_posix(const struct files_struct *fsp)
 		/*
 		 * There are outstanding locks on this dev/inode pair on
 		 * other fds. Add our fd to the pending close db. We also
-		 * set fsp->fh->fd to -1 inside fd_close() after returning
+		 * set fsp_get_io_fd(fsp) to -1 inside fd_close() after returning
 		 * from VFS layer.
 		 */
 
@@ -612,7 +604,7 @@ int fd_close_posix(const struct files_struct *fsp)
 		posix_pending_close_db,
 		fd_array_key_fsp(fsp),
 		fd_close_posix_fn,
-		NULL);
+		&saved_errno);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_WARNING("dbwrap_do_locked failed: %s\n",
 			    nt_errstr(status));
@@ -625,7 +617,7 @@ int fd_close_posix(const struct files_struct *fsp)
 	 * Finally close the fd associated with this fsp.
 	 */
 
-	ret = close(fsp->fh->fd);
+	ret = close(fsp_get_pathref_fd(fsp));
 
 	if (ret == 0 && saved_errno != 0) {
 		errno = saved_errno;
@@ -928,7 +920,7 @@ bool set_posix_lock_windows_flavour(files_struct *fsp,
 	}
 
 	/*
-	 * Windows is very strange. It allows read locks to be overlayed
+	 * Windows is very strange. It allows read locks to be overlaid
 	 * (even over a write lock), but leaves the write lock in force until the first
 	 * unlock. It also reference counts the locks. This means the following sequence :
 	 *
