@@ -168,26 +168,12 @@ DIR *vfs_not_implemented_fdopendir(vfs_handle_struct *handle, files_struct *fsp,
 _PUBLIC_
 struct dirent *vfs_not_implemented_readdir(vfs_handle_struct *handle,
 					   struct files_struct *dirfsp,
-					   DIR *dirp,
-					   SMB_STRUCT_STAT *sbuf)
+					   DIR *dirp)
 {
 	errno = ENOSYS;
 	return NULL;
 }
 
-_PUBLIC_
-void vfs_not_implemented_seekdir(vfs_handle_struct *handle, DIR *dirp, long offset)
-{
-	;
-}
-
-_PUBLIC_
-long vfs_not_implemented_telldir(vfs_handle_struct *handle, DIR *dirp)
-{
-	errno = ENOSYS;
-	return (long)-1;
-}
-
 _PUBLIC_
 void vfs_not_implemented_rewind_dir(vfs_handle_struct *handle, DIR *dirp)
 {
@@ -735,8 +721,10 @@ NTSTATUS vfs_not_implemented_get_real_filename_at(
 }
 
 _PUBLIC_
-const char *vfs_not_implemented_connectpath(struct vfs_handle_struct *handle,
-					    const struct smb_filename *smb_fname)
+const char *vfs_not_implemented_connectpath(
+	struct vfs_handle_struct *handle,
+	const struct files_struct *dirfsp,
+	const struct smb_filename *smb_fname)
 {
 	errno = ENOSYS;
 	return NULL;
@@ -1090,8 +1078,6 @@ static struct vfs_fn_pointers vfs_not_implemented_fns = {
 
 	.fdopendir_fn = vfs_not_implemented_fdopendir,
 	.readdir_fn = vfs_not_implemented_readdir,
-	.seekdir_fn = vfs_not_implemented_seekdir,
-	.telldir_fn = vfs_not_implemented_telldir,
 	.rewind_dir_fn = vfs_not_implemented_rewind_dir,
 	.mkdirat_fn = vfs_not_implemented_mkdirat,
 	.closedir_fn = vfs_not_implemented_closedir,
