@@ -28,6 +28,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "system/filesys.h"
 #include "vfs_vxfs.h"
 
+#undef strcasecmp
+
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_VFS
 
@@ -60,8 +62,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
+/*
+ * Note:
+ * XATTR_USER_NTACL: This extended attribute is used
+ * to store Access Control List system objects by VxFS from DLV11 onwards.
+ * XATTR_USER_NTACL_V0: This extended attribute was used
+ * to store Access Control List system objects by VxFS till DLV10.
+ */
 #ifndef XATTR_USER_NTACL
-#define XATTR_USER_NTACL "system.NTACL"
+#define XATTR_USER_NTACL    "system.NTACL"
+#define XATTR_USER_NTACL_V0 "user.NTACL"
 #endif
 
 /* type values */
@@ -73,6 +83,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define VXFS_ACL_OTHER           5
 #define VXFS_ACL_MASK            6
 
+/*
+ * Helper function for comparing two strings
+ */
+static int vxfs_strcasecmp(const char *str1, const char *str2)
+{
+	bool match = strequal_m(str1, str2);
+	if (match) {
+		return 0;
+	}
+	return 1;
+}
 
 /*
  * Compare aces
@@ -336,9 +357,8 @@ static char * vxfs_compact_buf(char *e_buf, int *new_count, int count,
 static bool vxfs_compare_acls(char *e_buf, char *n_buf, int n_count,
 			      int e_count) {
 
-	uint16_t e_type, n_type, e_perm, n_perm;
-	uint32_t e_id, n_id;
-	int i, offset = 0;
+	uint16_t e_type, n_type;
+	int offset = 0;
 
 	if (!e_buf && !n_buf) {
 		DEBUG(10, ("vfs_vxfs: Empty buffers!\n"));
@@ -396,24 +416,21 @@ static bool vxfs_compare_acls(char *e_buf, char *n_buf, int n_count,
  * 6. Else need to set New ACL
  */
 
-static bool vxfs_compare(connection_struct *conn,
-			 const struct smb_filename *smb_fname,
+static bool vxfs_compare(struct files_struct *fsp,
 			 SMB_ACL_T the_acl,
 			 SMB_ACL_TYPE_T the_acl_type)
 {
-	char *name = smb_fname->base_name;
 	SMB_ACL_T existing_acl = NULL;
 	bool ret = false;
-	int i, count = 0;
+	int count = 0;
 	TALLOC_CTX *mem_ctx = talloc_tos();
 	char *existing_buf = NULL, *new_buf = NULL, *compact_buf = NULL;
-	struct smb_filename *smb_fname = NULL;
 	int status;
+	NTSTATUS ntstatus;
 
-	DEBUG(10, ("vfs_vxfs: Getting existing ACL for %s\n", name));
+	DEBUG(10, ("vfs_vxfs: Getting existing ACL for %s\n", fsp_str_dbg(fsp)));
 
-	existing_acl = SMB_VFS_SYS_ACL_GET_FILE(conn, smb_fname, the_acl_type,
-						mem_ctx);
+	existing_acl = SMB_VFS_SYS_ACL_GET_FD(fsp, the_acl_type, mem_ctx);
 	if (existing_acl == NULL) {
 		DEBUG(10, ("vfs_vxfs: Failed to get ACL\n"));
 		goto out;
@@ -427,22 +444,23 @@ static bool vxfs_compare(connection_struct *conn,
 		goto out;
 	}
 
-	status = SMB_VFS_STAT(conn, smb_fname);
-	if (status == -1) {
+	ntstatus = vfs_stat_fsp(fsp);
+	if (!NT_STATUS_IS_OK(ntstatus)) {
 		DEBUG(10, ("vfs_vxfs: stat failed!\n"));
+		errno = map_errno_from_nt_status(ntstatus);
 		goto out;
 	}
 
 	DEBUG(10, ("vfs_vxfs: Sorting existing ACL\n"));
 	existing_buf = vxfs_sort_acl(existing_acl, mem_ctx,
-				     smb_fname->st.st_ex_uid,
-				     smb_fname->st.st_ex_gid);
+				     fsp->fsp_name->st.st_ex_uid,
+				     fsp->fsp_name->st.st_ex_gid);
 	if (!existing_buf)
 		goto out;
 
 	DEBUG(10, ("vfs_vxfs: Sorting new ACL\n"));
-	new_buf = vxfs_sort_acl(the_acl, mem_ctx, smb_fname->st.st_ex_uid,
-				smb_fname->st.st_ex_gid);
+	new_buf = vxfs_sort_acl(the_acl, mem_ctx, fsp->fsp_name->st.st_ex_uid,
+				fsp->fsp_name->st.st_ex_gid);
 	if (!new_buf) {
 		goto out;
 	}
@@ -483,132 +501,55 @@ out:
 	return ret;
 }
 
-static int vxfs_sys_acl_set_fd(vfs_handle_struct *handle, files_struct *fsp,
+#ifdef VXFS_ACL_SHARE
+static int vxfs_sys_acl_set_fd(vfs_handle_struct *handle,
+			       struct files_struct *fsp,
+			       SMB_ACL_TYPE_T type,
 			       SMB_ACL_T theacl)
 {
 
-	if (vxfs_compare(fsp->conn, fsp->fsp_name, theacl,
-			 SMB_ACL_TYPE_ACCESS)) {
+	if (vxfs_compare(fsp, theacl, type)) {
 		return 0;
 	}
 
-	return SMB_VFS_NEXT_SYS_ACL_SET_FD(handle, fsp, theacl);
-}
-
-static int vxfs_sys_acl_set_file(vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				SMB_ACL_TYPE_T acltype,
-				SMB_ACL_T theacl)
-{
-	if (vxfs_compare(handle->conn, smb_fname,
-			theacl, acltype)) {
-		return 0;
-	}
-
-	return SMB_VFS_NEXT_SYS_ACL_SET_FILE(handle, smb_fname,
-			acltype, theacl);
-}
-
-static int vxfs_set_xattr(struct vfs_handle_struct *handle,
-			const struct smb_filename *smb_fname_in,
-			const char *name,
-			const void *value,
-			size_t size,
-			int flags)
-{
-	struct smb_filename *smb_fname = NULL;
-	bool is_dir = false;
-	int ret = 0;
-	int saved_errno = 0;
-
-	DEBUG(10, ("In vxfs_set_xattr\n"));
-
-	smb_fname = cp_smb_filename_nostream(talloc_tos(), smb_fname_in);
-	if (smb_fname == NULL) {
-		errno = ENOMEM;
-		return -1;
-	}
-
-	if (SMB_VFS_NEXT_STAT(handle, smb_fname) != 0) {
-		TALLOC_FREE(smb_fname);
-		return -1;
-	}
-
-	is_dir = S_ISDIR(smb_fname->st.st_ex_mode);
-
-	ret = vxfs_setxattr_path(smb_fname_in->base_name, name, value, size,
-				 flags, is_dir);
-	if ((ret == 0) ||
-	    ((ret == -1) && (errno != ENOTSUP) && (errno != ENOSYS))) {
-		/*
-		 * Now remve old style xattr if it exists
-		 */
-		SMB_VFS_NEXT_REMOVEXATTR(handle, smb_fname, name);
-		/*
-		 * Do not bother about return value
-		 */
-		if (ret != 0) {
-			saved_errno = errno;
-		}
-		goto fail;
-	}
-
-	DEBUG(10, ("Fallback to xattr\n"));
-	if (strcmp(name, XATTR_NTACL_NAME) == 0) {
-		ret = SMB_VFS_NEXT_SETXATTR(handle, smb_fname,
-					    XATTR_USER_NTACL,
-					    value, size, flags);
-		if (ret != 0) {
-			saved_errno = errno;
-			goto fail;
-		}
-		return 0;
-	}
-
-	/* Clients can't set XATTR_USER_NTACL directly. */
-	if (strcasecmp(name, XATTR_USER_NTACL) == 0) {
-		saved_errno = EACCES;
-		ret = -1;
-		goto fail;
-	}
-
-	ret = SMB_VFS_NEXT_SETXATTR(handle, smb_fname,
-				    name, value, size, flags);
-	if (ret != 0) {
-		saved_errno = errno;
-		goto fail;
-	}
-
-fail:
-	TALLOC_FREE(smb_fname);
-	if (saved_errno != 0) {
-		saved_errno = errno;
-	}
-	return ret;
+	return SMB_VFS_NEXT_SYS_ACL_SET_FD(handle, fsp, type, theacl);
 }
+#endif
 
 static int vxfs_fset_xattr(struct vfs_handle_struct *handle,
 			   struct files_struct *fsp, const char *name,
 			   const void *value, size_t size,  int flags){
 	int ret = 0;
+	int tmp_ret = 0;
 
-	DEBUG(10, ("In vxfs_fset_xattr\n"));
+	DBG_DEBUG("In vxfs_fset_xattr\n");
 
-	ret = vxfs_setxattr_fd(fsp->fh->fd, name, value, size, flags);
+	ret = vxfs_setxattr_fd(fsp_get_io_fd(fsp), name, value, size, flags);
 	if ((ret == 0) ||
 	    ((ret == -1) && (errno != ENOTSUP) && (errno != ENOSYS))) {
-		SMB_VFS_NEXT_FREMOVEXATTR(handle, fsp, name);
+		/*
+		 * version 1: user.NTACL xattr without inheritance supported
+		 * version 2: user.NTACL xattr with inheritance supported
+		 * version 3: new styled xattr security.NTACL with inheritance supported
+		 * Hence, the old styled xattr user.NTACL should be removed
+		 */
+		tmp_ret = vxfs_strcasecmp(name, XATTR_NTACL_NAME);
+		if (tmp_ret == 0) {
+			SMB_VFS_NEXT_FREMOVEXATTR(handle, fsp, XATTR_USER_NTACL_V0);
+			DBG_DEBUG("Old style xattr %s removed...\n", XATTR_USER_NTACL_V0);
+		}
+
 		return ret;
 	}
 
-	DEBUG(10, ("Fallback to xattr"));
+	DBG_DEBUG("Fallback to xattr");
 	if (strcmp(name, XATTR_NTACL_NAME) == 0) {
 		return SMB_VFS_NEXT_FSETXATTR(handle, fsp, XATTR_USER_NTACL,
 					      value, size, flags);
 	}
 
 	/* Clients can't set XATTR_USER_NTACL directly. */
-	if (strcasecmp(name, XATTR_USER_NTACL) == 0) {
+	if (vxfs_strcasecmp(name, XATTR_USER_NTACL) == 0) {
 		errno = EACCES;
 		return -1;
 	}
@@ -616,35 +557,6 @@ static int vxfs_fset_xattr(struct vfs_handle_struct *handle,
 	return SMB_VFS_NEXT_FSETXATTR(handle, fsp, name, value, size, flags);
 }
 
-static ssize_t vxfs_get_xattr(struct vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				const char *name,
-				void *value,
-				size_t size){
-	int ret;
-
-	DEBUG(10, ("In vxfs_get_xattr\n"));
-	ret = vxfs_getxattr_path(smb_fname->base_name, name, value, size);
-	if ((ret != -1) || ((errno != ENOTSUP) &&
-			    (errno != ENOSYS) && (errno != ENODATA))) {
-		return ret;
-	}
-
-	DEBUG(10, ("Fallback to xattr\n"));
-	if (strcmp(name, XATTR_NTACL_NAME) == 0) {
-		return SMB_VFS_NEXT_GETXATTR(handle, smb_fname,
-				XATTR_USER_NTACL, value, size);
-	}
-
-	/* Clients can't see XATTR_USER_NTACL directly. */
-	if (strcasecmp(name, XATTR_USER_NTACL) == 0) {
-		errno = ENOATTR;
-		return -1;
-	}
-
-	return SMB_VFS_NEXT_GETXATTR(handle, smb_fname, name, value, size);
-}
-
 static ssize_t vxfs_fget_xattr(struct vfs_handle_struct *handle,
 			       struct files_struct *fsp, const char *name,
 			       void *value, size_t size){
@@ -652,7 +564,7 @@ static ssize_t vxfs_fget_xattr(struct vfs_handle_struct *handle,
 
 	DEBUG(10, ("In vxfs_fget_xattr\n"));
 
-	ret = vxfs_getxattr_fd(fsp->fh->fd, name, value, size);
+	ret = vxfs_getxattr_fd(fsp_get_io_fd(fsp), name, value, size);
 	if ((ret != -1) || ((errno != ENOTSUP) &&
 			    (errno != ENOSYS) && (errno != ENODATA))) {
 		return ret;
@@ -665,7 +577,7 @@ static ssize_t vxfs_fget_xattr(struct vfs_handle_struct *handle,
 	}
 
 	/* Clients can't see XATTR_USER_NTACL directly. */
-	if (strcasecmp(name, XATTR_USER_NTACL) == 0) {
+	if (vxfs_strcasecmp(name, XATTR_USER_NTACL) == 0) {
 		errno = ENOATTR;
 		return -1;
 	}
@@ -673,57 +585,6 @@ static ssize_t vxfs_fget_xattr(struct vfs_handle_struct *handle,
 	return SMB_VFS_NEXT_FGETXATTR(handle, fsp, name, value, size);
 }
 
-static int vxfs_remove_xattr(struct vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname_in,
-				const char *name)
-{
-	bool is_dir = false;
-	int ret = 0, ret_new = 0, old_errno;
-	struct smb_filename *smb_fname = NULL;
-
-	DEBUG(10, ("In vxfs_remove_xattr\n"));
-
-	smb_fname = cp_smb_filename_nostream(talloc_tos(), smb_fname_in);
-	if (smb_fname == NULL) {
-		errno = ENOMEM;
-		return -1;
-	}
-
-	/* Remove with old way */
-	if (strcmp(name, XATTR_NTACL_NAME) == 0) {
-		ret = SMB_VFS_NEXT_REMOVEXATTR(handle, smb_fname,
-					       XATTR_USER_NTACL);
-	} else {
-		if (strcasecmp(name, XATTR_USER_NTACL) != 0) {
-			ret = SMB_VFS_NEXT_REMOVEXATTR(handle, smb_fname,
-						       name);
-		}
-	}
-	/* Remove with new way */
-	old_errno = errno;
-
-	if (SMB_VFS_NEXT_STAT(handle, smb_fname) != 0) {
-		TALLOC_FREE(smb_fname);
-		return -1;
-	}
-
-	is_dir = S_ISDIR(smb_fname->st.st_ex_mode);
-	TALLOC_FREE(smb_fname);
-	/*
-	 * If both fail, return failuer else return whichever succeeded
-	 */
-	ret_new = vxfs_removexattr_path(smb_fname_in->base_name, name, is_dir);
-	if (errno == ENOTSUP || errno == ENOSYS) {
-		errno = old_errno;
-	}
-	if ((ret_new != -1) && (ret == -1)) {
-		ret = ret_new;
-	}
-
-	return ret;
-
-}
-
 static int vxfs_fremove_xattr(struct vfs_handle_struct *handle,
 			      struct files_struct *fsp, const char *name){
 	int ret = 0, ret_new = 0, old_errno;
@@ -736,7 +597,7 @@ static int vxfs_fremove_xattr(struct vfs_handle_struct *handle,
 						XATTR_USER_NTACL);
 	} else {
 		/* Clients can't remove XATTR_USER_NTACL directly. */
-		if (strcasecmp(name, XATTR_USER_NTACL) != 0) {
+		if (vxfs_strcasecmp(name, XATTR_USER_NTACL) != 0) {
 			ret = SMB_VFS_NEXT_FREMOVEXATTR(handle, fsp,
 							name);
 		}
@@ -744,9 +605,9 @@ static int vxfs_fremove_xattr(struct vfs_handle_struct *handle,
 	old_errno = errno;
 
 	/* Remove with new way */
-	ret_new = vxfs_removexattr_fd(fsp->fh->fd, name);
+	ret_new = vxfs_removexattr_fd(fsp_get_io_fd(fsp), name);
 	/*
-	 * If both fail, return failuer else return whichever succeeded
+	 * If both fail, return failure else return whichever succeeded
 	 */
 	if (errno == ENOTSUP || errno == ENOSYS) {
 		errno = old_errno;
@@ -765,7 +626,7 @@ static size_t vxfs_filter_list(char *list, size_t size)
 
 	while (str - list < size) {
 		size_t element_len = strlen(str) + 1;
-		if (strcasecmp(str, XATTR_USER_NTACL) == 0) {
+		if (vxfs_strcasecmp(str, XATTR_USER_NTACL) == 0) {
 			memmove(str,
 				str + element_len,
 				size - (str - list) - element_len);
@@ -777,37 +638,13 @@ static size_t vxfs_filter_list(char *list, size_t size)
 	return size;
 }
 
-static ssize_t vxfs_listxattr(vfs_handle_struct *handle,
-				const struct smb_filename *smb_fname,
-				char *list,
-				size_t size)
-{
-	ssize_t result;
-
-	result = vxfs_listxattr_path(smb_fname->base_name, list, size);
-	if (result >= 0 || ((errno != ENOTSUP) && (errno != ENOSYS))) {
-		return result;
-	}
-
-	result = SMB_VFS_NEXT_LISTXATTR(handle, smb_fname, list, size);
-
-	if (result <= 0) {
-		return result;
-	}
-
-	/* Remove any XATTR_USER_NTACL elements from the returned list. */
-	result = vxfs_filter_list(list, result);
-
-        return result;
-}
-
 static ssize_t vxfs_flistxattr(struct vfs_handle_struct *handle,
                                 struct files_struct *fsp, char *list,
                                 size_t size)
 {
 	ssize_t result;
 
-	result = vxfs_listxattr_fd(fsp->fh->fd, list, size);
+	result = vxfs_listxattr_fd(fsp_get_io_fd(fsp), list, size);
 	if (result >= 0 || ((errno != ENOTSUP) && (errno != ENOSYS))) {
 		return result;
 	}
@@ -824,64 +661,22 @@ static ssize_t vxfs_flistxattr(struct vfs_handle_struct *handle,
         return result;
 }
 
-static NTSTATUS vxfs_set_ea_dos_attributes(struct vfs_handle_struct *handle,
-					   const struct smb_filename *smb_fname,
-					   uint32_t dosmode)
-{
-	NTSTATUS	err;
-	int			ret = 0;
-	bool		attrset = false;
-	bool		is_dir = false;
-
-	DBG_DEBUG("Entered function\n");
-
-	is_dir = S_ISDIR(smb_fname->st.st_ex_mode);
-	if (!(dosmode & FILE_ATTRIBUTE_READONLY)) {
-		ret = vxfs_checkwxattr_path(smb_fname->base_name);
-		if (ret == -1) {
-			DBG_DEBUG("ret:%d\n", ret);
-			if ((errno != EOPNOTSUPP) && (errno != ENOENT)) {
-				return map_nt_error_from_unix(errno);
-			}
-		}
-	}
-	if (dosmode & FILE_ATTRIBUTE_READONLY) {
-		ret = vxfs_setwxattr_path(smb_fname->base_name, is_dir);
-		DBG_DEBUG("ret:%d\n", ret);
-		if (ret == -1) {
-			if ((errno != EOPNOTSUPP) && (errno != EINVAL)) {
-				return map_nt_error_from_unix(errno);
-			}
-		} else {
-			attrset = true;
-		}
-	}
-	err = SMB_VFS_NEXT_SET_DOS_ATTRIBUTES(handle, smb_fname, dosmode);
-	if (!NT_STATUS_IS_OK(err)) {
-		if (attrset) {
-			ret = vxfs_clearwxattr_path(smb_fname->base_name, is_dir);
-			DBG_DEBUG("ret:%d\n", ret);
-			if ((ret == -1) && (errno != ENOENT)) {
-				return map_nt_error_from_unix(errno);
-			}
-		}
-	}
-
-	return err;
-}
-
 static NTSTATUS vxfs_fset_ea_dos_attributes(struct vfs_handle_struct *handle,
 					    struct files_struct *fsp,
 					    uint32_t dosmode)
 {
 	NTSTATUS	err;
 	int		ret = 0;
-	bool		attrset = false;
 
 	DBG_DEBUG("Entered function\n");
 
+	err = SMB_VFS_NEXT_FSET_DOS_ATTRIBUTES(handle, fsp, dosmode);
+	if (!NT_STATUS_IS_OK(err)) {
+		DBG_DEBUG("err:%d\n", err);
+		return err;
+	}
 	if (!(dosmode & FILE_ATTRIBUTE_READONLY)) {
-		ret = vxfs_checkwxattr_fd(fsp->fh->fd);
+		ret = vxfs_checkwxattr_fd(fsp_get_io_fd(fsp));
 		if (ret == -1) {
 			DBG_DEBUG("ret:%d\n", ret);
 			if ((errno != EOPNOTSUPP) && (errno != ENOENT)) {
@@ -890,27 +685,15 @@ static NTSTATUS vxfs_fset_ea_dos_attributes(struct vfs_handle_struct *handle,
 		}
 	}
 	if (dosmode & FILE_ATTRIBUTE_READONLY) {
-		ret = vxfs_setwxattr_fd(fsp->fh->fd);
+		ret = vxfs_setwxattr_fd(fsp_get_io_fd(fsp));
 		DBG_DEBUG("ret:%d\n", ret);
 		if (ret == -1) {
 			if ((errno != EOPNOTSUPP) && (errno != EINVAL)) {
 				return map_nt_error_from_unix(errno);
 			}
-		} else {
-			attrset = true;
 		}
 	}
-	err = SMB_VFS_NEXT_FSET_DOS_ATTRIBUTES(handle, fsp, dosmode);
-	if (!NT_STATUS_IS_OK(err)) {
-		if (attrset) {
-			ret = vxfs_clearwxattr_fd(fsp->fh->fd);
-			DBG_DEBUG("ret:%d\n", ret);
-			if ((ret == -1) && (errno != ENOENT)) {
-				return map_nt_error_from_unix(errno);
-			}
-		}
-	}
-	return err;
+	return NT_STATUS_OK;
 }
 
 static int vfs_vxfs_connect(struct vfs_handle_struct *handle,
@@ -933,21 +716,15 @@ static struct vfs_fn_pointers vfs_vxfs_fns = {
 	.connect_fn = vfs_vxfs_connect,
 
 #ifdef VXFS_ACL_SHARE
-	.sys_acl_set_file_fn = vxfs_sys_acl_set_file,
 	.sys_acl_set_fd_fn = vxfs_sys_acl_set_fd,
 #endif
 
-	.set_dos_attributes_fn = vxfs_set_ea_dos_attributes,
 	.fset_dos_attributes_fn = vxfs_fset_ea_dos_attributes,
-	.getxattr_fn = vxfs_get_xattr,
 	.getxattrat_send_fn = vfs_not_implemented_getxattrat_send,
 	.getxattrat_recv_fn = vfs_not_implemented_getxattrat_recv,
 	.fgetxattr_fn = vxfs_fget_xattr,
-	.listxattr_fn = vxfs_listxattr,
 	.flistxattr_fn = vxfs_flistxattr,
-	.removexattr_fn = vxfs_remove_xattr,
 	.fremovexattr_fn = vxfs_fremove_xattr,
-	.setxattr_fn = vxfs_set_xattr,
 	.fsetxattr_fn = vxfs_fset_xattr,
 };
 
