@@ -24,6 +24,8 @@
 #include "../lib/util/select.h"
 #include "system/select.h"
 #include "libsmb/libsmb.h"
+#include "libsmb/unexpected.h"
+#include "lib/util/string_wrappers.h"
 
 extern int ClientNMB;
 extern int ClientDGRAM;
@@ -1683,16 +1685,16 @@ on subnet %s\n", rrec->response_id, inet_ntoa(rrec->packet->ip), subrec->subnet_
 struct socket_attributes {
 	enum packet_type type;
 	bool broadcast;
+	int fd;
+	bool triggered;
 };
 
-static bool create_listen_pollfds(struct pollfd **pfds,
-				  struct socket_attributes **pattrs,
+static bool create_listen_array(struct socket_attributes **pattrs,
 				  int *pnum_sockets)
 {
 	struct subnet_record *subrec = NULL;
 	int count = 0;
 	int num = 0;
-	struct pollfd *fds;
 	struct socket_attributes *attrs;
 
 	/* The ClientNMB and ClientDGRAM sockets */
@@ -1716,29 +1718,21 @@ static bool create_listen_pollfds(struct pollfd **pfds,
 		}
 	}
 
-	fds = talloc_zero_array(NULL, struct pollfd, count);
-	if (fds == NULL) {
-		DEBUG(1, ("create_listen_pollfds: malloc fail for fds. "
+	attrs = talloc_zero_array(NULL, struct socket_attributes, count);
+	if (attrs == NULL) {
+		DEBUG(1, ("talloc fail for attrs. "
 			  "size %d\n", count));
 		return true;
 	}
 
-	attrs = talloc_array(NULL, struct socket_attributes, count);
-	if (fds == NULL) {
-		DEBUG(1, ("create_listen_pollfds: malloc fail for attrs. "
-			  "size %d\n", count));
-		TALLOC_FREE(fds);
-		return true;
-	}
-
 	num = 0;
 
-	fds[num].fd = ClientNMB;
+	attrs[num].fd = ClientNMB;
 	attrs[num].type = NMB_PACKET;
 	attrs[num].broadcast = false;
 	num += 1;
 
-	fds[num].fd = ClientDGRAM;
+	attrs[num].fd = ClientDGRAM;
 	attrs[num].type = DGRAM_PACKET;
 	attrs[num].broadcast = false;
 	num += 1;
@@ -1746,37 +1740,34 @@ static bool create_listen_pollfds(struct pollfd **pfds,
 	for (subrec = FIRST_SUBNET; subrec; subrec = NEXT_SUBNET_EXCLUDING_UNICAST(subrec)) {
 
 		if (subrec->nmb_sock != -1) {
-			fds[num].fd = subrec->nmb_sock;
+			attrs[num].fd = subrec->nmb_sock;
 			attrs[num].type = NMB_PACKET;
 			attrs[num].broadcast = false;
 			num += 1;
 		}
 
 		if (subrec->nmb_bcast != -1) {
-			fds[num].fd = subrec->nmb_bcast;
+			attrs[num].fd = subrec->nmb_bcast;
 			attrs[num].type = NMB_PACKET;
 			attrs[num].broadcast = true;
 			num += 1;
 		}
 
 		if (subrec->dgram_sock != -1) {
-			fds[num].fd = subrec->dgram_sock;
+			attrs[num].fd = subrec->dgram_sock;
 			attrs[num].type = DGRAM_PACKET;
 			attrs[num].broadcast = false;
 			num += 1;
 		}
 
 		if (subrec->dgram_bcast != -1) {
-			fds[num].fd = subrec->dgram_bcast;
+			attrs[num].fd = subrec->dgram_bcast;
 			attrs[num].type = DGRAM_PACKET;
 			attrs[num].broadcast = true;
 			num += 1;
 		}
 	}
 
-	TALLOC_FREE(*pfds);
-	*pfds = fds;
-
 	TALLOC_FREE(*pattrs);
 	*pattrs = attrs;
 
@@ -1863,6 +1854,110 @@ static void free_processed_packet_list(struct processed_packet **pp_processed_pa
 	}
 }
 
+/****************************************************************************
+ Timeout callback - just notice we timed out.
+***************************************************************************/
+
+static void nmbd_timeout_handler(struct tevent_context *ev,
+			struct tevent_timer *te,
+			struct timeval current_time,
+			void *private_data)
+{
+	bool *got_timeout = private_data;
+	*got_timeout = true;
+}
+
+/****************************************************************************
+ fd callback - remember the fd that triggered.
+***************************************************************************/
+
+static void nmbd_fd_handler(struct tevent_context *ev,
+				struct tevent_fd *fde,
+				uint16_t flags,
+				void *private_data)
+{
+	struct socket_attributes *attr = private_data;
+	attr->triggered = true;
+}
+
+/****************************************************************************
+ Read from a socket.
+****************************************************************************/
+
+static ssize_t read_udp_v4_socket(
+	int fd,
+	char *buf,
+	size_t len,
+	struct sockaddr_storage *psa)
+{
+	ssize_t ret;
+	socklen_t socklen = sizeof(*psa);
+	struct sockaddr_in *si = (struct sockaddr_in *)psa;
+
+	memset((char *)psa,'\0',socklen);
+
+	ret = (ssize_t)sys_recvfrom(fd,buf,len,0,
+			(struct sockaddr *)psa,&socklen);
+	if (ret <= 0) {
+		/* Don't print a low debug error for a non-blocking socket. */
+		if (errno == EAGAIN) {
+			DEBUG(10,("read_udp_v4_socket: returned EAGAIN\n"));
+		} else {
+			DEBUG(2,("read_udp_v4_socket: failed. errno=%s\n",
+				strerror(errno)));
+		}
+		return 0;
+	}
+
+	if (psa->ss_family != AF_INET) {
+		DEBUG(2,("read_udp_v4_socket: invalid address family %d "
+			"(not IPv4)\n", (int)psa->ss_family));
+		return 0;
+	}
+
+	DEBUG(10,("read_udp_v4_socket: ip %s port %d read: %lu\n",
+			inet_ntoa(si->sin_addr),
+			si->sin_port,
+			(unsigned long)ret));
+
+	return ret;
+}
+
+/*******************************************************************
+ Read a packet from a socket and parse it, returning a packet ready
+ to be used or put on the queue. This assumes a UDP socket.
+******************************************************************/
+
+static struct packet_struct *read_packet(int fd,enum packet_type packet_type)
+{
+	struct packet_struct *packet;
+	struct sockaddr_storage sa;
+	struct sockaddr_in *si = (struct sockaddr_in *)&sa;
+	char buf[MAX_DGRAM_SIZE];
+	int length;
+
+	length = read_udp_v4_socket(fd,buf,sizeof(buf),&sa);
+	if (length < MIN_DGRAM_SIZE || sa.ss_family != AF_INET) {
+		return NULL;
+	}
+
+	packet = parse_packet(buf,
+			length,
+			packet_type,
+			si->sin_addr,
+			ntohs(si->sin_port));
+	if (!packet)
+		return NULL;
+
+	packet->recv_fd = fd;
+	packet->send_fd = -1;
+
+	DEBUG(5,("Received a packet of len %d from (%s) port %d\n",
+		 length, inet_ntoa(packet->ip), packet->port ) );
+
+	return(packet);
+}
+
 /****************************************************************************
   Listens for NMB or DGRAM packets, and queues them.
   return True if the socket is dead
@@ -1870,60 +1965,68 @@ static void free_processed_packet_list(struct processed_packet **pp_processed_pa
 
 bool listen_for_packets(struct messaging_context *msg, bool run_election)
 {
-	static struct pollfd *fds = NULL;
 	static struct socket_attributes *attrs = NULL;
 	static int listen_number = 0;
 	int num_sockets;
 	int i;
+	int loop_rtn;
+	int timeout_secs;
 
-	int pollrtn;
-	int timeout;
 #ifndef SYNC_DNS
 	int dns_fd;
 	int dns_pollidx = -1;
 #endif
 	struct processed_packet *processed_packet_list = NULL;
+	struct tevent_timer *te = NULL;
+	bool got_timeout = false;
+	TALLOC_CTX *frame = talloc_stackframe();
 
-	if ((fds == NULL) || rescan_listen_set) {
-		if (create_listen_pollfds(&fds, &attrs, &listen_number)) {
+	if ((attrs == NULL) || rescan_listen_set) {
+		if (create_listen_array(&attrs, &listen_number)) {
 			DEBUG(0,("listen_for_packets: Fatal error. unable to create listen set. Exiting.\n"));
+			TALLOC_FREE(frame);
 			return True;
 		}
 		rescan_listen_set = False;
 	}
 
-	/*
-	 * "fds" can be enlarged by event_add_to_poll_args
-	 * below. Shrink it again to what was given to us by
-	 * create_listen_pollfds.
-	 */
-
-	fds = talloc_realloc(NULL, fds, struct pollfd, listen_number);
-	if (fds == NULL) {
-		return true;
-	}
 	num_sockets = listen_number;
 
 #ifndef SYNC_DNS
 	dns_fd = asyncdns_fd();
 	if (dns_fd != -1) {
-		fds = talloc_realloc(NULL, fds, struct pollfd, num_sockets+1);
-		if (fds == NULL) {
+		attrs = talloc_realloc(NULL,
+					attrs,
+					struct socket_attributes,
+					num_sockets + 1);
+		if (attrs == NULL) {
+			TALLOC_FREE(frame);
 			return true;
 		}
 		dns_pollidx = num_sockets;
-		fds[num_sockets].fd = dns_fd;
+		attrs[dns_pollidx].fd = dns_fd;
+		/*
+		 * dummy values, we only need
+		 * fd and triggered.
+		 */
+		attrs[dns_pollidx].type = NMB_PACKET;
+		attrs[dns_pollidx].broadcast = false;
 		num_sockets += 1;
 	}
 #endif
 
 	for (i=0; i<num_sockets; i++) {
-		fds[i].events = POLLIN|POLLHUP;
-	}
-
-	/* Process a signal and timer events now... */
-	if (run_events_poll(nmbd_event_context(), 0, NULL, 0)) {
-		return False;
+		struct tevent_fd *tfd = tevent_add_fd(nmbd_event_context(),
+							frame,
+							attrs[i].fd,
+							TEVENT_FD_READ,
+							nmbd_fd_handler,
+							&attrs[i]);
+		if (tfd == NULL) {
+			TALLOC_FREE(frame);
+			return true;
+		}
+		attrs[i].triggered = false;
 	}
 
 	/*
@@ -1933,26 +2036,40 @@ bool listen_for_packets(struct messaging_context *msg, bool run_election)
 	 * the time we are expecting the next netbios packet.
 	 */
 
-	timeout = ((run_election||num_response_packets)
-		   ? 1 : NMBD_SELECT_LOOP) * 1000;
-
-	event_add_to_poll_args(nmbd_event_context(), NULL,
-			       &fds, &num_sockets, &timeout);
-
-	pollrtn = poll(fds, num_sockets, timeout);
-
-	if (run_events_poll(nmbd_event_context(), pollrtn, fds, num_sockets)) {
-		return False;
+	if (run_election||num_response_packets) {
+		timeout_secs = 1;
+	} else {
+		timeout_secs = NMBD_SELECT_LOOP;
 	}
 
-	if (pollrtn == -1) {
-		return False;
+	te = tevent_add_timer(nmbd_event_context(),
+				frame,
+				tevent_timeval_current_ofs(timeout_secs, 0),
+				nmbd_timeout_handler,
+				&got_timeout);
+	if (te == NULL) {
+		TALLOC_FREE(frame);
+		return true;
+	}
+
+	loop_rtn = tevent_loop_once(nmbd_event_context());
+
+	if (loop_rtn == -1) {
+		TALLOC_FREE(frame);
+		return true;
+	}
+
+	if (got_timeout) {
+		TALLOC_FREE(frame);
+		return false;
 	}
 
 #ifndef SYNC_DNS
 	if ((dns_fd != -1) && (dns_pollidx != -1) &&
-	    (fds[dns_pollidx].revents & (POLLIN|POLLHUP|POLLERR))) {
+	    attrs[dns_pollidx].triggered){
 		run_dns_queue(msg);
+		TALLOC_FREE(frame);
+		return false;
 	}
 #endif
 
@@ -1963,7 +2080,7 @@ bool listen_for_packets(struct messaging_context *msg, bool run_election)
 		int client_fd;
 		int client_port;
 
-		if ((fds[i].revents & (POLLIN|POLLHUP|POLLERR)) == 0) {
+		if (!attrs[i].triggered) {
 			continue;
 		}
 
@@ -1981,7 +2098,7 @@ bool listen_for_packets(struct messaging_context *msg, bool run_election)
 			client_port = DGRAM_PORT;
 		}
 
-		packet = read_packet(fds[i].fd, packet_type);
+		packet = read_packet(attrs[i].fd, packet_type);
 		if (!packet) {
 			continue;
 		}
@@ -1991,7 +2108,7 @@ bool listen_for_packets(struct messaging_context *msg, bool run_election)
 		 * only is set then check it came from one of our local nets.
 		 */
 		if (lp_bind_interfaces_only() &&
-		    (fds[i].fd == client_fd) &&
+		    (attrs[i].fd == client_fd) &&
 		    (!is_local_net_v4(packet->ip))) {
 			DEBUG(7,("discarding %s packet sent to broadcast socket from %s:%d\n",
 				packet_name, inet_ntoa(packet->ip), packet->port));
@@ -2030,16 +2147,17 @@ bool listen_for_packets(struct messaging_context *msg, bool run_election)
 
 		if (attrs[i].broadcast) {
 			/* this is a broadcast socket */
-			packet->send_fd = fds[i-1].fd;
+			packet->send_fd = attrs[i-1].fd;
 		} else {
 			/* this is already a unicast socket */
-			packet->send_fd = fds[i].fd;
+			packet->send_fd = attrs[i].fd;
 		}
 
 		queue_packet(packet);
 	}
 
 	free_processed_packet_list(&processed_packet_list);
+	TALLOC_FREE(frame);
 	return False;
 }
 
