@@ -27,6 +27,7 @@
 #include "../librpc/gen_ndr/samr.h"
 #include "../libcli/security/security.h"
 #include "passdb/pdb_smbpasswd.h"
+#include "lib/util/string_wrappers.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_PASSDB
@@ -166,7 +167,7 @@ static bool pw_file_unlock(int fd, int *plock_depth)
 }
 
 /**************************************************************
- Intialize a smb_passwd struct
+ Initialize a smb_passwd struct
  *************************************************************/
 
 static void pdb_init_smb(struct smb_passwd *user)
@@ -191,6 +192,7 @@ static FILE *startsmbfilepwent(const char *pfile, enum pwf_access_type type, int
 	const char *open_mode = NULL;
 	int race_loop = 0;
 	int lock_type = F_RDLCK;
+	struct stat st;
 
 	if (!*pfile) {
 		DEBUG(0, ("startsmbfilepwent: No SMB password file set\n"));
@@ -282,7 +284,7 @@ Error was %s\n", pfile, strerror(errno) ));
 			 * by doing a stat on the filename and an fstat on the fd. If the
 			 * two inodes differ then someone did a rename between the open and
 			 * the lock. Back off and try the open again. Only do this 5 times to
-			 * prevent infinate loops. JRA.
+			 * prevent infinite loops. JRA.
 			 */
 
 			if (sys_stat(pfile, &sbuf1, false) != 0) {
@@ -323,19 +325,38 @@ Error was %s\n", pfile, strerror(errno)));
 	/* Set a buffer to do more efficient reads */
 	setvbuf(fp, (char *)NULL, _IOFBF, 1024);
 
-	/* Make sure it is only rw by the owner */
-#ifdef HAVE_FCHMOD
-	if(fchmod(fileno(fp), S_IRUSR|S_IWUSR) == -1) {
-#else
-	if(chmod(pfile, S_IRUSR|S_IWUSR) == -1) {
-#endif
-		DEBUG(0, ("startsmbfilepwent_internal: failed to set 0600 permissions on password file %s. \
-Error was %s\n.", pfile, strerror(errno) ));
+	/* Ensure we have a valid stat. */
+	if (fstat(fileno(fp), &st) != 0) {
+		DBG_ERR("Unable to fstat file %s. Error was %s\n",
+			pfile,
+			strerror(errno));
 		pw_file_unlock(fileno(fp), lock_depth);
 		fclose(fp);
 		return NULL;
 	}
 
+	/* If file has invalid permissions != 0600, then [f]chmod(). */
+	if ((st.st_mode & 0777) != (S_IRUSR|S_IWUSR)) {
+		DBG_WARNING("file %s has invalid permissions 0%o should "
+			    "be 0600.\n",
+			    pfile,
+			    (unsigned int)st.st_mode & 0777);
+		/* Make sure it is only rw by the owner */
+#ifdef HAVE_FCHMOD
+		if (fchmod(fileno(fp), S_IRUSR|S_IWUSR) == -1) {
+#else
+		if (chmod(pfile, S_IRUSR|S_IWUSR) == -1) {
+#endif
+			DBG_ERR("Failed to set 0600 permissions on password file %s. "
+				"Error was %s\n.",
+				pfile,
+				strerror(errno));
+			pw_file_unlock(fileno(fp), lock_depth);
+			fclose(fp);
+			return NULL;
+		}
+	}
+
 	/* We have a lock on the file. */
 	return fp;
 }
@@ -1202,7 +1223,7 @@ static bool build_smb_pass (struct smb_passwd *smb_pw, const struct samu *sampas
 	if (!IS_SAM_DEFAULT(sampass, PDB_USERSID)) {
 		rid = pdb_get_user_rid(sampass);
 
-		/* If the user specified a RID, make sure its able to be both stored and retreived */
+		/* If the user specified a RID, make sure its able to be both stored and retrieved */
 		if (rid == DOMAIN_RID_GUEST) {
 			struct passwd *passwd = Get_Pwnam_alloc(NULL, lp_guest_account());
 			if (!passwd) {
