@@ -288,6 +288,9 @@ static bool get_oldest_entry_hook( EVENTLOG_INFO * info )
 
 static NTSTATUS elog_open( struct pipes_struct * p, const char *logname, struct policy_handle *hnd )
 {
+	struct dcesrv_call_state *dce_call = p->dce_call;
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	EVENTLOG_INFO *elog;
 
 	/* first thing is to validate the eventlog name */
@@ -321,7 +324,7 @@ static NTSTATUS elog_open( struct pipes_struct * p, const char *logname, struct
 			elog->logname = talloc_strdup( elog, ELOG_APPL );
 
 			/* do the access check */
-			if ( !elog_check_access( elog, p->session_info) ) {
+			if ( !elog_check_access( elog, session_info) ) {
 				TALLOC_FREE( elog );
 				return NT_STATUS_ACCESS_DENIED;
 			}
@@ -339,7 +342,7 @@ static NTSTATUS elog_open( struct pipes_struct * p, const char *logname, struct
 
 	/* now do the access check.  Close the tdb if we fail here */
 
-	if ( !elog_check_access( elog, p->session_info) ) {
+	if ( !elog_check_access( elog, session_info) ) {
 		TALLOC_FREE( elog );
 		return NT_STATUS_ACCESS_DENIED;
 	}
@@ -420,7 +423,7 @@ static bool sync_eventlog_params(TALLOC_CTX *mem_ctx,
 		DEBUG( 4, ( "No open tdb! (%s)\n", info->logname ) );
 		goto done;
 	}
-	/* set resonable defaults.  512Kb on size and 1 week on time */
+	/* set reasonable defaults.  512Kb on size and 1 week on time */
 
 	uiMaxSize = 0x80000;
 	uiRetention = 604800;
@@ -628,7 +631,6 @@ NTSTATUS _eventlog_ReadEventLogW(struct pipes_struct *p,
 				 struct eventlog_ReadEventLogW *r)
 {
 	EVENTLOG_INFO *info = find_eventlog_info_by_hnd( p, r->in.handle );
-	uint32_t num_records_read = 0;
 	int bytes_left, record_number;
 	uint32_t elog_read_type, elog_read_dir;
 
@@ -723,8 +725,6 @@ NTSTATUS _eventlog_ReadEventLogW(struct pipes_struct *p,
 		memcpy(&r->out.data[*(r->out.sent_size)],
 		       blob.data, blob.length);
 		*(r->out.sent_size) += blob.length;
-
-		num_records_read++;
 	}
 
 	if (r->in.offset == 0 && record_number == 0 && *r->out.sent_size == 0) {
@@ -1033,14 +1033,8 @@ static NTSTATUS eventlog_init_server(struct dcesrv_context *dce_ctx,
 		const struct dcesrv_endpoint_server *ep_server)
 {
 	struct messaging_context *msg_ctx = global_messaging_context();
-	NTSTATUS status;
 	bool ok;
 
-	status = dcesrv_init_ep_server(dce_ctx, "winreg");
-	if (!NT_STATUS_IS_OK(status)) {
-		return status;
-	}
-
 	ok = eventlog_init_winreg(msg_ctx);
 	if (!ok) {
 		return NT_STATUS_UNSUCCESSFUL;
