@@ -44,6 +44,8 @@
 #include "serverid.h"
 #include "lib/global_contexts.h"
 #include "source3/lib/substitute.h"
+#include "lib/tsocket/tsocket.h"
+#include "librpc/rpc/dcesrv_core.h"
 
 extern const struct generic_mapping file_generic_mapping;
 
@@ -626,6 +628,9 @@ static WERROR init_srv_share_info_ctr(struct pipes_struct *p,
 	struct dcesrv_call_state *dce_call = p->dce_call;
 	struct auth_session_info *session_info =
 		dcesrv_call_session_info(dce_call);
+	struct dcesrv_connection *dcesrv_conn = dce_call->conn;
+	const struct tsocket_address *local_address =
+		dcesrv_connection_get_local_address(dcesrv_conn);
 	const struct loadparm_substitution *lp_sub =
 		loadparm_s3_global_substitution();
 	uint32_t num_entries = 0;
@@ -678,8 +683,9 @@ static WERROR init_srv_share_info_ctr(struct pipes_struct *p,
         /* Count the number of entries. */
         for (snum = 0; snum < num_services; snum++) {
                 if (lp_browseable(snum) && lp_snum_ok(snum) &&
-                    is_enumeration_allowed(p, snum) &&
-                    (all_shares || !is_hidden_share(snum)) ) {
+                    lp_allow_local_address(snum, local_address) &&
+		    is_enumeration_allowed(p, snum) &&
+                    (all_shares || !is_hidden_share(snum))) {
                         DEBUG(10, ("counting service %s\n",
 				lp_servicename(talloc_tos(), lp_sub, snum) ? lp_servicename(talloc_tos(), lp_sub, snum) : "(null)"));
                         allowed[snum] = true;
@@ -1495,8 +1501,9 @@ WERROR _srvsvc_NetSrvGetInfo(struct pipes_struct *p,
 		info102->version_major	= SAMBA_MAJOR_NBT_ANNOUNCE_VERSION;
 		info102->version_minor	= SAMBA_MINOR_NBT_ANNOUNCE_VERSION;
 		info102->server_type	= lp_default_server_announce();
-		info102->comment	= string_truncate(lp_server_string(talloc_tos(), lp_sub),
-						MAX_SERVER_STRING_LENGTH);
+		info102->comment =
+			string_truncate(lp_server_string(info102, lp_sub),
+					MAX_SERVER_STRING_LENGTH);
 		info102->users		= 0xffffffff;
 		info102->disc		= 0xf;
 		info102->hidden		= 0;
@@ -1521,8 +1528,9 @@ WERROR _srvsvc_NetSrvGetInfo(struct pipes_struct *p,
 		info101->version_major	= SAMBA_MAJOR_NBT_ANNOUNCE_VERSION;
 		info101->version_minor	= SAMBA_MINOR_NBT_ANNOUNCE_VERSION;
 		info101->server_type	= lp_default_server_announce();
-		info101->comment	= string_truncate(lp_server_string(talloc_tos(), lp_sub),
-						MAX_SERVER_STRING_LENGTH);
+		info101->comment =
+			string_truncate(lp_server_string(info101, lp_sub),
+					MAX_SERVER_STRING_LENGTH);
 
 		r->out.info->info101 = info101;
 		break;
@@ -1987,7 +1995,7 @@ WERROR _srvsvc_NetShareSetInfo(struct pipes_struct *p,
 	case 1005:
                 /* XP re-sets the csc policy even if it wasn't changed by the
 		   user, so we must compare it to see if it's what is set in
-		   smb.conf, so that we can contine other ops like setting
+		   smb.conf, so that we can continue other ops like setting
 		   ACLs on a share */
 		client_csc_policy = (info->info1005->dfs_flags &
 				     SHARE_1005_CSC_POLICY_MASK) >>
