@@ -4,13 +4,13 @@
 #
 
 if [ $# -lt 1 ]; then
-    echo Usage: test_deadtime.sh IP
-    exit 1
+	echo Usage: test_deadtime.sh IP
+	exit 1
 fi
 
 server=$1
 
-incdir=`dirname $0`/../../../testprogs/blackbox
+incdir=$(dirname $0)/../../../testprogs/blackbox
 . $incdir/subunit.sh
 . $incdir/common_test_fns.inc
 
@@ -21,7 +21,7 @@ smbcontrol="$BINDIR/smbcontrol"
 
 global_inject_conf=$(dirname $SMB_CONF_PATH)/global_inject.conf
 
-echo "deadtime = 1" > $global_inject_conf
+echo "deadtime = 1" >$global_inject_conf
 $smbcontrol smbd reload-config
 
 cd $SELFTEST_TMPDIR || exit 1
@@ -39,12 +39,12 @@ export SAMBA_DEPRECATED_SUPPRESS=1
 trap "" SIGPIPE
 
 $smbclient //$server/tmp -U${USER}%${PASSWORD} \
-	     < smbclient-stdin > smbclient-stdout 2>smbclient-stderr &
+	<smbclient-stdin >smbclient-stdout 2>smbclient-stderr &
 client_pid=$!
 
 sleep 1
 
-exec 100>smbclient-stdin  101<smbclient-stdout 102<smbclient-stderr
+exec 100>smbclient-stdin 101<smbclient-stdout 102<smbclient-stderr
 
 # consume the smbclient startup message
 head -n 1 <&101
@@ -59,7 +59,7 @@ kill $client_pid
 echo "$err" | grep NT_STATUS_CONNECTION_DISCONNECTED
 testit "deadtime" test $? -eq 0 || failed=$(expr $failed + 1)
 
-echo "" > $global_inject_conf
+echo "" >$global_inject_conf
 $smbcontrol smbd reload-config
 
 rm -f smbclient-stdin smbclient-stdout smbclient-stderr
