@@ -189,44 +189,48 @@ mode_t unix_mode(connection_struct *conn, int dosmode,
 ****************************************************************************/
 
 static uint32_t dos_mode_from_sbuf(connection_struct *conn,
-				 const struct smb_filename *smb_fname)
+				   const struct stat_ex *st,
+				   struct files_struct *fsp)
 {
 	int result = 0;
-	enum mapreadonly_options ro_opts = (enum mapreadonly_options)lp_map_readonly(SNUM(conn));
+	enum mapreadonly_options ro_opts =
+		(enum mapreadonly_options)lp_map_readonly(SNUM(conn));
 
 #if defined(UF_IMMUTABLE) && defined(SF_IMMUTABLE)
 	/* if we can find out if a file is immutable we should report it r/o */
-	if (smb_fname->st.st_ex_flags & (UF_IMMUTABLE | SF_IMMUTABLE)) {
+	if (st->st_ex_flags & (UF_IMMUTABLE | SF_IMMUTABLE)) {
 		result |= FILE_ATTRIBUTE_READONLY;
 	}
 #endif
 	if (ro_opts == MAP_READONLY_YES) {
 		/* Original Samba method - map inverse of user "w" bit. */
-		if ((smb_fname->st.st_ex_mode & S_IWUSR) == 0) {
+		if ((st->st_ex_mode & S_IWUSR) == 0) {
 			result |= FILE_ATTRIBUTE_READONLY;
 		}
 	} else if (ro_opts == MAP_READONLY_PERMISSIONS) {
 		/* smb_fname->fsp can be NULL for an MS-DFS link. */
 		/* Check actual permissions for read-only. */
-		if (smb_fname->fsp != NULL) {
-			if (!can_write_to_fsp(smb_fname->fsp))
-			{
-				result |= FILE_ATTRIBUTE_READONLY;
-			}
+		if ((fsp != NULL) && !can_write_to_fsp(fsp)) {
+			result |= FILE_ATTRIBUTE_READONLY;
 		}
 	} /* Else never set the readonly bit. */
 
-	if (MAP_ARCHIVE(conn) && ((smb_fname->st.st_ex_mode & S_IXUSR) != 0))
+	if (MAP_ARCHIVE(conn) && ((st->st_ex_mode & S_IXUSR) != 0)) {
 		result |= FILE_ATTRIBUTE_ARCHIVE;
+	}
 
-	if (MAP_SYSTEM(conn) && ((smb_fname->st.st_ex_mode & S_IXGRP) != 0))
+	if (MAP_SYSTEM(conn) && ((st->st_ex_mode & S_IXGRP) != 0)) {
 		result |= FILE_ATTRIBUTE_SYSTEM;
+	}
 
-	if (MAP_HIDDEN(conn) && ((smb_fname->st.st_ex_mode & S_IXOTH) != 0))
+	if (MAP_HIDDEN(conn) && ((st->st_ex_mode & S_IXOTH) != 0)) {
 		result |= FILE_ATTRIBUTE_HIDDEN;
+	}
 
-	if (S_ISDIR(smb_fname->st.st_ex_mode))
-		result = FILE_ATTRIBUTE_DIRECTORY | (result & FILE_ATTRIBUTE_READONLY);
+	if (S_ISDIR(st->st_ex_mode)) {
+		result = FILE_ATTRIBUTE_DIRECTORY |
+			 (result & FILE_ATTRIBUTE_READONLY);
+	}
 
 	dos_mode_debug_print(__func__, result);
 
@@ -407,12 +411,13 @@ NTSTATUS fget_ea_dos_attribute(struct files_struct *fsp,
 ****************************************************************************/
 
 NTSTATUS set_ea_dos_attribute(connection_struct *conn,
-			      const struct smb_filename *smb_fname,
+			      struct smb_filename *smb_fname,
 			      uint32_t dosmode)
 {
-	struct xattr_DOSATTRIB dosattrib;
+	struct xattr_DOSATTRIB dosattrib = { .version = 0, };
 	enum ndr_err_code ndr_err;
-	DATA_BLOB blob;
+	DATA_BLOB blob = { .data = NULL, };
+	struct timespec btime;
 	int ret;
 
 	if (!lp_store_dos_attributes(SNUM(conn))) {
@@ -429,9 +434,6 @@ NTSTATUS set_ea_dos_attribute(connection_struct *conn,
 	 */
 	dosmode &= ~FILE_ATTRIBUTE_OFFLINE;
 
-	ZERO_STRUCT(dosattrib);
-	ZERO_STRUCT(blob);
-
 	dosattrib.version = 5;
 	dosattrib.info.info5.valid_flags = XATTR_DOSINFO_ATTRIB |
 					XATTR_DOSINFO_CREATE_TIME;
@@ -515,11 +517,11 @@ NTSTATUS set_ea_dos_attribute(connection_struct *conn,
 	 * We correctly stored the create time.
 	 * We *always* set XATTR_DOSINFO_CREATE_TIME,
 	 * so now it can no longer be considered
-	 * calculated.
+	 * calculated. Make sure to use the value rounded
+	 * to NTTIME granularity we've stored in the xattr.
 	 */
-	update_stat_ex_create_time(
-		&smb_fname->fsp->fsp_name->st,
-		smb_fname->st.st_ex_btime);
+	btime = nt_time_to_full_timespec(dosattrib.info.info5.create_time);
+	update_stat_ex_create_time(&smb_fname->st, btime);
 
 	DEBUG(10,("set_ea_dos_attribute: set EA 0x%x on file %s\n",
 		(unsigned int)dosmode,
@@ -527,46 +529,53 @@ NTSTATUS set_ea_dos_attribute(connection_struct *conn,
 	return NT_STATUS_OK;
 }
 
+static uint32_t
+dos_mode_from_name(connection_struct *conn, const char *name, uint32_t dosmode)
+{
+	const char *p = NULL;
+	uint32_t result = dosmode;
+
+	if (!(result & FILE_ATTRIBUTE_HIDDEN) &&
+	    lp_hide_dot_files(SNUM(conn)))
+	{
+		p = strrchr_m(name, '/');
+		if (p) {
+			p++;
+		} else {
+			p = name;
+		}
+
+		/* Only . and .. are not hidden. */
+		if ((p[0] == '.') && !(ISDOT(p) || ISDOTDOT(p))) {
+			result |= FILE_ATTRIBUTE_HIDDEN;
+		}
+	}
+
+	if (!(result & FILE_ATTRIBUTE_HIDDEN) && IS_HIDDEN_PATH(conn, name)) {
+		result |= FILE_ATTRIBUTE_HIDDEN;
+	}
+
+	return result;
+}
+
 /****************************************************************************
  Change a unix mode to a dos mode for an ms dfs link.
 ****************************************************************************/
 
 uint32_t dos_mode_msdfs(connection_struct *conn,
-		      const struct smb_filename *smb_fname)
+			const char *name,
+			const struct stat_ex *st)
 {
 	uint32_t result = 0;
 
-	DEBUG(8,("dos_mode_msdfs: %s\n", smb_fname_str_dbg(smb_fname)));
+	DEBUG(8, ("dos_mode_msdfs: %s\n", name));
 
-	if (!VALID_STAT(smb_fname->st)) {
+	if (!VALID_STAT(*st)) {
 		return 0;
 	}
 
-	/* First do any modifications that depend on the path name. */
-	/* hide files with a name starting with a . */
-	if (lp_hide_dot_files(SNUM(conn))) {
-		const char *p = strrchr_m(smb_fname->base_name, '/');
-		if (p) {
-			p++;
-		} else {
-			p = smb_fname->base_name;
-		}
-
-		/* Only . and .. are not hidden. */
-		if (p[0] == '.' && !((p[1] == '\0') ||
-				(p[1] == '.' && p[2] == '\0'))) {
-			result |= FILE_ATTRIBUTE_HIDDEN;
-		}
-	}
-
-	result |= dos_mode_from_sbuf(conn, smb_fname);
-
-	/* Optimization : Only call is_hidden_path if it's not already
-	   hidden. */
-	if (!(result & FILE_ATTRIBUTE_HIDDEN) &&
-	    IS_HIDDEN_PATH(conn, smb_fname->base_name)) {
-		result |= FILE_ATTRIBUTE_HIDDEN;
-	}
+	result = dos_mode_from_name(conn, name, result);
+	result |= dos_mode_from_sbuf(conn, st, NULL);
 
 	if (result == 0) {
 		result = FILE_ATTRIBUTE_NORMAL;
@@ -607,38 +616,6 @@ static NTSTATUS dos_mode_check_compressed(struct files_struct *fsp,
 	return NT_STATUS_OK;
 }
 
-static uint32_t dos_mode_from_name(connection_struct *conn,
-				   const struct smb_filename *smb_fname,
-				   uint32_t dosmode)
-{
-	const char *p = NULL;
-	uint32_t result = dosmode;
-
-	if (!(result & FILE_ATTRIBUTE_HIDDEN) &&
-	    lp_hide_dot_files(SNUM(conn)))
-	{
-		p = strrchr_m(smb_fname->base_name, '/');
-		if (p) {
-			p++;
-		} else {
-			p = smb_fname->base_name;
-		}
-
-		/* Only . and .. are not hidden. */
-		if ((p[0] == '.') && !(ISDOT(p) || ISDOTDOT(p))) {
-			result |= FILE_ATTRIBUTE_HIDDEN;
-		}
-	}
-
-	if (!(result & FILE_ATTRIBUTE_HIDDEN) &&
-	    IS_HIDDEN_PATH(conn, smb_fname->base_name))
-	{
-		result |= FILE_ATTRIBUTE_HIDDEN;
-	}
-
-	return result;
-}
-
 static uint32_t dos_mode_post(uint32_t dosmode,
 			      struct files_struct *fsp,
 			      const char *func)
@@ -677,7 +654,7 @@ static uint32_t dos_mode_post(uint32_t dosmode,
 		}
 	}
 
-	dosmode |= dos_mode_from_name(fsp->conn, smb_fname, dosmode);
+	dosmode |= dos_mode_from_name(fsp->conn, smb_fname->base_name, dosmode);
 
 	if (S_ISDIR(smb_fname->st.st_ex_mode)) {
 		dosmode |= FILE_ATTRIBUTE_DIRECTORY;
@@ -726,6 +703,10 @@ uint32_t fdos_mode(struct files_struct *fsp)
 		return FILE_ATTRIBUTE_NORMAL;
 	}
 
+	if (fsp->fsp_name->st.cached_dos_attributes != FILE_ATTRIBUTES_INVALID) {
+		return fsp->fsp_name->st.cached_dos_attributes;
+	}
+
 	/* Get the DOS attributes via the VFS if we can */
 	status = vfs_fget_dos_attributes(fsp, &result);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -733,12 +714,14 @@ uint32_t fdos_mode(struct files_struct *fsp)
 		 * Only fall back to using UNIX modes if we get NOT_IMPLEMENTED.
 		 */
 		if (NT_STATUS_EQUAL(status, NT_STATUS_NOT_IMPLEMENTED)) {
-			result |= dos_mode_from_sbuf(fsp->conn, fsp->fsp_name);
+			result |= dos_mode_from_sbuf(fsp->conn,
+						     &fsp->fsp_name->st,
+						     fsp);
 		}
 	}
 
-	result = dos_mode_post(result, fsp, __func__);
-	return result;
+	fsp->fsp_name->st.cached_dos_attributes = dos_mode_post(result, fsp, __func__);
+	return fsp->fsp_name->st.cached_dos_attributes;
 }
 
 struct dos_mode_at_state {
@@ -946,6 +929,7 @@ int file_set_dosmode(connection_struct *conn,
 	}
 
 	if (NT_STATUS_IS_OK(status)) {
+		smb_fname->st.cached_dos_attributes = dosmode;
 		ret = 0;
 		goto done;
 	}
@@ -1084,15 +1068,17 @@ NTSTATUS file_set_sparse(connection_struct *conn,
 	 * Windows Server 2008 & 2012 permit FSCTL_SET_SPARSE if any of the
 	 * following access flags are granted.
 	 */
-	if ((fsp->access_mask & (FILE_WRITE_DATA
-				| FILE_WRITE_ATTRIBUTES
-				| SEC_FILE_APPEND_DATA)) == 0) {
-		DEBUG(9,("file_set_sparse: fname[%s] set[%u] "
-			"access_mask[0x%08X] - access denied\n",
-			smb_fname_str_dbg(fsp->fsp_name),
-			sparse,
-			fsp->access_mask));
-		return NT_STATUS_ACCESS_DENIED;
+	status = check_any_access_fsp(fsp,
+				  FILE_WRITE_DATA
+				  | FILE_WRITE_ATTRIBUTES
+				  | SEC_FILE_APPEND_DATA);
+	if (!NT_STATUS_IS_OK(status)) {
+		DBG_DEBUG("fname[%s] set[%u] "
+			  "access_mask[0x%08X] - access denied\n",
+			  smb_fname_str_dbg(fsp->fsp_name),
+			  sparse,
+			  fsp->access_mask);
+		return status;
 	}
 
 	if (fsp->fsp_flags.is_directory) {
@@ -1113,7 +1099,7 @@ NTSTATUS file_set_sparse(connection_struct *conn,
 		 * MS-FSA 2.1.1.5 IsSparse
 		 *
 		 * This is a per stream attribute, but our backends don't
-		 * support it a consistent way, therefor just pretend
+		 * support it a consistent way, therefore just pretend
 		 * success and ignore the request.
 		 */
 		DBG_DEBUG("Ignoring request to set FILE_ATTRIBUTE_SPARSE on "
@@ -1153,6 +1139,7 @@ NTSTATUS file_set_sparse(connection_struct *conn,
 		     FILE_NOTIFY_CHANGE_ATTRIBUTES,
 		     fsp->fsp_name->base_name);
 
+	fsp->fsp_name->st.cached_dos_attributes = new_dosmode;
 	fsp->fsp_flags.is_sparse = sparse;
 
 	return NT_STATUS_OK;
