@@ -25,29 +25,17 @@
 
 #ifdef HAVE_KERNEL_OPLOCKS_LINUX
 
-#ifndef F_SETLEASE
-#define F_SETLEASE	1024
-#endif
-
-#ifndef F_GETLEASE
-#define F_GETLEASE	1025
-#endif
-
 #ifndef RT_SIGNAL_LEASE
 #define RT_SIGNAL_LEASE (SIGRTMIN+1)
 #endif
 
-#ifndef F_SETSIG
-#define F_SETSIG 10
-#endif
-
 /* 
  * Call to set the kernel lease signal handler
  */
 int linux_set_lease_sighandler(int fd)
 {
         if (fcntl(fd, F_SETSIG, RT_SIGNAL_LEASE) == -1) {
-                DEBUG(3,("Failed to set signal handler for kernel lease\n"));
+                DBG_NOTICE("Failed to set signal handler for kernel lease\n");
                 return -1;
         }
 
@@ -63,7 +51,7 @@ int linux_set_lease_sighandler(int fd)
 int linux_setlease(int fd, int leasetype)
 {
 	int ret;
-	int saved_errno;
+	int saved_errno = 0;
 
 	/*
 	 * Ensure the lease owner is root to allow
@@ -115,7 +103,7 @@ static void linux_oplock_signal_handler(struct tevent_context *ev_ctx,
 
 	fsp = file_find_fd(sconn, fd);
 	if (fsp == NULL) {
-		DEBUG(0,("linux_oplock_signal_handler: failed to find fsp for file fd=%d (file was closed ?)\n", fd ));
+		DBG_ERR("linux_oplock_signal_handler: failed to find fsp for file fd=%d (file was closed ?)\n", fd );
 		return;
 	}
 	break_kernel_oplock(sconn->msg_ctx, fsp);
@@ -134,7 +122,7 @@ static bool linux_set_kernel_oplock(struct kernel_oplocks *ctx,
 		DBG_NOTICE("Refused oplock on file %s, "
 			   "fd = %d, file_id = %s. (%s)\n",
 			   fsp_str_dbg(fsp),
-			   fsp->fh->fd,
+			   fsp_get_io_fd(fsp),
 			   file_id_str_buf(fsp->file_id, &idbuf),
 			   strerror(errno));
 		return False;
@@ -144,7 +132,7 @@ static bool linux_set_kernel_oplock(struct kernel_oplocks *ctx,
 		   "file_id = %s gen_id = %"PRIu64"\n",
 		   fsp_str_dbg(fsp),
 		   file_id_str_buf(fsp->file_id, &idbuf),
-		   fsp->fh->gen_id);
+		   fh_get_gen_id(fsp->fh));
 
 	return True;
 }
@@ -158,18 +146,18 @@ static void linux_release_kernel_oplock(struct kernel_oplocks *ctx,
 {
 	struct file_id_buf idbuf;
 
-	if (DEBUGLVL(10)) {
+	if (DEBUGLVL(DBGLVL_DEBUG)) {
 		/*
 		 * Check and print out the current kernel
 		 * oplock state of this file.
 		 */
-		int state = fcntl(fsp->fh->fd, F_GETLEASE, 0);
+		int state = fcntl(fsp_get_io_fd(fsp), F_GETLEASE, 0);
 		dbgtext("linux_release_kernel_oplock: file %s, file_id = %s "
 			"gen_id = %"PRIu64" has kernel oplock state "
 			"of %x.\n",
 			fsp_str_dbg(fsp),
 		        file_id_str_buf(fsp->file_id, &idbuf),
-			fsp->fh->gen_id,
+			fh_get_gen_id(fsp->fh),
 			state);
 	}
 
@@ -177,14 +165,14 @@ static void linux_release_kernel_oplock(struct kernel_oplocks *ctx,
 	 * Remove the kernel oplock on this file.
 	 */
 	if ( SMB_VFS_LINUX_SETLEASE(fsp, F_UNLCK) == -1) {
-		if (DEBUGLVL(0)) {
+		if (DEBUGLVL(DBGLVL_ERR)) {
 			dbgtext("linux_release_kernel_oplock: Error when "
 				"removing kernel oplock on file " );
 			dbgtext("%s, file_id = %s, gen_id = %"PRIu64". "
 				"Error was %s\n",
 				fsp_str_dbg(fsp),
 				file_id_str_buf(fsp->file_id, &idbuf),
-				fsp->fh->gen_id,
+				fh_get_gen_id(fsp->fh),
 				strerror(errno));
 		}
 	}
@@ -220,13 +208,13 @@ struct kernel_oplocks *linux_init_kernel_oplocks(struct smbd_server_connection *
 	struct tevent_signal *se;
 
 	if (!linux_oplocks_available()) {
-		DEBUG(3,("Linux kernel oplocks not available\n"));
+		DBG_NOTICE("Linux kernel oplocks not available\n");
 		return NULL;
 	}
 
 	ctx = talloc_zero(sconn, struct kernel_oplocks);
 	if (!ctx) {
-		DEBUG(0,("Linux Kernel oplocks talloc_Zero failed\n"));
+		DBG_ERR("Linux Kernel oplocks talloc_Zero failed\n");
 		return NULL;
 	}
 
@@ -239,12 +227,12 @@ struct kernel_oplocks *linux_init_kernel_oplocks(struct smbd_server_connection *
 			       linux_oplock_signal_handler,
 			       ctx);
 	if (!se) {
-		DEBUG(0,("Failed to setup RT_SIGNAL_LEASE handler"));
+		DBG_ERR("Failed to setup RT_SIGNAL_LEASE handler");
 		TALLOC_FREE(ctx);
 		return NULL;
 	}
 
-	DEBUG(3,("Linux kernel oplocks enabled\n"));
+	DBG_NOTICE("Linux kernel oplocks enabled\n");
 
 	return ctx;
 }
