@@ -63,8 +63,7 @@
 /* Custom version for processing POSIX paths. */
 #define IS_PATH_SEP(c,posix_only) ((c) == '/' || (!(posix_only) && (c) == '\\'))
 
-static NTSTATUS check_path_syntax_internal(char *path,
-					   bool posix_path)
+NTSTATUS check_path_syntax(char *path, bool posix_path)
 {
 	char *d = path;
 	const char *s = path;
@@ -189,10 +188,10 @@ static NTSTATUS check_path_syntax_internal(char *path,
 			}
 			*d++ = *s++;
 		} else {
-			size_t siz;
+			size_t ch_size;
 			/* Get the size of the next MB character. */
-			next_codepoint(s,&siz);
-			switch(siz) {
+			next_codepoint(s,&ch_size);
+			switch(ch_size) {
 				case 5:
 					*d++ = *s++;
 					FALL_THROUGH;
@@ -209,7 +208,7 @@ static NTSTATUS check_path_syntax_internal(char *path,
 					*d++ = *s++;
 					break;
 				default:
-					DEBUG(0,("check_path_syntax_internal: character length assumptions invalid !\n"));
+					DBG_ERR("character length assumptions invalid !\n");
 					*d = '\0';
 					return NT_STATUS_INVALID_PARAMETER;
 			}
@@ -223,69 +222,59 @@ static NTSTATUS check_path_syntax_internal(char *path,
 }
 
 /****************************************************************************
- Ensure we check the path in *exactly* the same way as W2K for regular pathnames.
- No wildcards allowed.
+ SMB2-only code to strip an MSDFS prefix from an incoming pathname.
 ****************************************************************************/
 
-NTSTATUS check_path_syntax(char *path)
+NTSTATUS smb2_strip_dfs_path(const char *in_path, const char **out_path)
 {
-	return check_path_syntax_internal(path, false);
-}
+	const char *path = in_path;
 
-/****************************************************************************
- Check the path for a POSIX client.
- We're assuming here that '/' is not the second byte in any multibyte char
- set (a safe assumption).
-****************************************************************************/
-
-NTSTATUS check_path_syntax_posix(char *path)
-{
-	return check_path_syntax_internal(path, true);
-}
-
-/****************************************************************************
- Check the path for an SMB2 DFS path.
- SMB2 DFS paths look like hostname\share (followed by a possible \extrapath.
- Path returned from here must look like:
-	hostname/share (followed by a possible /extrapath).
-****************************************************************************/
-
-static NTSTATUS check_path_syntax_smb2_msdfs(char *path)
-{
-	char *share = NULL;
-	char *remaining_path = NULL;
-	/* No SMB2 names can start with '\\' */
-	if (path[0] == '\\') {
-		return NT_STATUS_OBJECT_NAME_INVALID;
+	/* Match the Windows 2022 behavior for an empty DFS pathname. */
+	if (*path == '\0') {
+		return NT_STATUS_INVALID_PARAMETER;
 	}
-	/*
-	 * smbclient libraries sometimes set the DFS flag and send
-	 * local pathnames. Cope with this by just calling
-	 * check_path_syntax() on the whole path if it doesn't
-	 * look like a DFS path, similar to what parse_dfs_path() does.
-	 */
-	/* servername should be at path[0] */
-	share = strchr(path, '\\');
-	if (share == NULL) {
-		return check_path_syntax(path);
+	/* Strip any leading '\\' characters - MacOSX client behavior. */
+	while (*path == '\\') {
+		path++;
 	}
-	*share++ = '/';
-	remaining_path = strchr(share, '\\');
-	if (remaining_path == NULL) {
-		/* Only hostname\share. We're done. */
-		return NT_STATUS_OK;
+	/* We should now be pointing at the server name. Go past it. */
+	for (;;) {
+		if (*path == '\0') {
+			/* End of complete path. Exit OK. */
+			goto out;
+		}
+		if (*path == '\\') {
+			/* End of server name. Go past and break. */
+			path++;
+			break;
+		}
+		path++; /* Continue looking for end of server name or string. */
 	}
-	*remaining_path++ = '/';
-	return check_path_syntax(remaining_path);
-}
 
-NTSTATUS check_path_syntax_smb2(char *path, bool dfs_path)
-{
-	if (dfs_path) {
-		return check_path_syntax_smb2_msdfs(path);
-	} else {
-		return check_path_syntax(path);
+	/* We should now be pointing at the share name. Go past it. */
+	for (;;) {
+		if (*path == '\0') {
+			/* End of complete path. Exit OK. */
+			goto out;
+		}
+		if (*path == '\\') {
+			/* End of share name. Go past and break. */
+			path++;
+			break;
+		}
+		if (*path == ':') {
+			/* Only invalid character in sharename. */
+			return NT_STATUS_OBJECT_NAME_INVALID;
+		}
+		path++; /* Continue looking for end of share name or string. */
 	}
+
+	/* path now points at the start of the real filename (if any). */
+
+  out:
+	/* We have stripped the DFS path prefix (if any). */
+	*out_path = path;
+	return NT_STATUS_OK;
 }
 
 /****************************************************************************
@@ -444,11 +433,7 @@ static size_t srvstr_get_path_internal(TALLOC_CTX *ctx,
 
   local_path:
 
-	if (posix_pathnames) {
-		*err = check_path_syntax_posix(dst);
-	} else {
-		*err = check_path_syntax(dst);
-	}
+	*err = check_path_syntax(dst, posix_pathnames);
 
 	return ret;
 }
@@ -556,46 +541,6 @@ size_t srvstr_pull_req_talloc(TALLOC_CTX *ctx, struct smb_request *req,
 				  bufrem, flags);
 }
 
-/****************************************************************************
- Check if we have a correct fsp pointing to a file. Basic check for open fsp.
-****************************************************************************/
-
-bool check_fsp_open(connection_struct *conn, struct smb_request *req,
-		    files_struct *fsp)
-{
-	if ((fsp == NULL) || (conn == NULL)) {
-		reply_nterror(req, NT_STATUS_INVALID_HANDLE);
-		return False;
-	}
-	if ((conn != fsp->conn) || (req->vuid != fsp->vuid)) {
-		reply_nterror(req, NT_STATUS_INVALID_HANDLE);
-		return False;
-	}
-	return True;
-}
-
-/****************************************************************************
- Check if we have a correct fsp pointing to a file.
-****************************************************************************/
-
-bool check_fsp(connection_struct *conn, struct smb_request *req,
-	       files_struct *fsp)
-{
-	if (!check_fsp_open(conn, req, fsp)) {
-		return False;
-	}
-	if (fsp->fsp_flags.is_directory) {
-		reply_nterror(req, NT_STATUS_INVALID_DEVICE_REQUEST);
-		return False;
-	}
-	if (fsp_get_pathref_fd(fsp) == -1) {
-		reply_nterror(req, NT_STATUS_ACCESS_DENIED);
-		return False;
-	}
-	fsp->num_smb_operations++;
-	return True;
-}
-
 /****************************************************************************
  Check if we have a correct fsp pointing to a quota fake file. Replacement for
  the CHECK_NTQUOTA_HANDLE_OK macro.
@@ -604,7 +549,11 @@ bool check_fsp(connection_struct *conn, struct smb_request *req,
 bool check_fsp_ntquota_handle(connection_struct *conn, struct smb_request *req,
 			      files_struct *fsp)
 {
-	if (!check_fsp_open(conn, req, fsp)) {
+	if ((fsp == NULL) || (conn == NULL)) {
+		return false;
+	}
+
+	if ((conn != fsp->conn) || (req->vuid != fsp->vuid)) {
 		return false;
 	}
 
@@ -739,8 +688,7 @@ static bool netbios_session_retarget(struct smbXsrv_connection *xconn,
 	*(uint32_t *)(outbuf+4) = in_addr->sin_addr.s_addr;
 	*(uint16_t *)(outbuf+8) = htons(retarget_port);
 
-	if (!smb1_srv_send(xconn, (char *)outbuf, false, 0, false,
-			  NULL)) {
+	if (!smb1_srv_send(xconn, (char *)outbuf, false, 0, false)) {
 		exit_server_cleanly("netbios_session_retarget: smb1_srv_send "
 				    "failed.");
 	}
@@ -781,7 +729,7 @@ void reply_special(struct smbXsrv_connection *xconn, char *inbuf, size_t inbuf_s
 	switch (msg_type) {
 	case NBSSrequest: /* session request */
 	{
-		/* inbuf_size is guarenteed to be at least 4. */
+		/* inbuf_size is guaranteed to be at least 4. */
 		fstring name1,name2;
 		int name_type1, name_type2;
 		int name_len1, name_len2;
@@ -898,7 +846,7 @@ void reply_special(struct smbXsrv_connection *xconn, char *inbuf, size_t inbuf_s
 	DEBUG(5,("init msg_type=0x%x msg_flags=0x%x\n",
 		    msg_type, msg_flags));
 
-	if (!smb1_srv_send(xconn, outbuf, false, 0, false, NULL)) {
+	if (!smb1_srv_send(xconn, outbuf, false, 0, false)) {
 		exit_server_cleanly("reply_special: smb1_srv_send failed.");
 	}
 
@@ -1192,6 +1140,13 @@ ssize_t sendfile_short_send(struct smbXsrv_connection *xconn,
 static NTSTATUS can_rename(connection_struct *conn, files_struct *fsp,
 			uint16_t dirtype)
 {
+	NTSTATUS status;
+
+	if (fsp->fsp_name->twrp != 0) {
+		/* Get the error right, this is what Windows returns. */
+		return NT_STATUS_NOT_SAME_DEVICE;
+	}
+
 	if (!CAN_WRITE(conn)) {
 		return NT_STATUS_MEDIA_WRITE_PROTECTED;
 	}
@@ -1230,11 +1185,11 @@ static NTSTATUS can_rename(connection_struct *conn, files_struct *fsp,
 		return NT_STATUS_OK;
 	}
 
-	if (fsp->access_mask & (DELETE_ACCESS|FILE_WRITE_ATTRIBUTES)) {
-		return NT_STATUS_OK;
+	status = check_any_access_fsp(fsp, DELETE_ACCESS | FILE_WRITE_ATTRIBUTES);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
 	}
-
-	return NT_STATUS_ACCESS_DENIED;
+	return NT_STATUS_OK;
 }
 
 /****************************************************************************
@@ -1746,6 +1701,18 @@ NTSTATUS rename_internals_fsp(connection_struct *conn,
 		 * destination parent name.
 		 */
 		parent_dir_fname_src = parent_dir_fname_dst;
+
+		/*
+		 * Ensure we have a pathref fsp on the
+		 * parent_dir_fname_src_atname to match the code in the else
+		 * branch where we use parent_pathref().
+		 */
+		status = reference_smb_fname_fsp_link(
+			parent_dir_fname_src_atname,
+			fsp->fsp_name);
+		if (!NT_STATUS_IS_OK(status)) {
+			goto out;
+		}
 	} else {
 		/*
 		 * source and destination parent directories are
@@ -2161,9 +2128,7 @@ struct smbd_do_unlocking_state {
 };
 
 static void smbd_do_unlocking_fn(
-	const uint8_t *buf,
-	size_t buflen,
-	bool *pmodified_dependent,
+	struct share_mode_lock *lck,
 	void *private_data)
 {
 	struct smbd_do_unlocking_state *state = private_data;
@@ -2197,7 +2162,7 @@ static void smbd_do_unlocking_fn(
 		}
 	}
 
-	*pmodified_dependent = true;
+	share_mode_wakeup_waiters(fsp->file_id);
 }
 
 NTSTATUS smbd_do_unlocking(struct smb_request *req,
@@ -2214,11 +2179,11 @@ NTSTATUS smbd_do_unlocking(struct smb_request *req,
 
 	DBG_NOTICE("%s num_ulocks=%"PRIu16"\n", fsp_fnum_dbg(fsp), num_ulocks);
 
-	status = share_mode_do_locked(
+	status = share_mode_do_locked_vfs_allowed(
 		fsp->file_id, smbd_do_unlocking_fn, &state);
 
 	if (!NT_STATUS_IS_OK(status)) {
-		DBG_DEBUG("share_mode_do_locked failed: %s\n",
+		DBG_DEBUG("share_mode_do_locked_vfs_allowed failed: %s\n",
 			  nt_errstr(status));
 		return status;
 	}
