@@ -25,6 +25,7 @@
 #include "serverid.h"
 #include "session.h"
 #include "smbd/globals.h"
+#include "smbd/smbXsrv_open.h"
 #include "util_tdb.h"
 #include "librpc/gen_ndr/ndr_open_files.h"
 
@@ -231,7 +232,8 @@ done:
 	return ret;
 }
 
-static int wipedbs_traverse_open(struct smbXsrv_open_global0 *open,
+static int wipedbs_traverse_open(struct db_record *db_rec,
+				 struct smbXsrv_open_global0 *open,
 				 void *wipedbs_state)
 {
 	struct wipedbs_state *state =
@@ -299,9 +301,9 @@ static int wipedbs_traverse_open(struct smbXsrv_open_global0 *open,
 		goto done;
 	}
 
-	tmp = dbwrap_record_get_key(open->db_rec);
+	tmp = dbwrap_record_get_key(db_rec);
 	rec->key = tdb_data_talloc_copy(rec, tmp);
-	tmp = dbwrap_record_get_value(open->db_rec);
+	tmp = dbwrap_record_get_value(db_rec);
 	rec->val = tdb_data_talloc_copy(rec, tmp);
 
 	rec->desc = talloc_asprintf(
@@ -317,7 +319,7 @@ static int wipedbs_traverse_open(struct smbXsrv_open_global0 *open,
 		goto done;
 	}
 
-	state->open_db = dbwrap_record_get_db(open->db_rec);
+	state->open_db = dbwrap_record_get_db(db_rec);
 
 	DLIST_ADD(sd->open_records, rec);
 	ret = 0;
@@ -436,63 +438,89 @@ done:
 	return status;
 }
 
+struct wipedbs_delete_state {
+	struct wipedbs_record_marker *cur;
+	bool verbose;
+	bool dry_run;
+	size_t total;
+	size_t num;
+};
+
+static void wipedbs_delete_fn(
+	struct db_record *rec, TDB_DATA value, void *private_data)
+{
+	struct db_context *db = dbwrap_record_get_db(rec);
+	struct wipedbs_delete_state *state = private_data;
+	struct wipedbs_record_marker *cur = state->cur;
+	NTSTATUS status = NT_STATUS_OK;
+
+	state->total += 1;
+
+	if (!tdb_data_equal(value, cur->val)) {
+		DBG_ERR("Warning: record <%s> from %s changed,"
+			"skip record!\n",
+			cur->desc, dbwrap_name(db));
+		return;
+	}
+
+	if (state->verbose) {
+		d_printf("deleting %s\n", cur->desc);
+	}
+
+	if (!state->dry_run) {
+		status = dbwrap_record_delete(rec);
+	}
+
+	if (!NT_STATUS_IS_OK(status)) {
+		DBG_ERR("Failed to delete record <%s> from %s: %s\n",
+			cur->desc,
+			dbwrap_name(db),
+			nt_errstr(status));
+		return;
+	}
+
+	state->num += 1;
+}
+
 static int wipedbs_delete_records(struct db_context *db,
 				  struct wipedbs_record_marker *records,
 				  bool dry_run, bool verbose, int *count)
 {
-	struct wipedbs_record_marker *cur;
-	struct db_record *rec;
-	TDB_DATA val;
-	NTSTATUS status;
-	unsigned num=0, total=0;
+	struct wipedbs_delete_state state = {
+		.verbose = verbose, .dry_run = dry_run,
+	};
 
 	if (db == NULL) {
 		return 0;
 	}
 
-	for (cur = records; cur != NULL; cur = cur->next) {
-		total++;
-		rec = dbwrap_fetch_locked(db, talloc_tos(), cur->key);
-		if (rec == NULL) {
-			DEBUG(0, ("Failed to fetch record <%s> from %s",
-				  cur->desc, dbwrap_name(db)));
-			continue;
+	for (state.cur = records;
+	     state.cur != NULL;
+	     state.cur = state.cur->next) {
+
+		NTSTATUS status = dbwrap_do_locked(
+			db, state.cur->key, wipedbs_delete_fn, &state);
+
+		if (!NT_STATUS_IS_OK(status)) {
+			DBG_ERR("dbwrap_do_locked failed for record <%s> "
+				"from %s\n",
+				state.cur->desc,
+				dbwrap_name(db));
 		}
-		val = dbwrap_record_get_value(rec);
-		if (tdb_data_equal(val, cur->val)) {
-			if (dry_run) {
-				status = NT_STATUS_OK;
-			} else {
-				status = dbwrap_record_delete(rec);
-			}
-			if (NT_STATUS_IS_OK(status)) {
-				num ++;
-			} else {
-				DEBUG(0, ("Failed to delete record <%s> from %s"
-					  ": %s\n", cur->desc, dbwrap_name(db),
-					  nt_errstr(status)));
-			}
-		} else {
-			DEBUG(0, ("Warning: record <%s> from %s changed"
-				  ", skip record!\n",
-				  cur->desc, dbwrap_name(db)));
-		}
-		if (verbose) {
-			d_printf("deleting %s\n", cur->desc);
-		}
-		TALLOC_FREE(rec);
 	}
 
 	if (verbose) {
-		d_printf("Deleted %u of %u records from %s\n",
-			 num, total, dbwrap_name(db));
+		d_printf("Deleted %zu of %zu records from %s\n",
+			 state.num,
+			 state.total,
+			 dbwrap_name(db));
 	}
 
 	if (count) {
-		*count += total;
+		*count += state.total;
 	}
 
-	return total - num;
+	return state.total - state.num;
 }
 
 static int wipedbs_traverse_server_data(struct db_record *rec,
