@@ -1237,7 +1237,7 @@ static struct auth4_context *make_auth4_context_ntlm_auth(TALLOC_CTX *mem_ctx, b
 {
 	struct auth4_context *auth4_context = talloc_zero(mem_ctx, struct auth4_context);
 	if (auth4_context == NULL) {
-		DEBUG(10, ("failed to allocate auth4_context failed\n"));
+		DEBUG(10, ("failed to allocate auth4_context\n"));
 		return NULL;
 	}
 	auth4_context->generate_session_info = ntlm_auth_generate_session_info;
