@@ -46,7 +46,8 @@ enum ntlm_break {
  * Test the normal 'LM and NTLM' combination
  */
 
-static bool test_lm_ntlm_broken(enum ntlm_break break_which) 
+static bool test_lm_ntlm_broken(enum ntlm_break break_which,
+				bool lanman_support_expected)
 {
 	bool pass = True;
 	NTSTATUS nt_status;
@@ -54,7 +55,7 @@ static bool test_lm_ntlm_broken(enum ntlm_break break_which)
 	DATA_BLOB lm_response = data_blob(NULL, 24);
 	DATA_BLOB nt_response = data_blob(NULL, 24);
 	DATA_BLOB session_key = data_blob(NULL, 16);
-
+	uint8_t authoritative = 1;
 	uchar lm_key[8];
 	uchar user_session_key[16];
 	uchar lm_hash[16];
@@ -101,6 +102,7 @@ static bool test_lm_ntlm_broken(enum ntlm_break break_which)
 					      flags, 0,
 					      lm_key, 
 					      user_session_key,
+					      &authoritative,
 					      &error_string, NULL);
 	
 	data_blob_free(&lm_response);
@@ -113,14 +115,26 @@ static bool test_lm_ntlm_broken(enum ntlm_break break_which)
 		return break_which == BREAK_NT;
 	}
 
-	if (memcmp(lm_hash, lm_key, 
-		   sizeof(lm_key)) != 0) {
-		DEBUG(1, ("LM Key does not match expectations!\n"));
- 		DEBUG(1, ("lm_key:\n"));
-		dump_data(1, lm_key, 8);
-		DEBUG(1, ("expected:\n"));
-		dump_data(1, lm_hash, 8);
-		pass = False;
+	/* If we are told the DC is Samba4, expect an LM key of zeros */
+	if (!lanman_support_expected) {
+		if (!all_zero(lm_key,
+			      sizeof(lm_key))) {
+			DEBUG(1, ("LM Key does not match expectations!\n"));
+			DEBUG(1, ("lm_key:\n"));
+			dump_data(1, lm_key, 8);
+			DEBUG(1, ("expected: all zeros\n"));
+			pass = False;
+		}
+	} else {
+		if (memcmp(lm_hash, lm_key,
+			   sizeof(lm_key)) != 0) {
+			DEBUG(1, ("LM Key does not match expectations!\n"));
+			DEBUG(1, ("lm_key:\n"));
+			dump_data(1, lm_key, 8);
+			DEBUG(1, ("expected:\n"));
+			dump_data(1, lm_hash, 8);
+			pass = False;
+		}
 	}
 
 	if (break_which == NO_NT) {
@@ -151,32 +165,32 @@ static bool test_lm_ntlm_broken(enum ntlm_break break_which)
  * Test LM authentication, no NT response supplied
  */
 
-static bool test_lm(void) 
+static bool test_lm(bool lanman_support_expected)
 {
 
-	return test_lm_ntlm_broken(NO_NT);
+	return test_lm_ntlm_broken(NO_NT, lanman_support_expected);
 }
 
 /* 
  * Test the NTLM response only, no LM.
  */
 
-static bool test_ntlm(void) 
+static bool test_ntlm(bool lanman_support_expected)
 {
-	return test_lm_ntlm_broken(NO_LM);
+	return test_lm_ntlm_broken(NO_LM, lanman_support_expected);
 }
 
 /* 
  * Test the NTLM response only, but in the LM field.
  */
 
-static bool test_ntlm_in_lm(void) 
+static bool test_ntlm_in_lm(bool lanman_support_expected)
 {
 	bool pass = True;
 	NTSTATUS nt_status;
 	uint32_t flags = 0;
 	DATA_BLOB nt_response = data_blob(NULL, 24);
-
+	uint8_t authoritative = 1;
 	uchar lm_key[8];
 	uchar lm_hash[16];
 	uchar user_session_key[16];
@@ -200,6 +214,7 @@ static bool test_ntlm_in_lm(void)
 					      flags, 0,
 					      lm_key,
 					      user_session_key,
+					      &authoritative,
 					      &error_string, NULL);
 	
 	data_blob_free(&nt_response);
@@ -212,22 +227,42 @@ static bool test_ntlm_in_lm(void)
 		return False;
 	}
 
-	if (memcmp(lm_hash, lm_key, 
-		   sizeof(lm_key)) != 0) {
-		DEBUG(1, ("LM Key does not match expectations!\n"));
- 		DEBUG(1, ("lm_key:\n"));
-		dump_data(1, lm_key, 8);
-		DEBUG(1, ("expected:\n"));
-		dump_data(1, lm_hash, 8);
-		pass = False;
-	}
-	if (memcmp(lm_hash, user_session_key, 8) != 0) {
-		DEBUG(1, ("Session Key (first 8 lm hash) does not match expectations!\n"));
- 		DEBUG(1, ("user_session_key:\n"));
-		dump_data(1, user_session_key, 16);
- 		DEBUG(1, ("expected:\n"));
-		dump_data(1, lm_hash, 8);
-		pass = False;
+	/* If we are told the DC is Samba4, expect an LM key of zeros */
+	if (!lanman_support_expected) {
+		if (!all_zero(lm_key,
+			      sizeof(lm_key))) {
+			DEBUG(1, ("LM Key does not match expectations!\n"));
+			DEBUG(1, ("lm_key:\n"));
+			dump_data(1, lm_key, 8);
+			DEBUG(1, ("expected: all zeros\n"));
+			pass = False;
+		}
+		if (!all_zero(user_session_key,
+			      sizeof(user_session_key))) {
+			DEBUG(1, ("Session Key (normally first 8 lm hash) does not match expectations!\n"));
+			DEBUG(1, ("user_session_key:\n"));
+			dump_data(1, user_session_key, 16);
+			DEBUG(1, ("expected all zeros:\n"));
+			pass = False;
+		}
+	} else {
+		if (memcmp(lm_hash, lm_key,
+			   sizeof(lm_key)) != 0) {
+			DEBUG(1, ("LM Key does not match expectations!\n"));
+			DEBUG(1, ("lm_key:\n"));
+			dump_data(1, lm_key, 8);
+			DEBUG(1, ("expected:\n"));
+			dump_data(1, lm_hash, 8);
+			pass = False;
+		}
+		if (memcmp(lm_hash, user_session_key, 8) != 0) {
+			DEBUG(1, ("Session Key (first 8 lm hash) does not match expectations!\n"));
+			DEBUG(1, ("user_session_key:\n"));
+			dump_data(1, user_session_key, 16);
+			DEBUG(1, ("expected:\n"));
+			dump_data(1, lm_hash, 8);
+			pass = False;
+		}
 	}
         return pass;
 }
@@ -236,14 +271,14 @@ static bool test_ntlm_in_lm(void)
  * Test the NTLM response only, but in the both the NT and LM fields.
  */
 
-static bool test_ntlm_in_both(void) 
+static bool test_ntlm_in_both(bool lanman_support_expected)
 {
 	bool pass = True;
 	NTSTATUS nt_status;
 	uint32_t flags = 0;
 	DATA_BLOB nt_response = data_blob(NULL, 24);
 	DATA_BLOB session_key = data_blob(NULL, 16);
-
+	uint8_t authoritative = 1;
 	uint8_t lm_key[8];
 	uint8_t lm_hash[16];
 	uint8_t user_session_key[16];
@@ -271,6 +306,7 @@ static bool test_ntlm_in_both(void)
 					      flags, 0,
 					      lm_key,
 					      user_session_key,
+					      &authoritative,
 					      &error_string, NULL);
 	
 	data_blob_free(&nt_response);
@@ -283,14 +319,26 @@ static bool test_ntlm_in_both(void)
 		return False;
 	}
 
-	if (memcmp(lm_hash, lm_key, 
-		   sizeof(lm_key)) != 0) {
-		DEBUG(1, ("LM Key does not match expectations!\n"));
- 		DEBUG(1, ("lm_key:\n"));
-		dump_data(1, lm_key, 8);
-		DEBUG(1, ("expected:\n"));
-		dump_data(1, lm_hash, 8);
-		pass = False;
+	/* If we are told the DC is Samba4, expect an LM key of zeros */
+	if (!lanman_support_expected) {
+		if (!all_zero(lm_key,
+			      sizeof(lm_key))) {
+			DEBUG(1, ("LM Key does not match expectations!\n"));
+			DEBUG(1, ("lm_key:\n"));
+			dump_data(1, lm_key, 8);
+			DEBUG(1, ("expected: all zeros\n"));
+			pass = False;
+		}
+	} else {
+		if (memcmp(lm_hash, lm_key,
+			   sizeof(lm_key)) != 0) {
+			DEBUG(1, ("LM Key does not match expectations!\n"));
+			DEBUG(1, ("lm_key:\n"));
+			dump_data(1, lm_key, 8);
+			DEBUG(1, ("expected:\n"));
+			dump_data(1, lm_hash, 8);
+			pass = False;
+		}
 	}
 	if (memcmp(session_key.data, user_session_key, 
 		   sizeof(user_session_key)) != 0) {
@@ -319,7 +367,7 @@ static bool test_lmv2_ntlmv2_broken(enum ntlm_break break_which)
 	DATA_BLOB lmv2_response = data_blob_null;
 	DATA_BLOB ntlmv2_session_key = data_blob_null;
 	DATA_BLOB names_blob = NTLMv2_generate_names_blob(NULL, get_winbind_netbios_name(), get_winbind_domain());
-
+	uint8_t authoritative = 1;
 	uchar user_session_key[16];
 	DATA_BLOB chall = get_challenge();
 	char *error_string;
@@ -362,6 +410,7 @@ static bool test_lmv2_ntlmv2_broken(enum ntlm_break break_which)
 					      flags, 0,
 					      NULL, 
 					      user_session_key,
+					      &authoritative,
 					      &error_string, NULL);
 	
 	data_blob_free(&lmv2_response);
@@ -391,7 +440,7 @@ static bool test_lmv2_ntlmv2_broken(enum ntlm_break break_which)
  * Test the NTLMv2 and LMv2 responses
  */
 
-static bool test_lmv2_ntlmv2(void) 
+static bool test_lmv2_ntlmv2(bool lanman_support_expected)
 {
 	return test_lmv2_ntlmv2_broken(BREAK_NONE);
 }
@@ -400,7 +449,7 @@ static bool test_lmv2_ntlmv2(void)
  * Test the LMv2 response only
  */
 
-static bool test_lmv2(void) 
+static bool test_lmv2(bool lanman_support_expected)
 {
 	return test_lmv2_ntlmv2_broken(NO_NT);
 }
@@ -409,32 +458,32 @@ static bool test_lmv2(void)
  * Test the NTLMv2 response only
  */
 
-static bool test_ntlmv2(void) 
+static bool test_ntlmv2(bool lanman_support_expected)
 {
 	return test_lmv2_ntlmv2_broken(NO_LM);
 }
 
-static bool test_lm_ntlm(void) 
+static bool test_lm_ntlm(bool lanman_support_expected)
 {
-	return test_lm_ntlm_broken(BREAK_NONE);
+	return test_lm_ntlm_broken(BREAK_NONE, lanman_support_expected);
 }
 
-static bool test_ntlm_lm_broken(void) 
+static bool test_ntlm_lm_broken(bool lanman_support_expected)
 {
-	return test_lm_ntlm_broken(BREAK_LM);
+	return test_lm_ntlm_broken(BREAK_LM, lanman_support_expected);
 }
 
-static bool test_ntlm_ntlm_broken(void) 
+static bool test_ntlm_ntlm_broken(bool lanman_support_expected)
 {
-	return test_lm_ntlm_broken(BREAK_NT);
+	return test_lm_ntlm_broken(BREAK_NT, lanman_support_expected);
 }
 
-static bool test_ntlmv2_lmv2_broken(void) 
+static bool test_ntlmv2_lmv2_broken(bool lanman_support_expected)
 {
 	return test_lmv2_ntlmv2_broken(BREAK_LM);
 }
 
-static bool test_ntlmv2_ntlmv2_broken(void) 
+static bool test_ntlmv2_ntlmv2_broken(bool lanman_support_expected)
 {
 	return test_lmv2_ntlmv2_broken(BREAK_NT);
 }
@@ -448,7 +497,7 @@ static bool test_plaintext(enum ntlm_break break_which)
 	char *password;
 	smb_ucs2_t *nt_response_ucs2;
 	size_t converted_size;
-
+	uint8_t authoritative = 1;
 	uchar user_session_key[16];
 	uchar lm_key[16];
 	static const uchar zeros[8] = { 0, };
@@ -513,6 +562,7 @@ static bool test_plaintext(enum ntlm_break break_which)
 					      flags, MSV1_0_CLEARTEXT_PASSWORD_ALLOWED,
 					      lm_key,
 					      user_session_key,
+					      &authoritative,
 					      &error_string, NULL);
 	
 	TALLOC_FREE(nt_response.data);
@@ -530,23 +580,23 @@ static bool test_plaintext(enum ntlm_break break_which)
         return break_which != BREAK_NT;
 }
 
-static bool test_plaintext_none_broken(void) {
+static bool test_plaintext_none_broken(bool lanman_support_expected) {
 	return test_plaintext(BREAK_NONE);
 }
 
-static bool test_plaintext_lm_broken(void) {
+static bool test_plaintext_lm_broken(bool lanman_support_expected) {
 	return test_plaintext(BREAK_LM);
 }
 
-static bool test_plaintext_nt_broken(void) {
+static bool test_plaintext_nt_broken(bool lanman_support_expected) {
 	return test_plaintext(BREAK_NT);
 }
 
-static bool test_plaintext_nt_only(void) {
+static bool test_plaintext_nt_only(bool lanman_support_expected) {
 	return test_plaintext(NO_LM);
 }
 
-static bool test_plaintext_lm_only(void) {
+static bool test_plaintext_lm_only(bool lanman_support_expected) {
 	return test_plaintext(NO_NT);
 }
 
@@ -569,37 +619,102 @@ static bool test_plaintext_lm_only(void) {
 */
 
 static const struct ntlm_tests {
-	bool (*fn)(void);
+	bool (*fn)(bool lanman_support_expected);
 	const char *name;
+	bool lanman;
 } test_table[] = {
-	{test_lm, "LM"},
-	{test_lm_ntlm, "LM and NTLM"},
-	{test_ntlm, "NTLM"},
-	{test_ntlm_in_lm, "NTLM in LM"},
-	{test_ntlm_in_both, "NTLM in both"},
-	{test_ntlmv2, "NTLMv2"},
-	{test_lmv2_ntlmv2, "NTLMv2 and LMv2"},
-	{test_lmv2, "LMv2"},
-	{test_ntlmv2_lmv2_broken, "NTLMv2 and LMv2, LMv2 broken"},
-	{test_ntlmv2_ntlmv2_broken, "NTLMv2 and LMv2, NTLMv2 broken"},
-	{test_ntlm_lm_broken, "NTLM and LM, LM broken"},
-	{test_ntlm_ntlm_broken, "NTLM and LM, NTLM broken"},
-	{test_plaintext_none_broken, "Plaintext"},
-	{test_plaintext_lm_broken, "Plaintext LM broken"},
-	{test_plaintext_nt_broken, "Plaintext NT broken"},
-	{test_plaintext_nt_only, "Plaintext NT only"},
-	{test_plaintext_lm_only, "Plaintext LM only"},
-	{NULL, NULL}
+	{
+		.fn = test_lm,
+		.name = "LM",
+		.lanman = true
+	},
+	{
+		.fn = test_lm_ntlm,
+		.name = "LM and NTLM"
+	},
+	{
+		.fn = test_ntlm,
+		.name = "NTLM"
+	},
+	{
+		.fn = test_ntlm_in_lm,
+		.name = "NTLM in LM"
+	},
+	{
+		.fn = test_ntlm_in_both,
+		.name = "NTLM in both"
+	},
+	{
+		.fn = test_ntlmv2,
+		.name = "NTLMv2"
+	},
+	{
+		.fn = test_lmv2_ntlmv2,
+		.name = "NTLMv2 and LMv2"
+	},
+	{
+		.fn = test_lmv2,
+		.name = "LMv2"
+	},
+	{
+		.fn = test_ntlmv2_lmv2_broken,
+		.name = "NTLMv2 and LMv2, LMv2 broken"
+	},
+	{
+		.fn = test_ntlmv2_ntlmv2_broken,
+		.name = "NTLMv2 and LMv2, NTLMv2 broken"
+	},
+	{
+		.fn = test_ntlm_lm_broken,
+		.name = "NTLM and LM, LM broken"
+	},
+	{
+		.fn = test_ntlm_ntlm_broken,
+		.name = "NTLM and LM, NTLM broken"
+	},
+	{
+		.fn = test_plaintext_none_broken,
+		.name = "Plaintext"
+	},
+	{
+		.fn = test_plaintext_lm_broken,
+		.name = "Plaintext LM broken"
+	},
+	{
+		.fn = test_plaintext_nt_broken,
+		.name = "Plaintext NT broken"
+	},
+	{
+		.fn = test_plaintext_nt_only,
+		.name = "Plaintext NT only"
+	},
+	{
+		.fn = test_plaintext_lm_only,
+		.name = "Plaintext LM only",
+		.lanman = true
+	},
+	{
+		.fn = NULL
+	}
 };
 
-bool diagnose_ntlm_auth(void)
+bool diagnose_ntlm_auth(bool lanman_support_expected)
 {
 	unsigned int i;
 	bool pass = True;
 
 	for (i=0; test_table[i].fn; i++) {
-		if (!test_table[i].fn()) {
-			DEBUG(1, ("Test %s failed!\n", test_table[i].name));
+		bool test_pass = test_table[i].fn(lanman_support_expected);
+		if (!lanman_support_expected
+		    && test_table[i].lanman) {
+			if (test_pass) {
+				DBG_ERR("Test %s unexpectedly passed "
+					"(server should have rejected LM)!\n",
+					test_table[i].name);
+				pass = false;
+			}
+		} else if (!test_pass) {
+			DBG_ERR("Test %s failed!\n", test_table[i].name);
 			pass = False;
 		}
 	}
