@@ -35,6 +35,7 @@
 #include "util_tdb.h"
 #include "../lib/util/pidfile.h"
 #include "serverid.h"
+#include "lib/util/server_id_db.h"
 #include "cmdline_contexts.h"
 #include "lib/util/string_wrappers.h"
 #include "lib/global_contexts.h"
@@ -1341,6 +1342,18 @@ static bool do_winbind_validate_cache(struct tevent_context *ev_ctx,
 	return num_replies;
 }
 
+static bool do_reload_certs(struct tevent_context *ev_ctx,
+					struct messaging_context *msg_ctx,
+					const struct server_id pid,
+					const int argc, const char **argv)
+{
+	if (argc != 1) {
+		fprintf(stderr, "Usage: smbcontrol ldap_server reload-certs \n");
+		return false;
+	}
+
+	return send_message(msg_ctx, pid, MSG_RELOAD_TLS_CERTIFICATES, NULL, 0);
+}
 static bool do_reload_config(struct tevent_context *ev_ctx,
 			     struct messaging_context *msg_ctx,
 			     const struct server_id pid,
@@ -1542,6 +1555,11 @@ static const struct {
 		.fn   = do_drvupgrade,
 		.help = "Notify a printer driver has changed",
 	},
+	{
+		.name = "reload-certs",
+		.fn   = do_reload_certs,
+		.help = "Reload TLS certificates"
+	},
 	{
 		.name = "reload-config",
 		.fn   = do_reload_config,
@@ -1614,8 +1632,8 @@ static void usage(poptContext pc)
 	poptPrintHelp(pc, stderr, 0);
 
 	fprintf(stderr, "\n");
-	fprintf(stderr, "<destination> is one of \"nmbd\", \"smbd\", \"winbindd\" or a "
-		"process ID\n");
+	fprintf(stderr, "<destination> is one of \"nmbd\", \"smbd\", \"winbindd\", "
+		"\"ldap_server\" or a process ID\n");
 
 	fprintf(stderr, "\n");
 	fprintf(stderr, "<message-type> is one of:\n");
@@ -1642,6 +1660,8 @@ static struct server_id parse_dest(struct messaging_context *msg,
 		.pid = (uint64_t)-1,
 	};
 	pid_t pid;
+	struct server_id_db *names_db = NULL;
+	bool ok;
 
 	/* Zero is a special return value for broadcast to all processes */
 
@@ -1674,6 +1694,16 @@ static struct server_id parse_dest(struct messaging_context *msg,
 		return pid_to_procid(pid);
 	}
 
+	names_db = messaging_names_db(msg);
+	if (names_db == NULL) {
+		goto fail;
+	}
+	ok = server_id_db_lookup_one(names_db, dest, &result);
+	if (ok) {
+		return result;
+	}
+
+fail:
 	fprintf(stderr,"Can't find pid for destination '%s'\n", dest);
 
 	return result;
