@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    Winbind rpc backend functions
@@ -544,6 +544,60 @@ done:
 	return status;
 }
 
+/* lookup alias membership */
+static NTSTATUS msrpc_lookup_aliasmem(struct winbindd_domain *domain,
+				      TALLOC_CTX *mem_ctx,
+				      const struct dom_sid *alias_sid,
+				      enum lsa_SidType type,
+				      uint32_t *pnum_sids,
+				      struct dom_sid **sid_mem)
+{
+	struct rpc_pipe_client *samr_pipe = NULL;
+	struct policy_handle dom_pol;
+	struct dom_sid *alias_members = NULL;
+	struct dom_sid_buf buf;
+	uint32_t num_groups = 0;
+	TALLOC_CTX *tmp_ctx = talloc_stackframe();
+	NTSTATUS status;
+
+	D_INFO("Lookup alias members in domain=%s for sid=%s.\n",
+	       domain->name,
+	       dom_sid_str_buf(alias_sid, &buf));
+
+	*pnum_sids = 0;
+
+	if (!winbindd_can_contact_domain(domain)) {
+		D_DEBUG("No incoming trust for domain %s\n", domain->name);
+		status = NT_STATUS_OK;
+		goto done;
+	}
+
+	status = cm_connect_sam(domain, tmp_ctx, false, &samr_pipe, &dom_pol);
+	if (!NT_STATUS_IS_OK(status)) {
+		goto done;
+	}
+
+	status = rpc_lookup_aliasmem(tmp_ctx,
+				     samr_pipe,
+				     &dom_pol,
+				     &domain->sid,
+				     alias_sid,
+				     type,
+				     &num_groups,
+				     &alias_members);
+	if (!NT_STATUS_IS_OK(status)) {
+		goto done;
+	}
+
+	*pnum_sids = num_groups;
+	if (sid_mem) {
+		*sid_mem = talloc_move(mem_ctx, &alias_members);
+	}
+
+done:
+	talloc_free(tmp_ctx);
+	return status;
+}
 
 /* Lookup group membership given a rid.   */
 static NTSTATUS msrpc_lookup_groupmem(struct winbindd_domain *domain,
@@ -954,16 +1008,13 @@ NTSTATUS winbindd_lookup_sids(TALLOC_CTX *mem_ctx,
 	/* And restore our original timeout. */
 	dcerpc_binding_handle_set_timeout(b, orig_timeout);
 
-	if (NT_STATUS_EQUAL(status, NT_STATUS_ACCESS_DENIED) ||
-	    NT_STATUS_EQUAL(status, NT_STATUS_RPC_SEC_PKG_ERROR) ||
-	    NT_STATUS_EQUAL(status, NT_STATUS_NETWORK_ACCESS_DENIED)) {
+	if (reset_cm_connection_on_error(domain, b, status)) {
 		/*
 		 * This can happen if the schannel key is not
 		 * valid anymore, we need to invalidate the
 		 * all connections to the dc and reestablish
 		 * a netlogon connection first.
 		 */
-		invalidate_cm_connection(domain);
 		domain->can_do_ncacn_ip_tcp = domain->active_directory;
 		if (!retried) {
 			retried = true;
@@ -1033,16 +1084,13 @@ static NTSTATUS winbindd_lookup_names(TALLOC_CTX *mem_ctx,
 	/* And restore our original timeout. */
 	dcerpc_binding_handle_set_timeout(b, orig_timeout);
 
-	if (NT_STATUS_EQUAL(status, NT_STATUS_ACCESS_DENIED) ||
-	    NT_STATUS_EQUAL(status, NT_STATUS_RPC_SEC_PKG_ERROR) ||
-	    NT_STATUS_EQUAL(status, NT_STATUS_NETWORK_ACCESS_DENIED)) {
+	if (reset_cm_connection_on_error(domain, b, status)) {
 		/*
 		 * This can happen if the schannel key is not
 		 * valid anymore, we need to invalidate the
 		 * all connections to the dc and reestablish
 		 * a netlogon connection first.
 		 */
-		invalidate_cm_connection(domain);
 		if (!retried) {
 			retried = true;
 			goto connect;
@@ -1069,6 +1117,7 @@ struct winbindd_methods msrpc_methods = {
 	msrpc_lookup_usergroups,
 	msrpc_lookup_useraliases,
 	msrpc_lookup_groupmem,
+	msrpc_lookup_aliasmem,
 	msrpc_lockout_policy,
 	msrpc_password_policy,
 	msrpc_trusted_domains,
