@@ -20,10 +20,10 @@
 */
 
 #include "includes.h"
-#include "smbd/service_task.h"
-#include "smbd/service.h"
-#include "smbd/service_stream.h"
-#include "smbd/process_model.h"
+#include "samba/service_task.h"
+#include "samba/service.h"
+#include "samba/service_stream.h"
+#include "samba/process_model.h"
 #include "lib/events/events.h"
 #include "lib/socket/socket.h"
 #include "lib/tsocket/tsocket.h"
@@ -107,7 +107,7 @@ struct dns_process_state {
 	struct dns_server *dns;
 	struct dns_name_packet in_packet;
 	struct dns_request_state state;
-	uint16_t dns_err;
+	WERROR dns_err;
 	struct dns_name_packet out_packet;
 	DATA_BLOB out;
 };
@@ -146,7 +146,9 @@ static struct tevent_req *dns_process_send(TALLOC_CTX *mem_ctx,
 		(ndr_pull_flags_fn_t)ndr_pull_dns_name_packet);
 
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
-		state->dns_err = DNS_RCODE_FORMERR;
+		DBG_NOTICE("ndr_pull_dns_name_packet() failed with %s\n",
+			   ndr_map_error2string(ndr_err));
+		state->dns_err = DNS_ERR(FORMAT_ERROR);
 		tevent_req_done(req);
 		return tevent_req_post(req, ev);
 	}
@@ -155,7 +157,7 @@ static struct tevent_req *dns_process_send(TALLOC_CTX *mem_ctx,
 	}
 
 	if (state->in_packet.operation & DNS_FLAG_REPLY) {
-		DEBUG(1, ("Won't reply to replies.\n"));
+		DBG_INFO("Won't reply to replies.\n");
 		tevent_req_werror(req, WERR_INVALID_PARAMETER);
 		return tevent_req_post(req, ev);
 	}
@@ -175,7 +177,9 @@ static struct tevent_req *dns_process_send(TALLOC_CTX *mem_ctx,
 	ret = dns_verify_tsig(dns, state, &state->state,
 			      &state->out_packet, in);
 	if (!W_ERROR_IS_OK(ret)) {
-		state->dns_err = werr_to_dns_err(ret);
+		DBG_INFO("dns_verify_tsig() failed with %s\n",
+			 win_errstr(ret));
+		state->dns_err = ret;
 		tevent_req_done(req);
 		return tevent_req_post(req, ev);
 	}
@@ -197,13 +201,16 @@ static struct tevent_req *dns_process_send(TALLOC_CTX *mem_ctx,
 			&state->out_packet.nsrecs,  &state->out_packet.nscount,
 			&state->out_packet.additional,
 			&state->out_packet.arcount);
+		DBG_DEBUG("dns_server_process_update(): %s\n",
+			  win_errstr(ret));
 		break;
 	default:
 		ret = WERR_DNS_ERROR_RCODE_NOT_IMPLEMENTED;
+		DBG_NOTICE("OPCODE[0x%x]: %s\n",
+			   (state->in_packet.operation & DNS_OPCODE),
+			   win_errstr(ret));
 	}
-	if (!W_ERROR_IS_OK(ret)) {
-		state->dns_err = werr_to_dns_err(ret);
-	}
+	state->dns_err = ret;
 	tevent_req_done(req);
 	return tevent_req_post(req, ev);
 }
@@ -223,9 +230,9 @@ static void dns_process_done(struct tevent_req *subreq)
 		&state->out_packet.additional, &state->out_packet.arcount);
 	TALLOC_FREE(subreq);
 
-	if (!W_ERROR_IS_OK(ret)) {
-		state->dns_err = werr_to_dns_err(ret);
-	}
+	DBG_DEBUG("dns_server_process_query_recv(): %s\n",
+		  win_errstr(ret));
+	state->dns_err = ret;
 	tevent_req_done(req);
 }
 
@@ -235,19 +242,32 @@ static WERROR dns_process_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 	struct dns_process_state *state = tevent_req_data(
 		req, struct dns_process_state);
 	enum ndr_err_code ndr_err;
+	uint16_t dns_err;
 	WERROR ret;
 
 	if (tevent_req_is_werror(req, &ret)) {
+		DBG_NOTICE("ERROR: %s from %s\n", win_errstr(ret),
+			   tevent_req_print(state, req));
 		return ret;
 	}
-	if ((state->dns_err != DNS_RCODE_OK) &&
-	    (state->dns_err != DNS_RCODE_NXDOMAIN) &&
-	    (state->dns_err != DNS_RCODE_NOTAUTH))
+	dns_err = werr_to_dns_err(state->dns_err);
+	if ((dns_err != DNS_RCODE_OK) &&
+	    (dns_err != DNS_RCODE_NXDOMAIN) &&
+	    (dns_err != DNS_RCODE_NOTAUTH))
 	{
+		DBG_INFO("FAILURE: %s from %s\n",
+			 win_errstr(state->dns_err),
+			 tevent_req_print(state, req));
 		goto drop;
 	}
-	if (state->dns_err != DNS_RCODE_OK) {
-		state->out_packet.operation |= state->dns_err;
+	if (dns_err != DNS_RCODE_OK) {
+		DBG_DEBUG("INFO: %s from %s\n",
+			  win_errstr(state->dns_err),
+			  tevent_req_print(state, req));
+		state->out_packet.operation |= dns_err;
+	} else {
+		DBG_DEBUG("OK: %s\n",
+			  tevent_req_print(state, req));
 	}
 	state->out_packet.operation |= state->state.flags;
 
@@ -255,7 +275,9 @@ static WERROR dns_process_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 		ret = dns_sign_tsig(state->dns, mem_ctx, &state->state,
 				    &state->out_packet, 0);
 		if (!W_ERROR_IS_OK(ret)) {
-			state->dns_err = DNS_RCODE_SERVFAIL;
+			DBG_WARNING("dns_sign_tsig() failed %s\n",
+				    win_errstr(ret));
+			dns_err = DNS_RCODE_SERVFAIL;
 			goto drop;
 		}
 	}
@@ -268,9 +290,9 @@ static WERROR dns_process_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 		out, mem_ctx, &state->out_packet,
 		(ndr_push_flags_fn_t)ndr_push_dns_name_packet);
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
-		DEBUG(1, ("Failed to push packet: %s!\n",
-			  ndr_errstr(ndr_err)));
-		state->dns_err = DNS_RCODE_SERVFAIL;
+		DBG_WARNING("Failed to push packet: %s!\n",
+			    ndr_errstr(ndr_err));
+		dns_err = DNS_RCODE_SERVFAIL;
 		goto drop;
 	}
 	return WERR_OK;
@@ -281,7 +303,7 @@ drop:
 		return WERR_NOT_ENOUGH_MEMORY;
 	}
 	out->data[2] |= 0x80; /* Toggle DNS_FLAG_REPLY */
-	out->data[3] |= state->dns_err;
+	out->data[3] |= dns_err;
 	return WERR_OK;
 }
 
@@ -700,7 +722,7 @@ static NTSTATUS dns_startup_interfaces(struct dns_server *dns,
 							    iface_list_n_ip(ifaces, i));
 
 			status = dns_add_socket(dns, model_ops, "dns", address,
-						DNS_SERVICE_PORT);
+						lpcfg_dns_port(dns->task->lp_ctx));
 			NT_STATUS_NOT_OK_RETURN(status);
 		}
 	} else {
@@ -713,7 +735,7 @@ static NTSTATUS dns_startup_interfaces(struct dns_server *dns,
 		}
 		for (i = 0; wcard[i] != NULL; i++) {
 			status = dns_add_socket(dns, model_ops, "dns", wcard[i],
-						DNS_SERVICE_PORT);
+						lpcfg_dns_port(dns->task->lp_ctx));
 			if (NT_STATUS_IS_OK(status)) {
 				num_binds++;
 			}
@@ -754,7 +776,7 @@ static NTSTATUS dns_server_reload_zones(struct dns_server *dns)
 {
 	NTSTATUS status;
 	struct dns_server_zone *new_list = NULL;
-	struct dns_server_zone *old_list = NULL;
+	struct dns_server_zone *old_list = dns->zones;
 	struct dns_server_zone *old_zone;
 	status = dns_common_zones(dns->samdb, dns, NULL, &new_list);
 	if (!NT_STATUS_IS_OK(status)) {
@@ -790,7 +812,7 @@ static NTSTATUS dns_reload_zones(struct irpc_message *msg,
 	return NT_STATUS_OK;
 }
 
-static void dns_task_init(struct task_server *task)
+static NTSTATUS dns_task_init(struct task_server *task)
 {
 	struct dns_server *dns;
 	NTSTATUS status;
@@ -800,14 +822,15 @@ static void dns_task_init(struct task_server *task)
 	struct ldb_message *dns_acc;
 	char *hostname_lower;
 	char *dns_spn;
+	bool ok;
 
 	switch (lpcfg_server_role(task->lp_ctx)) {
 	case ROLE_STANDALONE:
 		task_server_terminate(task, "dns: no DNS required in standalone configuration", false);
-		return;
+		return NT_STATUS_INVALID_DOMAIN_ROLE;
 	case ROLE_DOMAIN_MEMBER:
 		task_server_terminate(task, "dns: no DNS required in member server configuration", false);
-		return;
+		return NT_STATUS_INVALID_DOMAIN_ROLE;
 	case ROLE_ACTIVE_DIRECTORY_DC:
 		/* Yes, we want a DNS */
 		break;
@@ -818,7 +841,7 @@ static void dns_task_init(struct task_server *task)
 
 		if (iface_list_count(ifaces) == 0) {
 			task_server_terminate(task, "dns: no network interfaces configured", false);
-			return;
+			return NT_STATUS_UNSUCCESSFUL;
 		}
 	}
 
@@ -827,7 +850,7 @@ static void dns_task_init(struct task_server *task)
 	dns = talloc_zero(task, struct dns_server);
 	if (dns == NULL) {
 		task_server_terminate(task, "dns: out of memory", true);
-		return;
+		return NT_STATUS_NO_MEMORY;
 	}
 
 	dns->task = task;
@@ -835,7 +858,7 @@ static void dns_task_init(struct task_server *task)
 	dns->server_credentials = cli_credentials_init(dns);
 	if (!dns->server_credentials) {
 		task_server_terminate(task, "Failed to init server credentials\n", true);
-		return;
+		return NT_STATUS_UNSUCCESSFUL;
 	}
 
 	dns->samdb = samdb_connect(dns,
@@ -846,10 +869,16 @@ static void dns_task_init(struct task_server *task)
 				   0);
 	if (!dns->samdb) {
 		task_server_terminate(task, "dns: samdb_connect failed", true);
-		return;
+		return NT_STATUS_UNSUCCESSFUL;
 	}
 
-	cli_credentials_set_conf(dns->server_credentials, task->lp_ctx);
+	ok = cli_credentials_set_conf(dns->server_credentials, task->lp_ctx);
+	if (!ok) {
+		task_server_terminate(task,
+				      "dns: failed to load smb.conf",
+				      true);
+		return NT_STATUS_UNSUCCESSFUL;
+	}
 
 	hostname_lower = strlower_talloc(dns, lpcfg_netbios_name(task->lp_ctx));
 	dns_spn = talloc_asprintf(dns, "DNS/%s.%s",
@@ -865,7 +894,7 @@ static void dns_task_init(struct task_server *task)
 		TALLOC_FREE(dns_acc);
 		if (!dns_spn) {
 			task_server_terminate(task, "dns: talloc_asprintf failed", true);
-			return;
+			return NT_STATUS_UNSUCCESSFUL;
 		}
 		status = cli_credentials_set_stored_principal(dns->server_credentials, task->lp_ctx, dns_spn);
 		if (!NT_STATUS_IS_OK(status)) {
@@ -874,7 +903,7 @@ static void dns_task_init(struct task_server *task)
 							      "despite finding it in the samdb! %s\n",
 							      nt_errstr(status)),
 					      true);
-			return;
+			return status;
 		}
 	} else {
 		TALLOC_FREE(dns_spn);
@@ -884,48 +913,51 @@ static void dns_task_init(struct task_server *task)
 					      talloc_asprintf(task, "Failed to obtain server credentials, perhaps a standalone server?: %s\n",
 							      nt_errstr(status)),
 					      true);
-			return;
+			return status;
 		}
 	}
 
 	dns->tkeys = tkey_store_init(dns, TKEY_BUFFER_SIZE);
 	if (!dns->tkeys) {
 		task_server_terminate(task, "Failed to allocate tkey storage\n", true);
-		return;
+		return NT_STATUS_NO_MEMORY;
 	}
 
 	status = dns_server_reload_zones(dns);
 	if (!NT_STATUS_IS_OK(status)) {
 		task_server_terminate(task, "dns: failed to load DNS zones", true);
-		return;
+		return status;
 	}
 
 	status = dns_startup_interfaces(dns, ifaces, task->model_ops);
 	if (!NT_STATUS_IS_OK(status)) {
 		task_server_terminate(task, "dns failed to setup interfaces", true);
-		return;
+		return status;
 	}
 
 	/* Setup the IRPC interface and register handlers */
 	status = irpc_add_name(task->msg_ctx, "dnssrv");
 	if (!NT_STATUS_IS_OK(status)) {
 		task_server_terminate(task, "dns: failed to register IRPC name", true);
-		return;
+		return status;
 	}
 
 	status = IRPC_REGISTER(task->msg_ctx, irpc, DNSSRV_RELOAD_DNS_ZONES,
 			       dns_reload_zones, dns);
 	if (!NT_STATUS_IS_OK(status)) {
 		task_server_terminate(task, "dns: failed to setup reload handler", true);
-		return;
+		return status;
 	}
+	return NT_STATUS_OK;
 }
 
 NTSTATUS server_service_dns_init(TALLOC_CTX *ctx)
 {
-	struct service_details details = {
+	static const struct service_details details = {
 		.inhibit_fork_on_accept = true,
 		.inhibit_pre_fork = true,
+		.task_init = dns_task_init,
+		.post_fork = NULL
 	};
-	return register_server_service(ctx, "dns", dns_task_init, &details);
+	return register_server_service(ctx, "dns", &details);
 }
