@@ -41,10 +41,12 @@ static void check_timestamp(time_t before, const char* timestamp)
 	struct tm tm;
 	time_t after;
 	time_t actual;
-	const double lower = -1;
+	struct timeval tv;
 
 
-	after = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	after = tv.tv_sec;
 
 	/*
 	 * Convert the ISO 8601 timestamp into a time_t
@@ -71,12 +73,8 @@ static void check_timestamp(time_t before, const char* timestamp)
 
 	/*
 	 * The timestamp should be before <= actual <= after
-	 * Note: as the microsecond portion of the time is truncated we use
-	 *       a -1 as the lower bound for the time difference instead of
-	 *       zero
 	 */
-	assert_true(difftime(actual, before) >= lower);
-	assert_true(difftime(after, actual) >= lower);
+	assert_in_range(actual, before, after);
 }
 
 static void test_has_password_changed(void **state)
@@ -295,6 +293,8 @@ static void test_operation_json_empty(void **state)
 	json_t *v = NULL;
 	json_t *o = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 
 	TALLOC_CTX *ctx = talloc_new(NULL);
@@ -310,7 +310,9 @@ static void test_operation_json_empty(void **state)
 	reply = talloc_zero(ctx, struct ldb_reply);
 	reply->error = LDB_SUCCESS;
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = operation_json(module, req, reply);
 	assert_int_equal(3, json_object_size(json.root));
 
@@ -426,6 +428,8 @@ static void test_operation_json(void **state)
 	json_t *f = NULL;
 	json_t *g = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 
 	TALLOC_CTX *ctx = talloc_new(NULL);
@@ -465,7 +469,9 @@ static void test_operation_json(void **state)
 	reply = talloc_zero(ctx, struct ldb_reply);
 	reply->error = LDB_ERR_OPERATIONS_ERROR;
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = operation_json(module, req, reply);
 	assert_int_equal(3, json_object_size(json.root));
 
@@ -621,6 +627,8 @@ static void test_as_system_operation_json(void **state)
 	json_t *f = NULL;
 	json_t *g = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 
 	TALLOC_CTX *ctx = talloc_new(NULL);
@@ -669,7 +677,9 @@ static void test_as_system_operation_json(void **state)
 	reply = talloc_zero(ctx, struct ldb_reply);
 	reply->error = LDB_ERR_OPERATIONS_ERROR;
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = operation_json(module, req, reply);
 	assert_int_equal(3, json_object_size(json.root));
 
@@ -796,6 +806,8 @@ static void test_password_change_json_empty(void **state)
 	json_t *v = NULL;
 	json_t *o = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 
 	TALLOC_CTX *ctx = talloc_new(NULL);
@@ -811,7 +823,9 @@ static void test_password_change_json_empty(void **state)
 	reply = talloc_zero(ctx, struct ldb_reply);
 	reply->error = LDB_SUCCESS;
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = password_change_json(module, req, reply);
 	assert_int_equal(3, json_object_size(json.root));
 
@@ -828,11 +842,14 @@ static void test_password_change_json_empty(void **state)
 	audit = json_object_get(json.root, "passwordChange");
 	assert_non_null(audit);
 	assert_true(json_is_object(audit));
-	assert_int_equal(9, json_object_size(audit));
+	assert_int_equal(10, json_object_size(audit));
 
 	o = json_object_get(audit, "version");
 	assert_non_null(o);
 
+	v = json_object_get(audit, "eventId");
+	assert_non_null(v);
+
 	v = json_object_get(audit, "statusCode");
 	assert_non_null(v);
 
@@ -896,6 +913,8 @@ static void test_password_change_json(void **state)
 	json_t *v = NULL;
 	json_t *o = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 	TALLOC_CTX *ctx = talloc_new(NULL);
 
@@ -933,7 +952,9 @@ static void test_password_change_json(void **state)
 	reply = talloc_zero(ctx, struct ldb_reply);
 	reply->error = LDB_SUCCESS;
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = password_change_json(module, req, reply);
 	assert_int_equal(3, json_object_size(json.root));
 
@@ -950,12 +971,17 @@ static void test_password_change_json(void **state)
 	audit = json_object_get(json.root, "passwordChange");
 	assert_non_null(audit);
 	assert_true(json_is_object(audit));
-	assert_int_equal(9, json_object_size(audit));
+	assert_int_equal(10, json_object_size(audit));
 
 	o = json_object_get(audit, "version");
 	assert_non_null(o);
 	check_version(o, PASSWORD_MAJOR,PASSWORD_MINOR);
 
+	v = json_object_get(audit, "eventId");
+	assert_non_null(v);
+	assert_true(json_is_integer(v));
+	assert_int_equal(EVT_ID_PASSWORD_RESET, json_integer_value(v));
+
 	v = json_object_get(audit, "statusCode");
 	assert_non_null(v);
 	assert_true(json_is_integer(v));
@@ -1017,10 +1043,14 @@ static void test_transaction_json(void **state)
 	json_t *v = NULL;
 	json_t *o = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 	GUID_from_string(GUID, &guid);
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = transaction_json("delete", &guid, 10000099);
 
 	assert_int_equal(3, json_object_size(json.root));
@@ -1078,10 +1108,14 @@ static void test_commit_failure_json(void **state)
 	json_t *v = NULL;
 	json_t *o = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 	GUID_from_string(GUID, &guid);
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = commit_failure_json(
 		"prepare",
 		987876,
@@ -1165,6 +1199,8 @@ static void test_replicated_update_json_empty(void **state)
 	json_t *v = NULL;
 	json_t *o = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 
 	TALLOC_CTX *ctx = talloc_new(NULL);
@@ -1185,7 +1221,9 @@ static void test_replicated_update_json_empty(void **state)
 	reply = talloc_zero(ctx, struct ldb_reply);
 	reply->error = LDB_SUCCESS;
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = replicated_update_json(module, req, reply);
 	assert_int_equal(3, json_object_size(json.root));
 
@@ -1301,6 +1339,8 @@ static void test_replicated_update_json(void **state)
 	json_t *v = NULL;
 	json_t *o = NULL;
 	time_t before;
+	struct timeval tv;
+	int rc;
 
 
 	TALLOC_CTX *ctx = talloc_new(NULL);
@@ -1337,7 +1377,9 @@ static void test_replicated_update_json(void **state)
 	reply = talloc_zero(ctx, struct ldb_reply);
 	reply->error = LDB_ERR_NO_SUCH_OBJECT;
 
-	before = time(NULL);
+	rc = gettimeofday(&tv, NULL);
+	assert_return_code(rc, errno);
+	before = tv.tv_sec;
 	json = replicated_update_json(module, req, reply);
 	assert_int_equal(3, json_object_size(json.root));
 
