@@ -255,14 +255,16 @@ int dsdb_module_search(struct ldb_module *module,
 }
 
 /*
-  find a DN given a GUID. This searches across all partitions
+  find an object given a GUID. This searches across all partitions
  */
-int dsdb_module_dn_by_guid(struct ldb_module *module, TALLOC_CTX *mem_ctx,
-			   const struct GUID *guid, struct ldb_dn **dn,
-			   struct ldb_request *parent)
+int dsdb_module_obj_by_guid(struct ldb_module *module,
+			    TALLOC_CTX *mem_ctx,
+			    struct ldb_message **_msg,
+			    const struct GUID *guid,
+			    const char * const *attrs,
+			    struct ldb_request *parent)
 {
 	struct ldb_result *res;
-	const char *attrs[] = { NULL };
 	TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
 	int ret;
 
@@ -289,7 +291,36 @@ int dsdb_module_dn_by_guid(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 		return LDB_ERR_OPERATIONS_ERROR;
 	}
 
-	*dn = talloc_steal(mem_ctx, res->msgs[0]->dn);
+	*_msg = talloc_steal(mem_ctx, res->msgs[0]);
+
+	talloc_free(tmp_ctx);
+	return LDB_SUCCESS;
+}
+
+/*
+  find a DN given a GUID. This searches across all partitions
+ */
+int dsdb_module_dn_by_guid(struct ldb_module *module, TALLOC_CTX *mem_ctx,
+			   const struct GUID *guid, struct ldb_dn **dn,
+			   struct ldb_request *parent)
+{
+	struct ldb_message *msg = NULL;
+	static const char * const attrs[] = { NULL };
+	TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
+	int ret;
+
+	ret = dsdb_module_obj_by_guid(module,
+				      tmp_ctx,
+				      &msg,
+				      guid,
+				      attrs,
+				      parent);
+	if (ret != LDB_SUCCESS) {
+		talloc_free(tmp_ctx);
+		return ret;
+	}
+
+	*dn = talloc_steal(mem_ctx, msg->dn);
 
 	talloc_free(tmp_ctx);
 	return LDB_SUCCESS;
@@ -301,7 +332,7 @@ int dsdb_module_dn_by_guid(struct ldb_module *module, TALLOC_CTX *mem_ctx,
 int dsdb_module_guid_by_dn(struct ldb_module *module, struct ldb_dn *dn, struct GUID *guid,
 			   struct ldb_request *parent)
 {
-	const char *attrs[] = { NULL };
+	static const char * const attrs[] = { NULL };
 	struct ldb_result *res;
 	TALLOC_CTX *tmp_ctx = talloc_new(module);
 	int ret;
@@ -690,7 +721,7 @@ int dsdb_check_samba_compatible_feature(struct ldb_module *module,
 {
 	struct ldb_context *ldb = ldb_module_get_ctx(module);
 	struct ldb_result *res;
-	static const char *samba_dsdb_attrs[] = {
+	static const char * const samba_dsdb_attrs[] = {
 		SAMBA_COMPATIBLE_FEATURES_ATTR,
 		NULL
 	};
@@ -746,7 +777,7 @@ int dsdb_check_optional_feature(struct ldb_module *module, struct GUID op_featur
 	struct ldb_result *res;
 	struct ldb_dn *search_dn;
 	struct GUID search_guid;
-	const char *attrs[] = {"msDS-EnabledFeature", NULL};
+	static const char * const attrs[] = {"msDS-EnabledFeature", NULL};
 	int ret;
 	unsigned int i;
 	struct ldb_message_element *el;
@@ -771,7 +802,7 @@ int dsdb_check_optional_feature(struct ldb_module *module, struct GUID op_featur
 		return LDB_ERR_NO_SUCH_OBJECT;
 	}
 	if (res->msgs[0]->num_elements > 0) {
-		const char *attrs2[] = {"msDS-OptionalFeatureGUID", NULL};
+		static const char * const attrs2[] = {"msDS-OptionalFeatureGUID", NULL};
 
 		el = ldb_msg_find_element(res->msgs[0],"msDS-EnabledFeature");
 
@@ -1349,7 +1380,7 @@ const struct ldb_val *dsdb_module_find_dsheuristics(struct ldb_module *module,
 	int ret;
 	struct ldb_dn *new_dn;
 	struct ldb_context *ldb = ldb_module_get_ctx(module);
-	static const char *attrs[] = { "dSHeuristics", NULL };
+	static const char * const attrs[] = { "dSHeuristics", NULL };
 	struct ldb_result *res;
 
 	new_dn = ldb_dn_copy(mem_ctx, ldb_get_config_basedn(ldb));
