@@ -18,7 +18,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
 import sys
 import unittest
 
@@ -31,7 +30,7 @@ from samba.dcerpc import security
 from samba.dcerpc import drsblobs
 from samba.dcerpc.drsuapi import *
 from samba.tests.password_test import PasswordCommon
-from samba.compat import get_string
+from samba.common import get_string
 
 import samba.tests
 from ldb import (SCOPE_BASE, FLAG_MOD_ADD, FLAG_MOD_DELETE, FLAG_MOD_REPLACE, Dn, Message,
@@ -61,7 +60,10 @@ class RestoredObjectAttributesBaseTestCase(samba.tests.TestCase):
     def GUID_string(self, guid):
         return get_string(self.samdb.schema_format_value("objectGUID", guid))
 
-    def search_guid(self, guid, attrs=["*"]):
+    def search_guid(self, guid, attrs=None):
+        if attrs is None:
+            attrs = ["*"]
+
         res = self.samdb.search(base="<GUID=%s>" % self.GUID_string(guid),
                                 scope=SCOPE_BASE, attrs=attrs,
                                 controls=["show_deleted:1"])
@@ -664,7 +666,7 @@ class RestoreUserPwdObjectTestCase(RestoredObjectAttributesBaseTestCase):
             (DRSUAPI_ATTID_supplementalCredentials, 2),
             (DRSUAPI_ATTID_objectSid, 1),
             (DRSUAPI_ATTID_accountExpires, 2),
-            (DRSUAPI_ATTID_lmPwdHistory, 2),
+            (DRSUAPI_ATTID_lmPwdHistory, None),
             (DRSUAPI_ATTID_sAMAccountName, 1),
             (DRSUAPI_ATTID_sAMAccountType, 2),
             (DRSUAPI_ATTID_lastKnownParent, 1),
@@ -726,7 +728,7 @@ class RestoreUserPwdObjectTestCase(RestoredObjectAttributesBaseTestCase):
             (DRSUAPI_ATTID_objectSid, 1),
             (DRSUAPI_ATTID_adminCount, 1),
             (DRSUAPI_ATTID_accountExpires, 3),
-            (DRSUAPI_ATTID_lmPwdHistory, 3),
+            (DRSUAPI_ATTID_lmPwdHistory, None),
             (DRSUAPI_ATTID_sAMAccountName, 1),
             (DRSUAPI_ATTID_sAMAccountType, 3),
             (DRSUAPI_ATTID_lastKnownParent, 1),
