@@ -59,10 +59,10 @@ int main (int argc, char **argv)
 
 	keytab_name = argv[1];
 
-	initialize_krb5_error_table();
-
-	ret = krb5_init_context(&context);
+	ret = smb_krb5_init_context_common(&context);
 	if (ret) {
+		DBG_ERR("kerberos init context failed (%s)\n",
+			error_message(ret));
 		smb_krb5_err(mem_ctx, context, 1, ret, "krb5_context");
 	}
 
@@ -96,11 +96,11 @@ int main (int argc, char **argv)
 						 enctype,
 						 &enctype_str);
 		if (ret) {
-			smb_krb5_err(mem_ctx, context, 1, ret, "krb5_enctype_to_string");
+			printf("%s (%d)\n", principal, (int)enctype);
+		} else {
+			printf("%s (%s)\n", principal, enctype_str);
 		}
 
-		printf("%s (%s)\n", principal, enctype_str);
-
 		TALLOC_FREE(principal);
 		SAFE_FREE(enctype_str);
 		smb_krb5_kt_free_entry(context, &entry);
