@@ -40,6 +40,7 @@
 #include "lib/messaging/messages_dgm_ref.h"
 #include "../source3/lib/messages_util.h"
 #include <tdb.h>
+#include "lib/util/idtree.h"
 
 /* change the message version with any incompatible changes in the protocol */
 #define IMESSAGING_VERSION 1
@@ -315,19 +316,21 @@ NTSTATUS imessaging_register_tmp(struct imessaging_context *msg, void *private_d
 }
 
 /*
-  De-register the function for a particular message type.
+  De-register the function for a particular message type. Return the number of
+  functions deregistered.
 */
-void imessaging_deregister(struct imessaging_context *msg, uint32_t msg_type, void *private_data)
+size_t imessaging_deregister(struct imessaging_context *msg, uint32_t msg_type, void *private_data)
 {
 	struct dispatch_fn *d, *next;
+	size_t removed = 0;
 
 	if (msg_type >= msg->num_types) {
 		d = (struct dispatch_fn *)idr_find(msg->dispatch_tree,
 						   msg_type);
-		if (!d) return;
+		if (!d) return 0;
 		idr_remove(msg->dispatch_tree, msg_type);
 		talloc_free(d);
-		return;
+		return 1;
 	}
 
 	for (d = msg->dispatch[msg_type]; d; d = next) {
@@ -335,8 +338,11 @@ void imessaging_deregister(struct imessaging_context *msg, uint32_t msg_type, vo
 		if (d->private_data == private_data) {
 			DLIST_REMOVE(msg->dispatch[msg_type], d);
 			talloc_free(d);
+			++removed;
 		}
 	}
+
+	return removed;
 }
 
 /*
@@ -1404,6 +1410,38 @@ static bool irpc_bh_ref_alloc(struct dcerpc_binding_handle *h)
 	return true;
 }
 
+static void irpc_bh_do_ndr_print(struct dcerpc_binding_handle *h,
+				 int ndr_flags,
+				 const void *_struct_ptr,
+				 const struct ndr_interface_call *call)
+{
+	void *struct_ptr = discard_const(_struct_ptr);
+	bool print_in = false;
+	bool print_out = false;
+
+	if (DEBUGLEVEL >= 11) {
+		print_in = true;
+		print_out = true;
+	}
+
+	if (ndr_flags & NDR_IN) {
+		if (print_in) {
+			ndr_print_function_debug(call->ndr_print,
+						 call->name,
+						 ndr_flags,
+						 struct_ptr);
+		}
+	}
+	if (ndr_flags & NDR_OUT) {
+		if (print_out) {
+			ndr_print_function_debug(call->ndr_print,
+						 call->name,
+						 ndr_flags,
+						 struct_ptr);
+		}
+	}
+}
+
 static const struct dcerpc_binding_handle_ops irpc_bh_ops = {
 	.name			= "wbint",
 	.is_connected		= irpc_bh_is_connected,
@@ -1414,6 +1452,7 @@ static const struct dcerpc_binding_handle_ops irpc_bh_ops = {
 	.disconnect_recv	= irpc_bh_disconnect_recv,
 
 	.ref_alloc		= irpc_bh_ref_alloc,
+	.do_ndr_print		= irpc_bh_do_ndr_print,
 };
 
 /* initialise a irpc binding handle */
